@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -31,51 +31,44 @@
 #include <utility>
 #include <cmath>
 #include <limits>
-#include <guisim/GUINet.h>
+#include <foreign/rtree/SUMORTree.h>
+#include <gui/GUIApplicationWindow.h>
+#include <gui/GUIGlobals.h>
 #include <guisim/GUIEdge.h>
 #include <guisim/GUILane.h>
+#include <guisim/GUINet.h>
 #include <guisim/GUIVehicle.h>
 #include <guisim/GUIVehicleControl.h>
-#include <microsim/MSGlobals.h>
+#include <mesogui/GUIMEVehicle.h>
+#include <mesogui/GUIMEVehicleControl.h>
 #include <microsim/MSEdge.h>
-#include <microsim/MSLane.h>
+#include <microsim/MSGlobals.h>
 #include <microsim/MSJunctionControl.h>
+#include <microsim/MSLane.h>
 #include <microsim/MSStoppingPlace.h>
-#include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <microsim/traffic_lights/MSSimpleTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <utils/common/RGBColor.h>
-#include <utils/geom/PositionVector.h>
-#include <utils/shapes/ShapeContainer.h>
-#include "GUISUMOViewParent.h"
-#include "GUIViewTraffic.h"
-#include <utils/gui/windows/GUISUMOAbstractView.h>
-#include <utils/gui/windows/GUIPerspectiveChanger.h>
-#include <utils/gui/windows/GUIAppEnum.h>
+#include <utils/foxtools/MFXButtonTooltip.h>
 #include <utils/foxtools/MFXCheckableButton.h>
-#include <utils/gui/images/GUIIconSubSys.h>
-#include <gui/GUIApplicationWindow.h>
-#include <utils/gui/windows/GUIDialog_ViewSettings.h>
-#include <utils/gui/settings/GUICompleteSchemeStorage.h>
 #include <utils/foxtools/MFXImageHelper.h>
-#include <utils/gui/globjects/GUIGlObjectStorage.h>
-#include <foreign/rtree/SUMORTree.h>
+#include <utils/geom/PositionVector.h>
 #include <utils/gui/div/GLHelper.h>
+#include <utils/gui/div/GUIDesigns.h>
 #include <utils/gui/div/GUIGlobalSelection.h>
 #include <utils/gui/globjects/GLIncludes.h>
+#include <utils/gui/globjects/GUIGlObjectStorage.h>
+#include <utils/gui/globjects/GUIShapeContainer.h>
+#include <utils/gui/images/GUIIconSubSys.h>
+#include <utils/gui/settings/GUICompleteSchemeStorage.h>
+#include <utils/gui/windows/GUIAppEnum.h>
+#include <utils/gui/windows/GUIDialog_ViewSettings.h>
+#include <utils/gui/windows/GUIPerspectiveChanger.h>
+#include <utils/gui/windows/GUISUMOAbstractView.h>
+#include <utils/shapes/ShapeContainer.h>
 
-/* -------------------------------------------------------------------------
- * GUIViewTraffic - FOX callback mapping
- * ----------------------------------------------------------------------- */
-FXDEFMAP(GUIViewTraffic) GUIViewTrafficMap[] = {
-    FXMAPFUNC(SEL_COMMAND, MID_CLOSE_LANE, GUIViewTraffic::onCmdCloseLane),
-    FXMAPFUNC(SEL_COMMAND, MID_CLOSE_EDGE, GUIViewTraffic::onCmdCloseEdge),
-    FXMAPFUNC(SEL_COMMAND, MID_ADD_REROUTER, GUIViewTraffic::onCmdAddRerouter),
-    FXMAPFUNC(SEL_COMMAND, MID_REACHABILITY, GUIViewTraffic::onCmdShowReachability),
-};
-
-
-FXIMPLEMENT_ABSTRACT(GUIViewTraffic, GUISUMOAbstractView, GUIViewTrafficMap, ARRAYNUMBER(GUIViewTrafficMap))
-
+#include "GUISUMOViewParent.h"
+#include "GUIViewTraffic.h"
 
 // ===========================================================================
 // member method definitions
@@ -106,58 +99,57 @@ GUIViewTraffic::buildViewToolBars(GUIGlChildWindow* v) {
     {
         const std::vector<std::string>& names = gSchemeStorage.getNames();
         for (std::vector<std::string>::const_iterator i = names.begin(); i != names.end(); ++i) {
-            v->getColoringSchemesCombo()->appendItem(i->c_str());
+            v->getColoringSchemesCombo()->appendIconItem(i->c_str());
             if ((*i) == myVisualizationSettings->name) {
                 v->getColoringSchemesCombo()->setCurrentItem(v->getColoringSchemesCombo()->getNumItems() - 1);
             }
         }
-        v->getColoringSchemesCombo()->setNumVisible(MAX2(5, (int)names.size() + 1));
     }
     // for junctions
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Junctions\tLocate a junction within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEJUNCTION), v, MID_LOCATEJUNCTION,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Junctions") + std::string("\t") + TL("Locate a junction within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEJUNCTION), v, MID_HOTKEY_SHIFT_J_LOCATEJUNCTION,
+                         GUIDesignButtonPopup);
     // for edges
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Edges\tLocate an edge within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEEDGE), v, MID_LOCATEEDGE,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Edges") + std::string("\t") + TL("Locate an edge within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEEDGE), v, MID_HOTKEY_SHIFT_E_LOCATEEDGE,
+                         GUIDesignButtonPopup);
     // for vehicles
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Vehicles\tLocate a vehicle within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEVEHICLE), v, MID_LOCATEVEHICLE,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Vehicles") + std::string("\t") + TL("Locate a vehicle within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEVEHICLE), v, MID_HOTKEY_SHIFT_V_LOCATEVEHICLE,
+                         GUIDesignButtonPopup);
     // for persons
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Persons\tLocate a person within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEPERSON), v, MID_LOCATEPERSON,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Persons") + std::string("\t") + TL("Locate a person within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEPERSON), v, MID_HOTKEY_SHIFT_P_LOCATEPERSON,
+                         GUIDesignButtonPopup);
     // for containers
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Container\tLocate a container within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATECONTAINER), v, MID_LOCATECONTAINER,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Container") + std::string("\t") + TL("Locate a container within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATECONTAINER), v, MID_HOTKEY_SHIFT_C_LOCATECONTAINER,
+                         GUIDesignButtonPopup);
     // for tls
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate TLS\tLocate a tls within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATETLS), v, MID_LOCATETLS,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate TLS") + std::string("\t") + TL("Locate a tls within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATETLS), v, MID_HOTKEY_SHIFT_T_LOCATETLS,
+                         GUIDesignButtonPopup);
     // for additional stuff
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Additional\tLocate an additional structure within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEADD), v, MID_LOCATEADD,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Additional") + std::string("\t") + TL("Locate an additional structure within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEADD), v, MID_HOTKEY_SHIFT_A_LOCATEADDITIONAL,
+                         GUIDesignButtonPopup);
     // for pois
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate PoI\tLocate a PoI within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEPOI), v, MID_LOCATEPOI,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate PoI") + std::string("\t") + TL("Locate a PoI within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEPOI), v, MID_HOTKEY_SHIFT_O_LOCATEPOI,
+                         GUIDesignButtonPopup);
     // for polygons
-    new FXButton(v->getLocatorPopup(),
-                 "\tLocate Polygon\tLocate a Polygon within the network.",
-                 GUIIconSubSys::getIcon(GUIIcon::LOCATEPOLY), v, MID_LOCATEPOLY,
-                 ICON_ABOVE_TEXT | FRAME_THICK | FRAME_RAISED);
+    new MFXButtonTooltip(v->getLocatorPopup(), myApp->getStaticTooltipMenu(),
+                         (std::string("\t") + TL("Locate Polygon") + std::string("\t") + TL("Locate a Polygon within the network.")).c_str(),
+                         GUIIconSubSys::getIcon(GUIIcon::LOCATEPOLY), v, MID_HOTKEY_SHIFT_L_LOCATEPOLY,
+                         GUIDesignButtonPopup);
 }
 
 
@@ -166,9 +158,9 @@ GUIViewTraffic::setColorScheme(const std::string& name) {
     if (!gSchemeStorage.contains(name)) {
         return false;
     }
-    if (myVisualizationChanger != nullptr) {
-        if (myVisualizationChanger->getCurrentScheme() != name) {
-            myVisualizationChanger->setCurrentScheme(name);
+    if (myGUIDialogViewSettings != nullptr) {
+        if (myGUIDialogViewSettings->getCurrentScheme() != name) {
+            myGUIDialogViewSettings->setCurrentScheme(name);
         }
     }
     myVisualizationSettings = &gSchemeStorage.get(name.c_str());
@@ -180,11 +172,12 @@ GUIViewTraffic::setColorScheme(const std::string& name) {
 
 void
 GUIViewTraffic::buildColorRainbow(const GUIVisualizationSettings& s, GUIColorScheme& scheme, int active, GUIGlObjectType objectType,
-                                  bool hide, double hideThreshold) {
+                                  const GUIVisualizationRainbowSettings& rs) {
     assert(!scheme.isFixed());
     double minValue = std::numeric_limits<double>::infinity();
     double maxValue = -std::numeric_limits<double>::infinity();
     // retrieve range
+    bool hasMissingData = false;
     if (objectType == GLO_LANE) {
         // XXX (see #3409) multi-colors are not currently handled. this is a quick hack
         if (active == 22) {
@@ -197,6 +190,7 @@ GUIViewTraffic::buildColorRainbow(const GUIVisualizationSettings& s, GUIColorSch
             if (MSGlobals::gUseMesoSim) {
                 const double val = static_cast<GUIEdge*>(*it)->getColorValue(s, active);
                 if (val == s.MISSING_DATA) {
+                    hasMissingData = true;
                     continue;
                 }
                 minValue = MIN2(minValue, val);
@@ -206,6 +200,7 @@ GUIViewTraffic::buildColorRainbow(const GUIVisualizationSettings& s, GUIColorSch
                 for (std::vector<MSLane*>::const_iterator it_l = lanes.begin(); it_l != lanes.end(); it_l++) {
                     const double val = static_cast<GUILane*>(*it_l)->getColorValue(s, active);
                     if (val == s.MISSING_DATA) {
+                        hasMissingData = true;
                         continue;
                     }
                     minValue = MIN2(minValue, val);
@@ -213,6 +208,23 @@ GUIViewTraffic::buildColorRainbow(const GUIVisualizationSettings& s, GUIColorSch
                 }
             }
         }
+    } else if (objectType == GLO_VEHICLE) {
+        MSVehicleControl& c = MSNet::getInstance()->getVehicleControl();
+        for (MSVehicleControl::constVehIt it_v = c.loadedVehBegin(); it_v != c.loadedVehEnd(); ++it_v) {
+            const GUIGlObject* veh;
+            if (MSGlobals::gUseMesoSim) {
+                veh = static_cast<const GUIMEVehicle*>(it_v->second);
+            } else {
+                veh = static_cast<const GUIVehicle*>(it_v->second);
+            }
+            const double val = veh->getColorValue(s, active);
+            if (val == s.MISSING_DATA) {
+                hasMissingData = true;
+                continue;
+            }
+            minValue = MIN2(minValue, val);
+            maxValue = MAX2(maxValue, val);
+        }
     } else if (objectType == GLO_JUNCTION) {
         if (active == 3) {
             std::set<const MSJunction*> junctions;
@@ -238,32 +250,12 @@ GUIViewTraffic::buildColorRainbow(const GUIVisualizationSettings& s, GUIColorSch
         int step = MAX2(1, 360 / (int)codes.size());
         int hue = 0;
         for (SVCPermissions p : codes) {
-            scheme.addColor(RGBColor::fromHSV(hue, 1, 1), p);
+            scheme.addColor(RGBColor::fromHSV(hue, 1, 1), (double)p);
             hue = (hue + step) % 360;
         }
         return;
     }
-
-    if (minValue != std::numeric_limits<double>::infinity()) {
-        scheme.clear();
-        // add new thresholds
-        if (scheme.getName() == GUIVisualizationSettings::SCHEME_NAME_EDGEDATA_NUMERICAL) {
-            scheme.addColor(RGBColor(204, 204, 204), std::numeric_limits<double>::max(), "missing data");
-        }
-        if (hide) {
-            const double rawRange = maxValue - minValue;
-            minValue = MAX2(hideThreshold + MIN2(1.0, rawRange / 100.0), minValue);
-            scheme.addColor(RGBColor(204, 204, 204), hideThreshold);
-        }
-        double range = maxValue - minValue;
-        scheme.addColor(RGBColor::RED, (minValue));
-        scheme.addColor(RGBColor::ORANGE, (minValue + range * 1 / 6.0));
-        scheme.addColor(RGBColor::YELLOW, (minValue + range * 2 / 6.0));
-        scheme.addColor(RGBColor::GREEN, (minValue + range * 3 / 6.0));
-        scheme.addColor(RGBColor::CYAN, (minValue + range * 4 / 6.0));
-        scheme.addColor(RGBColor::BLUE, (minValue + range * 5 / 6.0));
-        scheme.addColor(RGBColor::MAGENTA, (maxValue));
-    }
+    buildMinMaxRainbow(s, scheme, rs, minValue, maxValue, hasMissingData);
 }
 
 
@@ -276,6 +268,23 @@ GUIViewTraffic::getEdgeDataAttrs() const {
 }
 
 
+std::vector<std::string>
+GUIViewTraffic::getMeanDataIDs() const {
+    if (GUINet::getGUIInstance() != nullptr) {
+        return GUINet::getGUIInstance()->getMeanDataIDs();
+    }
+    return std::vector<std::string>();
+}
+
+std::vector<std::string>
+GUIViewTraffic::getMeanDataAttrs(const std::string& meanDataID) const {
+    if (GUINet::getGUIInstance() != nullptr) {
+        return GUINet::getGUIInstance()->getMeanDataAttrs(meanDataID);
+    }
+    return std::vector<std::string>();
+}
+
+
 std::vector<std::string>
 GUIViewTraffic::getEdgeLaneParamKeys(bool edgeKeys) const {
     std::set<std::string> keys;
@@ -299,7 +308,12 @@ GUIViewTraffic::getEdgeLaneParamKeys(bool edgeKeys) const {
 std::vector<std::string>
 GUIViewTraffic::getVehicleParamKeys(bool /*vTypeKeys*/) const {
     std::set<std::string> keys;
-    GUIVehicleControl* vc = GUINet::getGUIInstance()->getGUIVehicleControl();
+    MSVehicleControl* vc = nullptr;
+    if (MSGlobals::gUseMesoSim) {
+        vc = GUINet::getGUIInstance()->getGUIMEVehicleControl();
+    } else {
+        vc = GUINet::getGUIInstance()->getGUIVehicleControl();
+    }
     vc->secureVehicles();
     for (auto vehIt = vc->loadedVehBegin(); vehIt != vc->loadedVehEnd(); ++vehIt) {
         for (auto kv : vehIt->second->getParameter().getParametersMap()) {
@@ -322,14 +336,20 @@ GUIViewTraffic::getPOIParamKeys() const {
     return std::vector<std::string>(keys.begin(), keys.end());
 }
 
+
+void
+GUIViewTraffic::centerTo(GUIGlID id, bool applyZoom, double zoomDist) {
+    GUIGlobals::gSecondaryShape = myVisualizationSettings->secondaryShape;
+    GUISUMOAbstractView::centerTo(id, applyZoom, zoomDist);
+    GUIGlobals::gSecondaryShape = false;
+}
+
 int
 GUIViewTraffic::doPaintGL(int mode, const Boundary& bound) {
-    // (uncomment the next line to check select mode)
-    //myVisualizationSettings->drawForPositionSelection = true;
     // init view settings
     glRenderMode(mode);
     glMatrixMode(GL_MODELVIEW);
-    glPushMatrix();
+    GLHelper::pushMatrix();
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_ALPHA_TEST);
     glEnable(GL_BLEND);
@@ -338,19 +358,19 @@ GUIViewTraffic::doPaintGL(int mode, const Boundary& bound) {
 
     // draw decals (if not in grabbing mode)
     drawDecals();
+    myVisualizationSettings->scale = m2p(SUMO_const_laneWidth);
     if (myVisualizationSettings->showGrid) {
         paintGLGrid();
     }
-
-
     glLineWidth(1);
     glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     const float minB[2] = { (float)bound.xmin(), (float)bound.ymin() };
     const float maxB[2] = { (float)bound.xmax(), (float)bound.ymax() };
-    myVisualizationSettings->scale = myVisualizationSettings->drawForPositionSelection ? myVisualizationSettings->scale : m2p(SUMO_const_laneWidth);
     glEnable(GL_POLYGON_OFFSET_FILL);
     glEnable(GL_POLYGON_OFFSET_LINE);
-    int hits2 = myGrid->Search(minB, maxB, *myVisualizationSettings);
+    const SUMORTree& grid = GUINet::getGUIInstance()->getVisualisationSpeedUp(myVisualizationSettings->secondaryShape);
+    int hits2 = grid.Search(minB, maxB, *myVisualizationSettings);
+    GUIGlobals::gSecondaryShape = myVisualizationSettings->secondaryShape;
     // Draw additional objects
     if (myAdditionallyDrawn.size() > 0) {
         glTranslated(0, 0, -.01);
@@ -361,7 +381,7 @@ GUIViewTraffic::doPaintGL(int mode, const Boundary& bound) {
         GUINet::getGUIInstance()->unlock();
         glTranslated(0, 0, .01);
     }
-    glPopMatrix();
+    GLHelper::popMatrix();
     /*
     // draw legends
     glMatrixMode(GL_MODELVIEW);
@@ -378,6 +398,13 @@ GUIViewTraffic::doPaintGL(int mode, const Boundary& bound) {
 void
 GUIViewTraffic::startTrack(int id) {
     myTrackedID = id;
+    GUIGlObject* o = GUIGlObjectStorage::gIDStorage.getObjectBlocking(id);
+    if (o != nullptr) {
+        GUIBaseVehicle* v = dynamic_cast<GUIBaseVehicle*>(o);
+        if (v != nullptr) {
+            v->addActiveAddVisualisation(this, GUIBaseVehicle::VO_TRACK);
+        }
+    }
 }
 
 
@@ -397,9 +424,11 @@ void
 GUIViewTraffic::onGamingClick(Position pos) {
     if (myTLSGame) {
         MSTLLogicControl& tlsControl = MSNet::getInstance()->getTLSControl();
-        const MSTrafficLightLogic* minTll = nullptr;
+        MSTrafficLightLogic* minTll = nullptr;
+        GUIGlObject* minRR = nullptr;
         double minDist = std::numeric_limits<double>::infinity();
-        for (const MSTrafficLightLogic* const tll : tlsControl.getAllLogics()) {
+        double minDistRR = std::numeric_limits<double>::infinity();
+        for (MSTrafficLightLogic* const tll : tlsControl.getAllLogics()) {
             if (tlsControl.isActive(tll) && tll->getProgramID() != "off") {
                 // get the links
                 const MSTrafficLightLogic::LaneVector& lanes = tll->getLanesAt(0);
@@ -412,30 +441,88 @@ GUIViewTraffic::onGamingClick(Position pos) {
                 }
             }
         }
-        if (minTll != nullptr) {
-            const MSTLLogicControl::TLSLogicVariants& vars = tlsControl.get(minTll->getID());
-            const std::vector<MSTrafficLightLogic*> logics = vars.getAllLogics();
-            if (logics.size() > 1) {
-                MSSimpleTrafficLightLogic* l = (MSSimpleTrafficLightLogic*) logics[0];
-                for (int i = 0; i < (int)logics.size() - 1; ++i) {
-                    if (minTll->getProgramID() == logics[i]->getProgramID()) {
-                        l = (MSSimpleTrafficLightLogic*) logics[i + 1];
-                        tlsControl.switchTo(minTll->getID(), l->getProgramID());
+        if (makeCurrent()) {
+            for (GUIGlObject* o : getGUIGlObjectsAtPosition(getPositionInformation(), MIN2(minDist, 20.0))) {
+                if (o->getType() == GLO_REROUTER_EDGE) {
+                    const double dist = o->getCenter().distanceTo2D(pos);
+                    if (dist < minDistRR) {
+                        minDistRR = dist;
+                        minRR = o;
                     }
                 }
-                if (l == logics[0]) {
-                    tlsControl.switchTo(minTll->getID(), l->getProgramID());
-                }
-                l->changeStepAndDuration(tlsControl, MSNet::getInstance()->getCurrentTimeStep(), 0, l->getPhase(0).duration);
-                update();
             }
+            makeNonCurrent();
+        }
+        if (minDistRR < minDist && minRR != nullptr) {
+            minRR->onLeftBtnPress(nullptr);
+            update();
+            return;
+        }
+
+        if (minTll != nullptr) {
+            if (minTll->getPhaseNumber() == 0) {
+                // MSRailSignal
+                return;
+            }
+            const int ci = minTll->getCurrentPhaseIndex();
+            const int n = minTll->getPhaseNumber();
+            int greenCount = 0;
+            for (auto& phase : minTll->getPhases()) {
+                if (phase->isGreenPhase()) {
+                    greenCount++;
+                }
+            }
+            int nextPhase = (ci + 1) % n;
+            SUMOTime nextDuration = 0;
+            if (minTll->getCurrentPhaseDef().isGreenPhase() || (greenCount == 1 && minTll->getCurrentPhaseDef().isAllRedPhase())) {
+                nextDuration = minTll->getPhase(nextPhase).duration;
+            } else {
+                // we are in transition to a green phase
+                // -> skip forward to the transition into the next green phase
+                // but ensure that the total transition time is maintained
+                // taking into account how much time was already spent
+                SUMOTime spentTransition = minTll->getSpentDuration();
+                // the transition may consist of more than one phase so we
+                // search backwards until the prior green phase
+                for (int i = ci - 1; i != ci; i--) {
+                    if (i < 0) {
+                        i = n - 1;
+                    }
+                    if (minTll->getPhase(i).isGreenPhase()) {
+                        break;
+                    }
+                    spentTransition += minTll->getPhase(i).duration;
+                }
+                // now we skip past the next greenphase
+                int numGreen = 0;
+                int i = nextPhase;
+                for (; numGreen < 2; i = (i + 1) % n) {
+                    if (minTll->getPhase(i).isGreenPhase()) {
+                        numGreen++;
+                        continue;
+                    }
+                    // transition after the next green
+                    if (numGreen == 1) {
+                        SUMOTime dur = minTll->getPhase(i).duration;
+                        if (dur <= spentTransition) {
+                            spentTransition -= dur;
+                        } else {
+                            nextPhase = i;
+                            nextDuration = dur - spentTransition;
+                            break;
+                        }
+                    }
+                }
+            }
+            minTll->changeStepAndDuration(tlsControl, MSNet::getInstance()->getCurrentTimeStep(), nextPhase, nextDuration);
+            update();
         }
     } else {
         // DRT game
         if (MSGlobals::gUseMesoSim) {
             return;
         }
-        const std::set<GUIGlID>& sel = gSelected.getSelected(GLO_VEHICLE);
+        const auto& sel = gSelected.getSelected(GLO_VEHICLE);
         if (sel.size() == 0) {
             // find closest pt vehicle
             double minDist = std::numeric_limits<double>::infinity();
@@ -488,7 +575,7 @@ GUIViewTraffic::onGamingClick(Position pos) {
 
 void
 GUIViewTraffic::onGamingRightClick(Position /*pos*/) {
-    const std::set<GUIGlID>& sel = gSelected.getSelected(GLO_VEHICLE);
+    const auto& sel = gSelected.getSelected(GLO_VEHICLE);
     if (sel.size() > 0) {
         GUIGlID id = *sel.begin();
         GUIVehicle* veh = dynamic_cast<GUIVehicle*>(GUIGlObjectStorage::gIDStorage.getObjectBlocking(id));
@@ -507,21 +594,6 @@ GUIViewTraffic::getCurrentTimeStep() const {
 }
 
 
-GUILane*
-GUIViewTraffic::getLaneUnderCursor() {
-    if (makeCurrent()) {
-        int id = getObjectUnderCursor();
-        if (id != 0) {
-            GUIGlObject* o = GUIGlObjectStorage::gIDStorage.getObjectBlocking(id);
-            if (o != nullptr) {
-                return dynamic_cast<GUILane*>(o);
-            }
-        }
-        makeNonCurrent();
-    }
-    return nullptr;
-}
-
 long
 GUIViewTraffic::onCmdCloseLane(FXObject*, FXSelector, void*) {
     GUILane* lane = getLaneUnderCursor();
@@ -559,22 +631,21 @@ GUIViewTraffic::onCmdAddRerouter(FXObject*, FXSelector, void*) {
 
 
 long
-GUIViewTraffic::onCmdShowReachability(FXObject* menu, FXSelector, void*) {
-    GUILane* lane = getLaneUnderCursor();
+GUIViewTraffic::showLaneReachability(GUILane* lane, FXObject* menu, FXSelector) {
     if (lane != nullptr) {
         // reset
-        const double UNREACHABLE = -1;
+        const double UNREACHED = INVALID_DOUBLE;
         gSelected.clear();
         for (const MSEdge* const e : MSEdge::getAllEdges()) {
             for (MSLane* const l : e->getLanes()) {
                 GUILane* gLane = dynamic_cast<GUILane*>(l);
-                gLane->setReachability(UNREACHABLE);
+                gLane->setReachability(UNREACHED);
             }
         }
         // prepare
         FXMenuCommand* mc = dynamic_cast<FXMenuCommand*>(menu);
         const SUMOVehicleClass svc = SumoVehicleClassStrings.get(mc->getText().text());
-        const double defaultMaxSpeed = SUMOVTypeParameter::VClassDefaultValues(svc).maxSpeed;
+        const double defaultMaxSpeed = SUMOVTypeParameter::VClassDefaultValues(svc).desiredMaxSpeed;
         // find reachable
         std::map<MSEdge*, double> reachableEdges;
         reachableEdges[&lane->getEdge()] = 0;
@@ -587,11 +658,13 @@ GUIViewTraffic::onCmdShowReachability(FXObject* menu, FXSelector, void*) {
             for (MSLane* const l : e->getLanes()) {
                 if (l->allowsVehicleClass(svc)) {
                     GUILane* gLane = dynamic_cast<GUILane*>(l);
-                    gSelected.select(gLane->getGlID());
+                    gSelected.select(gLane->getGlID(), false);
                     gLane->setReachability(traveltime);
                 }
             }
-            traveltime += e->getLength() / MIN2(e->getSpeedLimit(), defaultMaxSpeed);
+            const double dt = e->getLength() / MIN2(e->getSpeedLimit(), defaultMaxSpeed);
+            // ensure algorithm termination
+            traveltime += MAX2(dt, NUMERICAL_EPS);
             for (MSEdge* const nextEdge : e->getSuccessors(svc)) {
                 if (reachableEdges.count(nextEdge) == 0 ||
                         // revisit edge via faster path
@@ -600,7 +673,44 @@ GUIViewTraffic::onCmdShowReachability(FXObject* menu, FXSelector, void*) {
                     check.push_back(nextEdge);
                 }
             }
+            if (svc == SVC_PEDESTRIAN) {
+                // can also walk backwards
+                for (MSEdge* const prevEdge : e->getPredecessors()) {
+                    if (prevEdge->allowedLanes(*e, svc) != nullptr &&
+                            (reachableEdges.count(prevEdge) == 0 ||
+                             // revisit edge via faster path
+                             reachableEdges[prevEdge] > traveltime)) {
+                        reachableEdges[prevEdge] = traveltime;
+                        check.push_back(prevEdge);
+                    }
+                }
+                // and connect to arbitrary incoming if there are no walkingareas
+                if (!MSNet::getInstance()->hasPedestrianNetwork()) {
+                    for (const MSEdge* const in_const : e->getToJunction()->getIncoming()) {
+                        MSEdge* in = const_cast<MSEdge*>(in_const);
+                        if ((in->getPermissions() & svc) == svc &&
+                                (reachableEdges.count(in) == 0 ||
+                                 // revisit edge via faster path
+                                 reachableEdges[in] > traveltime)) {
+                            reachableEdges[in] = traveltime;
+                            check.push_back(in);
+                        }
+                    }
+                }
+            }
         }
+        gSelected.notifyChanged();
+    }
+    return 1;
+}
+
+
+long
+GUIViewTraffic::onCmdShowReachability(FXObject* menu, FXSelector selector, void*) {
+    GUILane* lane = getLaneUnderCursor();
+    if (lane != nullptr) {
+        // reset
+        showLaneReachability(lane, menu, selector);
         // switch to 'color by selection' unless coloring 'by reachability'
         if (myVisualizationSettings->laneColorer.getActive() != 36) {
             myVisualizationSettings->laneColorer.setActive(1);
@@ -611,6 +721,22 @@ GUIViewTraffic::onCmdShowReachability(FXObject* menu, FXSelector, void*) {
 }
 
 
+GUILane*
+GUIViewTraffic::getLaneUnderCursor() {
+    if (makeCurrent()) {
+        int id = getObjectUnderCursor();
+        if (id != 0) {
+            GUIGlObject* o = GUIGlObjectStorage::gIDStorage.getObjectBlocking(id);
+            if (o != nullptr) {
+                return dynamic_cast<GUILane*>(o);
+            }
+        }
+        makeNonCurrent();
+    }
+    return nullptr;
+}
+
+
 long
 GUIViewTraffic::onDoubleClicked(FXObject*, FXSelector, void*) {
     // leave fullscreen mode
@@ -666,4 +792,27 @@ GUIViewTraffic::retrieveBreakpoints() const {
 }
 
 
+void
+GUIViewTraffic::drawPedestrianNetwork(const GUIVisualizationSettings& s) const {
+    GUIShapeContainer& shapeContainer = dynamic_cast<GUIShapeContainer&>(GUINet::getInstance()->getShapeContainer());
+    if (s.showPedestrianNetwork) {
+        shapeContainer.removeInactivePolygonTypes(std::set<std::string> {"jupedsim.pedestrian_network"});
+    } else {
+        shapeContainer.addInactivePolygonTypes(std::set<std::string> {"jupedsim.pedestrian_network"});
+    }
+    update();
+}
+
+
+void
+GUIViewTraffic::changePedestrianNetworkColor(const GUIVisualizationSettings& s) const {
+    GUIShapeContainer& shapeContainer = dynamic_cast<GUIShapeContainer&>(GUINet::getInstance()->getShapeContainer());
+    for (auto polygonwithID : shapeContainer.getPolygons()) {
+        if (polygonwithID.second->getShapeType() == "jupedsim.pedestrian_network") {
+            polygonwithID.second->setShapeColor(s.pedestrianNetworkColor);
+        }
+    }
+    update();
+}
+
 /****************************************************************************/
