@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -27,6 +27,8 @@
 #include "MSEdge.h"
 #include "MSNet.h"
 #include "MSRouteHandler.h"
+#include "MSEventControl.h"
+#include "MSStop.h"
 #include <microsim/devices/MSVehicleDevice.h>
 #include <microsim/devices/MSDevice_Tripinfo.h>
 #include <utils/common/FileHelpers.h>
@@ -36,9 +38,6 @@
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/router/IntermodalRouter.h>
-#ifdef HAVE_FOX
-#include <utils/foxtools/FXConditionalLock.h>
-#endif
 
 
 // ===========================================================================
@@ -50,30 +49,29 @@ MSVehicleControl::MSVehicleControl() :
     myEndedVehNo(0),
     myDiscarded(0),
     myCollisions(0),
+    myTeleportsCollision(0),
     myTeleportsJam(0),
     myTeleportsYield(0),
     myTeleportsWrongLane(0),
     myEmergencyStops(0),
+    myEmergencyBrakingCount(0),
     myStoppedVehicles(0),
     myTotalDepartureDelay(0),
     myTotalTravelTime(0),
-    myDefaultVTypeMayBeDeleted(true),
-    myDefaultPedTypeMayBeDeleted(true),
-    myDefaultContainerTypeMayBeDeleted(true),
-    myDefaultBikeTypeMayBeDeleted(true),
-    myDefaultTaxiTypeMayBeDeleted(true),
     myWaitingForTransportable(0),
     myMaxSpeedFactor(1),
     myMinDeceleration(SUMOVTypeParameter::getDefaultDecel(SVC_IGNORING)),
+    myMinDecelerationRail(SUMOVTypeParameter::getDefaultDecel(SVC_RAIL)),
     myPendingRemovals(MSGlobals::gNumSimThreads > 1) {
 
     initDefaultTypes();
     myScale = OptionsCont::getOptions().getFloat("scale");
+    myKeepTime = string2time(OptionsCont::getOptions().getString("keep-after-arrival"));
 }
 
 
 MSVehicleControl::~MSVehicleControl() {
-    clearState();
+    clearState(false);
 }
 
 
@@ -94,32 +92,38 @@ MSVehicleControl::initDefaultTypes() {
     defTaxiType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
     myVTypeDict[DEFAULT_TAXITYPE_ID] = MSVehicleType::build(defTaxiType);
 
-    SUMOVTypeParameter defContainerType(DEFAULT_CONTAINERTYPE_ID, SVC_IGNORING);
-    // ISO Container TEU (cannot set this based on vClass)
-    defContainerType.length = 6.1;
-    defContainerType.width = 2.4;
-    defContainerType.height = 2.6;
+    SUMOVTypeParameter defRailType(DEFAULT_RAILTYPE_ID, SVC_RAIL);
+    defRailType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
+    myVTypeDict[DEFAULT_RAILTYPE_ID] = MSVehicleType::build(defRailType);
+
+    SUMOVTypeParameter defContainerType(DEFAULT_CONTAINERTYPE_ID, SVC_CONTAINER);
     defContainerType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
     myVTypeDict[DEFAULT_CONTAINERTYPE_ID] = MSVehicleType::build(defContainerType);
+
+    myReplaceableDefaultVTypes = DEFAULT_VTYPES;
 }
 
 
 SUMOVehicle*
 MSVehicleControl::buildVehicle(SUMOVehicleParameter* defs,
-                               const MSRoute* route, MSVehicleType* type,
-                               const bool ignoreStopErrors, const bool fromRouteFile) {
-    MSVehicle* built = new MSVehicle(defs, route, type, type->computeChosenSpeedDeviation(fromRouteFile ? MSRouteHandler::getParsingRNG() : nullptr));
-    initVehicle(built, ignoreStopErrors);
+                               ConstMSRoutePtr route, MSVehicleType* type,
+                               const bool ignoreStopErrors, const VehicleDefinitionSource source, bool addRouteStops) {
+    const double speedFactor = (source == VehicleDefinitionSource::STATE ? 1 :
+                                type->computeChosenSpeedDeviation(source == VehicleDefinitionSource::ROUTEFILE ? MSRouteHandler::getParsingRNG() : nullptr));
+    MSVehicle* built = new MSVehicle(defs, route, type, speedFactor);
+    initVehicle(built, ignoreStopErrors, addRouteStops, source);
     return built;
 }
 
 
 void
-MSVehicleControl::initVehicle(MSBaseVehicle* built, const bool ignoreStopErrors) {
+MSVehicleControl::initVehicle(MSBaseVehicle* built, const bool ignoreStopErrors, bool addRouteStops, const VehicleDefinitionSource source) {
     myLoadedVehNo++;
     try {
         built->initDevices();
-        built->addStops(ignoreStopErrors);
+        if (source != VehicleDefinitionSource::STATE) {
+            built->addStops(ignoreStopErrors, nullptr, addRouteStops);
+        }
     } catch (ProcessError&) {
         delete built;
         throw;
@@ -146,6 +150,7 @@ MSVehicleControl::isPendingRemoval(SUMOVehicle* veh) {
 #endif
 }
 
+
 void
 MSVehicleControl::removePending() {
     OutputDevice* const tripinfoOut = OptionsCont::getOptions().isSet("tripinfo-output") ? &OutputDevice::getDeviceByOption("tripinfo-output") : nullptr;
@@ -168,7 +173,11 @@ MSVehicleControl::removePending() {
             // close tag after tripinfo (possibly including emissions from another device) have been written
             tripinfoOut->closeTag();
         }
-        deleteVehicle(veh);
+        if (myKeepTime == 0) {
+            deleteVehicle(veh);
+        } else {
+            deleteKeptVehicle(veh);
+        }
     }
     vehs.clear();
     if (tripinfoOut != nullptr) {
@@ -181,6 +190,12 @@ MSVehicleControl::removePending() {
 }
 
 
+void
+MSVehicleControl::deleteKeptVehicle(SUMOVehicle* veh) {
+    myEndedVehNo++;
+    MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(new DeleteKeptVehicle(veh), SIMSTEP + myKeepTime);
+}
+
 void
 MSVehicleControl::vehicleDeparted(const SUMOVehicle& v) {
     ++myRunningVehNo;
@@ -190,6 +205,8 @@ MSVehicleControl::vehicleDeparted(const SUMOVehicle& v) {
     if ((v.getVClass() & (SVC_PEDESTRIAN | SVC_NON_ROAD)) == 0) {
         // only  worry about deceleration of road users
         myMinDeceleration = MIN2(myMinDeceleration, v.getVehicleType().getCarFollowModel().getMaxDecel());
+    } else if ((v.getVClass() & SVC_RAIL_CLASSES) != 0) {
+        myMinDecelerationRail = MIN2(myMinDecelerationRail, v.getVehicleType().getCarFollowModel().getMaxDecel());
     }
 }
 
@@ -213,59 +230,62 @@ MSVehicleControl::saveState(OutputDevice& out) {
     out.writeAttr(SUMO_ATTR_DEPART, myTotalDepartureDelay);
     out.writeAttr(SUMO_ATTR_TIME, myTotalTravelTime).closeTag();
     // save vehicle types
-    VTypeDictType vTypes = myVTypeDict;
-    if (myDefaultVTypeMayBeDeleted) {
-        vTypes.erase(DEFAULT_VTYPE_ID);
+    for (const auto& item : myVTypeDict) {
+        if (myReplaceableDefaultVTypes.count(item.first) == 0) {
+            item.second->getParameter().write(out);
+        }
     }
-    if (myDefaultPedTypeMayBeDeleted) {
-        vTypes.erase(DEFAULT_PEDTYPE_ID);
-    }
-    if (myDefaultContainerTypeMayBeDeleted) {
-        vTypes.erase(DEFAULT_CONTAINERTYPE_ID);
-    }
-    if (myDefaultBikeTypeMayBeDeleted) {
-        vTypes.erase(DEFAULT_BIKETYPE_ID);
-    }
-    if (myDefaultTaxiTypeMayBeDeleted) {
-        vTypes.erase(DEFAULT_TAXITYPE_ID);
-    }
-    for (const auto& item : vTypes) {
-        item.second->getParameter().write(out);
-    }
-    for (VTypeDistDictType::iterator it = myVTypeDistDict.begin(); it != myVTypeDistDict.end(); ++it) {
-        out.openTag(SUMO_TAG_VTYPE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, it->first);
-        out.writeAttr(SUMO_ATTR_VTYPES, (*it).second->getVals());
-        out.writeAttr(SUMO_ATTR_PROBS, (*it).second->getProbs());
+    for (const auto& item : myVTypeDistDict) {
+        out.openTag(SUMO_TAG_VTYPE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, item.first);
+        out.writeAttr(SUMO_ATTR_VTYPES, item.second->getVals());
+        out.writeAttr(SUMO_ATTR_PROBS, item.second->getProbs());
         out.closeTag();
     }
-    for (VehicleDictType::iterator it = myVehicleDict.begin(); it != myVehicleDict.end(); ++it) {
-        (*it).second->saveState(out);
+    std::vector<SUMOVehicle*> sortedVehs;
+    for (const auto& item : myVehicleDict) {
+        sortedVehs.push_back(item.second);
+    }
+    std::sort(sortedVehs.begin(), sortedVehs.end(), ComparatorNumericalIdLess());
+    for (SUMOVehicle* veh : sortedVehs) {
+        veh->saveState(out);
     }
 }
 
 
 void
-MSVehicleControl::clearState() {
-    for (VehicleDictType::iterator i = myVehicleDict.begin(); i != myVehicleDict.end(); ++i) {
-        delete (*i).second;
+MSVehicleControl::clearState(const bool reinit) {
+    for (const auto& item : myVehicleDict) {
+        delete item.second;
     }
     myVehicleDict.clear();
     // delete vehicle type distributions
-    for (VTypeDistDictType::iterator i = myVTypeDistDict.begin(); i != myVTypeDistDict.end(); ++i) {
-        delete (*i).second;
+    for (const auto& item : myVTypeDistDict) {
+        delete item.second;
     }
     myVTypeDistDict.clear();
     // delete vehicle types
-    for (VTypeDictType::iterator i = myVTypeDict.begin(); i != myVTypeDict.end(); ++i) {
-        delete (*i).second;
+    for (const auto& item : myVTypeDict) {
+        delete item.second;
     }
     myVTypeDict.clear();
     myPendingRemovals.clear(); // could be leftovers from MSVehicleTransfer::checkInsertions (teleport beyond arrival)
-    myDefaultVTypeMayBeDeleted = true;
-    myDefaultPedTypeMayBeDeleted = true;
-    myDefaultContainerTypeMayBeDeleted = true;
-    myDefaultBikeTypeMayBeDeleted = true;
-    myDefaultTaxiTypeMayBeDeleted = true;
+    if (reinit) {
+        initDefaultTypes();
+    }
+    myLoadedVehNo = 0;
+    myRunningVehNo = 0;
+    myEndedVehNo = 0;
+    myDiscarded = 0;
+    myCollisions = 0;
+    myTeleportsCollision = 0;
+    myTeleportsJam = 0;
+    myTeleportsYield = 0;
+    myTeleportsWrongLane = 0;
+    myEmergencyStops = 0;
+    myEmergencyBrakingCount = 0;
+    myStoppedVehicles = 0;
+    myTotalDepartureDelay = 0;
+    myTotalTravelTime = 0;
 }
 
 
@@ -275,16 +295,8 @@ MSVehicleControl::addVehicle(const std::string& id, SUMOVehicle* v) {
     if (it == myVehicleDict.end()) {
         // id not in myVehicleDict.
         myVehicleDict[id] = v;
+        handleTriggeredDepart(v, true);
         const SUMOVehicleParameter& pars = v->getParameter();
-        if (pars.departProcedure == DEPART_TRIGGERED || pars.departProcedure == DEPART_CONTAINER_TRIGGERED || pars.departProcedure == DEPART_SPLIT) {
-            const MSEdge* const firstEdge = v->getRoute().getEdges()[0];
-            if (!MSGlobals::gUseMesoSim) {
-                // position will be checked against person position later
-                static_cast<MSVehicle*>(v)->setTentativeLaneAndPosition(firstEdge->getLanes()[0], v->getParameter().departPos);
-            }
-            firstEdge->addWaiting(v);
-            registerOneWaiting();
-        }
         if (v->getVClass() != SVC_TAXI && pars.line != "" && pars.repetitionNumber < 0) {
             myPTVehicles.push_back(v);
         }
@@ -294,6 +306,38 @@ MSVehicleControl::addVehicle(const std::string& id, SUMOVehicle* v) {
 }
 
 
+void
+MSVehicleControl::handleTriggeredDepart(SUMOVehicle* v, bool add) {
+    const SUMOVehicleParameter& pars = v->getParameter();
+    if (pars.departProcedure == DepartDefinition::TRIGGERED || pars.departProcedure == DepartDefinition::CONTAINER_TRIGGERED || pars.departProcedure == DepartDefinition::SPLIT) {
+        const MSEdge* const firstEdge = v->getRoute().getEdges()[pars.departEdge];
+        if (add) {
+            if (!MSGlobals::gUseMesoSim) {
+                // position will be checked against person position later
+                static_cast<MSVehicle*>(v)->setTentativeLaneAndPosition(nullptr, v->getParameter().departPos);
+            }
+            if (firstEdge->isTazConnector()) {
+                for (MSEdge* out : firstEdge->getSuccessors()) {
+                    out->addWaiting(v);
+                }
+            } else {
+                firstEdge->addWaiting(v);
+            }
+            registerOneWaiting();
+        } else {
+            if (firstEdge->isTazConnector()) {
+                for (MSEdge* out : firstEdge->getSuccessors()) {
+                    out->removeWaiting(v);
+                }
+            } else {
+                firstEdge->removeWaiting(v);
+            }
+            unregisterOneWaiting();
+        }
+    }
+}
+
+
 SUMOVehicle*
 MSVehicleControl::getVehicle(const std::string& id) const {
     VehicleDictType::const_iterator it = myVehicleDict.find(id);
@@ -305,10 +349,12 @@ MSVehicleControl::getVehicle(const std::string& id) const {
 
 
 void
-MSVehicleControl::deleteVehicle(SUMOVehicle* veh, bool discard) {
-    myEndedVehNo++;
-    if (discard) {
-        myDiscarded++;
+MSVehicleControl::deleteVehicle(SUMOVehicle* veh, bool discard, bool wasKept) {
+    if (!wasKept) {
+        myEndedVehNo++;
+        if (discard) {
+            myDiscarded++;
+        }
     }
     if (veh != nullptr) {
         myVehicleDict.erase(veh->getID());
@@ -323,46 +369,9 @@ MSVehicleControl::deleteVehicle(SUMOVehicle* veh, bool discard) {
 
 bool
 MSVehicleControl::checkVType(const std::string& id) {
-    if (id == DEFAULT_VTYPE_ID) {
-        if (myDefaultVTypeMayBeDeleted) {
-            delete myVTypeDict[id];
-            myVTypeDict.erase(myVTypeDict.find(id));
-            myDefaultVTypeMayBeDeleted = false;
-        } else {
-            return false;
-        }
-    } else if (id == DEFAULT_PEDTYPE_ID) {
-        if (myDefaultPedTypeMayBeDeleted) {
-            delete myVTypeDict[id];
-            myVTypeDict.erase(myVTypeDict.find(id));
-            myDefaultPedTypeMayBeDeleted = false;
-        } else {
-            return false;
-        }
-    } else if (id == DEFAULT_CONTAINERTYPE_ID) {
-        if (myDefaultContainerTypeMayBeDeleted) {
-            delete myVTypeDict[id];
-            myVTypeDict.erase(myVTypeDict.find(id));
-            myDefaultContainerTypeMayBeDeleted = false;
-        } else {
-            return false;
-        }
-    } else if (id == DEFAULT_BIKETYPE_ID) {
-        if (myDefaultBikeTypeMayBeDeleted) {
-            delete myVTypeDict[id];
-            myVTypeDict.erase(myVTypeDict.find(id));
-            myDefaultBikeTypeMayBeDeleted = false;
-        } else {
-            return false;
-        }
-    } else if (id == DEFAULT_TAXITYPE_ID) {
-        if (myDefaultTaxiTypeMayBeDeleted) {
-            delete myVTypeDict[id];
-            myVTypeDict.erase(myVTypeDict.find(id));
-            myDefaultTaxiTypeMayBeDeleted = false;
-        } else {
-            return false;
-        }
+    if (myReplaceableDefaultVTypes.erase(id) > 0) {
+        delete myVTypeDict[id];
+        myVTypeDict.erase(myVTypeDict.find(id));
     } else {
         if (myVTypeDict.find(id) != myVTypeDict.end() || myVTypeDistDict.find(id) != myVTypeDistDict.end()) {
             return false;
@@ -371,6 +380,7 @@ MSVehicleControl::checkVType(const std::string& id) {
     return true;
 }
 
+
 bool
 MSVehicleControl::addVType(MSVehicleType* vehType) {
     if (checkVType(vehType->getID())) {
@@ -383,7 +393,7 @@ MSVehicleControl::addVType(MSVehicleType* vehType) {
 
 void
 MSVehicleControl::removeVType(const MSVehicleType* vehType) {
-    assert(vehType != 0);
+    assert(vehType != nullptr);
     assert(myVTypeDict.find(vehType->getID()) != myVTypeDict.end());
     myVTypeDict.erase(vehType->getID());
     if (myVTypeToDist.find(vehType->getID()) != myVTypeToDist.end()) {
@@ -424,7 +434,7 @@ MSVehicleControl::hasVTypeDistribution(const std::string& id) const {
 
 
 MSVehicleType*
-MSVehicleControl::getVType(const std::string& id, std::mt19937* rng, bool readOnly) {
+MSVehicleControl::getVType(const std::string& id, SumoRNG* rng, bool readOnly) {
     VTypeDictType::iterator it = myVTypeDict.find(id);
     if (it == myVTypeDict.end()) {
         VTypeDistDictType::iterator it2 = myVTypeDistDict.find(id);
@@ -433,10 +443,8 @@ MSVehicleControl::getVType(const std::string& id, std::mt19937* rng, bool readOn
         }
         return it2->second->get(rng);
     }
-    if (id == DEFAULT_VTYPE_ID && !readOnly) {
-        myDefaultVTypeMayBeDeleted = false;
-    } else if (id == DEFAULT_PEDTYPE_ID && !readOnly) {
-        myDefaultPedTypeMayBeDeleted = false;
+    if (!readOnly && myReplaceableDefaultVTypes.erase(id) > 0) {
+        it->second->check();
     }
     return it->second;
 }
@@ -463,21 +471,62 @@ MSVehicleControl::getVTypeDistributionMembership(const std::string& id) const {
     return it->second;
 }
 
+
 const RandomDistributor<MSVehicleType*>*
 MSVehicleControl::getVTypeDistribution(const std::string& typeDistID) const {
-    auto it = myVTypeDistDict.find(typeDistID);
+    const auto it = myVTypeDistDict.find(typeDistID);
     if (it != myVTypeDistDict.end()) {
         return it->second;
-    } else {
-        return nullptr;
     }
+    return nullptr;
 }
 
+
+const std::vector<MSVehicleType*>
+MSVehicleControl::getPedestrianTypes(void) const {
+    std::vector<MSVehicleType*> pedestrianTypes;
+    for (auto const& e : myVTypeDict)
+        if (e.second->getVehicleClass() == SUMOVehicleClass::SVC_PEDESTRIAN) {
+            pedestrianTypes.push_back(e.second);
+        }
+    return pedestrianTypes;
+}
+
+
 void
 MSVehicleControl::abortWaiting() {
     for (VehicleDictType::iterator i = myVehicleDict.begin(); i != myVehicleDict.end(); ++i) {
-        WRITE_WARNINGF("Vehicle '%' aborted waiting for a % that will never come.", i->first,
-                       i->second->getParameter().departProcedure == DEPART_SPLIT ? "split" : "person or container")
+        SUMOVehicle* veh = i->second;
+        std::string waitReason;
+        if (veh->isStoppedTriggered()) {
+            const MSStop& stop = veh->getNextStop();
+            if (stop.triggered) {
+                waitReason = "for a person that will never come";
+            } else if (stop.containerTriggered) {
+                waitReason = "for a container that will never come";
+            } else if (stop.joinTriggered) {
+                if (stop.pars.join != "") {
+                    waitReason = "to be joined to vehicle '" + stop.pars.join + "'";
+                } else {
+                    waitReason = "for a joining vehicle that will never come";
+                }
+            } else {
+                waitReason = "for an unknown trigger";
+            }
+        } else if (!veh->hasDeparted()) {
+            if (veh->getParameter().departProcedure == DepartDefinition::SPLIT) {
+                waitReason = "for a train from which to split";
+            } else if (veh->getParameter().departProcedure == DepartDefinition::TRIGGERED) {
+                waitReason = "at insertion for a person that will never come";
+            } else if (veh->getParameter().departProcedure == DepartDefinition::CONTAINER_TRIGGERED) {
+                waitReason = "at insertion for a container that will never come";
+            } else {
+                waitReason = "for an unknown departure trigger";
+            }
+        } else {
+            waitReason = "for an unknown reason";
+        }
+        WRITE_WARNINGF(TL("Vehicle '%' aborted waiting %."), i->first, waitReason);
     }
 }
 
@@ -495,7 +544,6 @@ MSVehicleControl::getHaltingVehicleNo() const {
 }
 
 
-
 std::pair<double, double>
 MSVehicleControl::getVehicleMeanSpeeds() const {
     double speedSum = 0;
@@ -506,7 +554,7 @@ MSVehicleControl::getVehicleMeanSpeeds() const {
         if ((veh->isOnRoad() || veh->isRemoteControlled()) && !veh->isStopped()) {
             count++;
             speedSum += veh->getSpeed();
-            relSpeedSum += veh->getSpeed() / veh->getEdge()->getSpeedLimit();
+            relSpeedSum += veh->getEdge()->getSpeedLimit() > 0 ? veh->getSpeed() / veh->getEdge()->getSpeedLimit() : 0;
         }
     }
     if (count > 0) {
@@ -528,20 +576,30 @@ MSVehicleControl::getQuota(double frac, int loaded) const {
     return getScalingQuota(frac, origLoaded);
 }
 
+
 int
 MSVehicleControl::getTeleportCount() const {
-    return (MSLane::teleportOnCollision() ? myCollisions : 0) + myTeleportsJam + myTeleportsYield + myTeleportsWrongLane;
+    return myTeleportsCollision + myTeleportsJam + myTeleportsYield + myTeleportsWrongLane;
 }
 
 
 void
-MSVehicleControl::adaptIntermodalRouter(MSNet::MSIntermodalRouter& router) const {
+MSVehicleControl::adaptIntermodalRouter(MSTransportableRouter& router) const {
     for (const SUMOVehicle* const veh : myPTVehicles) {
         // add single vehicles with line attribute which are not part of a flow
-        const MSRoute* const route = MSRoute::dictionary(veh->getParameter().routeid);
+        ConstMSRoutePtr const route = MSRoute::dictionary(veh->getParameter().routeid);
         router.getNetwork()->addSchedule(veh->getParameter(), route == nullptr ? nullptr : &route->getStops());
     }
 }
 
+// ===========================================================================
+// MSVehicleControl::DeleteKeptVehicle method definitions
+// ===========================================================================
+
+SUMOTime
+MSVehicleControl::DeleteKeptVehicle::execute(SUMOTime /*currentTime*/) {
+    MSNet::getInstance()->getVehicleControl().deleteVehicle(myVehicle, false, true);
+    return 0;
+}
 
 /****************************************************************************/
