@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2010-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -32,11 +32,11 @@
 class MSSOTLPlatoonPolicy: public MSSOTLPolicy, public SigmoidLogic, public PushButtonLogic {
 
 public:
-    MSSOTLPlatoonPolicy(const std::map<std::string, std::string>& parameters);
+    MSSOTLPlatoonPolicy(const Parameterised::Map& parameters);
     MSSOTLPlatoonPolicy(MSSOTLPolicyDesirability* desirabilityAlgorithm);
 
     MSSOTLPlatoonPolicy(MSSOTLPolicyDesirability* desirabilityAlgorithm,
-                        const std::map<std::string, std::string>& parameters);
+                        const Parameterised::Map& parameters);
 
     bool canRelease(SUMOTime elapsed, bool thresholdPassed, bool pushButtonPressed,
                     const MSPhaseDefinition* stage, int vehicleCount);
