@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2010-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2010-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -36,7 +36,7 @@ public:
     MSSOTLWaveTrafficLightLogic(MSTLLogicControl& tlcontrol,
                                 const std::string& id, const std::string& programID,
                                 const Phases& phases, int step, SUMOTime delay,
-                                const std::map<std::string, std::string>& parameters);
+                                const Parameterised::Map& parameters);
 
     /**
      * @brief Constructor with sensors passed
@@ -50,7 +50,7 @@ public:
     MSSOTLWaveTrafficLightLogic(MSTLLogicControl& tlcontrol,
                                 const std::string& id, const std::string& programID,
                                 const Phases& phases, int step, SUMOTime delay,
-                                const std::map<std::string, std::string>& parameters,
+                                const Parameterised::Map& parameters,
                                 MSSOTLSensors* sensors);
 
     /** @brief Returns the type of the logic as a string
