@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -35,27 +35,51 @@
 // ===========================================================================
 // class definitions
 // ===========================================================================
-/**
- * @class GNENet
- * @brief A NBNetBuilder extended by visualisation and editing capabilities
- */
+
 class GNENet : public GUIGlObject {
 
 public:
-    /**@brief Constructor
-     * @param[in] netbuilder the netbuilder which may already have been filled
-     * GNENet becomes responsible for cleaning this up
-     **/
-    GNENet(NBNetBuilder* netBuilder);
+    /// @brief constructor
+    GNENet(GNEApplicationWindow* applicationWindow, NBNetBuilder* netBuilder);
 
     /// @brief Destructor
     ~GNENet();
 
-    /// @brief retrieve all attribute carriers of Net
+    /// @brief get tag properties database
+    GNEApplicationWindow* getGNEApplicationWindow() const;
+
+    /// @brief get view net (used for simplify code)
+    GNEViewNet* getViewNet() const;
+
+    /// @brief get view parent (used for simplify code)
+    GNEViewParent* getViewParent() const;
+
+    /// @brief get undo list(used for simplify code)
+    GNEUndoList* getUndoList() const;
+
+    /// @brief get tag properties database (used for simplify code)
+    const GNETagPropertiesDatabase* getTagPropertiesDatabase() const;
+
+    /// @brief get net builder
+    NBNetBuilder* getNetBuilder() const;
+
+    /// @brief get all attribute carriers used in this net
     GNENetHelper::AttributeCarriers* getAttributeCarriers() const;
 
-    /// @brief get path manager
-    GNEPathManager* getPathManager();
+    /// @brief get all attribute carriers templates used in this net
+    GNENetHelper::ACTemplate* getACTemplates() const;
+
+    /// @brief get saving status
+    GNENetHelper::SavingStatus* getSavingStatus() const;
+
+    /// @brief get network path manager
+    GNEPathManager* getNetworkPathManager();
+
+    /// @brief get demand path manager
+    GNEPathManager* getDemandPathManager();
+
+    /// @brief get data path manager
+    GNEPathManager* getDataPathManager();
 
     /// @name inherited from GUIGlObject
     /// @{
@@ -66,7 +90,7 @@ public:
      * @return The built popup-menu
      * @see GUIGlObject::getPopUpMenu
      */
-    GUIGLObjectPopupMenu* getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent);
+    GUIGLObjectPopupMenu* getPopUpMenu(GUIMainWindow& app, GUISUMOAbstractView& parent) override;
 
     /**@brief Returns an own parameter window
      *
@@ -75,14 +99,14 @@ public:
      * @return The built parameter window
      * @see GUIGlObject::getParameterWindow
      */
-    GUIParameterTableWindow* getParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent);
+    GUIParameterTableWindow* getParameterWindow(GUIMainWindow& app, GUISUMOAbstractView& parent) override;
 
     /**@brief Returns the boundary to which the view shall be centered in order to show the object
      *
      * @return The boundary the object is within
      * @see GUIGlObject::getCenteringBoundary
      */
-    Boundary getCenteringBoundary() const;
+    Boundary getCenteringBoundary() const override;
 
     /// @brief expand boundary
     void expandBoundary(const Boundary& newBoundary);
@@ -97,7 +121,8 @@ public:
      * @param[in] s The settings for the current view (may influence drawing)
      * @see GUIGlObject::drawGL
      */
-    void drawGL(const GUIVisualizationSettings& s) const;
+    void drawGL(const GUIVisualizationSettings& s) const override;
+
     /// @}
 
     /// @brief returns the bounder of the network
@@ -105,10 +130,13 @@ public:
 
     /**@brief Returns the RTree used for visualisation speed-up
      * @return The visualisation speed-up
-     * @note only use in GNEViewNet constructor
+     * @note only use in GNEViewNet constructor. For edit grid ALWAYS use addGLObjectIntoGrid/removeGLObjectIntoGrid
      */
     SUMORTree& getGrid();
 
+    /// @brief et edges and number of lanes
+    const std::map<std::string, int>& getEdgesAndNumberOfLanes() const;
+
     /**@brief creates a new junction
      * @param[in] position The position of the new junction
      * @param[in] undoList The undolist in which to mark changes
@@ -132,6 +160,12 @@ public:
                         const std::string& suggestedName = "", bool wasSplit = false, bool allowDuplicateGeom = false,
                         bool recomputeConnections = true);
 
+    /**@brief delete network element
+     * @param[in] networkElement The network element to be removed
+     * @param[in] undoList The undolist in which to mark changes
+     */
+    void deleteNetworkElement(GNENetworkElement* networkElement, GNEUndoList* undoList);
+
     /**@brief removes junction and all incident edges
      * @param[in] junction The junction to be removed
      * @param[in] undoList The undolist in which to mark changes
@@ -170,22 +204,16 @@ public:
     void deleteCrossing(GNECrossing* crossing, GNEUndoList* undoList);
 
     /**@brief remove additional
-     * @param[in] additional The Shape to be removed
+     * @param[in] additional The additional to be removed
      * @param[in] undoList The undolist in which to mark changes
      */
     void deleteAdditional(GNEAdditional* additional, GNEUndoList* undoList);
 
-    /**@brief remove shape
-     * @param[in] shape The Shape to be removed
+    /**@brief remove TAZSourceSink
+     * @param[in] TAZSourceSink The TAZSourceSink to be removed
      * @param[in] undoList The undolist in which to mark changes
      */
-    void deleteShape(GNEShape* shape, GNEUndoList* undoList);
-
-    /**@brief remove TAZElement
-     * @param[in] TAZElement The TAZElement to be removed
-     * @param[in] undoList The undolist in which to mark changes
-     */
-    void deleteTAZElement(GNETAZElement* TAZElement, GNEUndoList* undoList);
+    void deleteTAZSourceSink(GNETAZSourceSink* TAZSourceSink, GNEUndoList* undoList);
 
     /**@brief remove demand element
      * @param[in] demandElement The Shape to be removed
@@ -211,6 +239,12 @@ public:
      */
     void deleteGenericData(GNEGenericData* genericData, GNEUndoList* undoList);
 
+    /**@brief remove generic data
+     * @param[in] genericData The generic data to be removed
+     * @param[in] undoList The undolist in which to mark changes
+     */
+    void deleteMeanData(GNEMeanData* meanData, GNEUndoList* undoList);
+
     /**@brief duplicates lane
      * @param[in] lane The lane to be duplicated
      * @param[in] undoList The undolist in which to mark changes
@@ -227,10 +261,18 @@ public:
     /**@brief add restricted lane to edge
      * @param[in] vclass vehicle class to restrict
      * @param[in] edge The edge in which insert restricted lane
+     * @param[in] index to be changed
      * @param[in] undoList The undolist in which to mark changes
      */
     bool addRestrictedLane(SUMOVehicleClass vclass, GNEEdge* edge, int index, GNEUndoList* undoList);
 
+    /**@brief add restricted lane to edge
+     * @param[in] edge The edge in which insert restricted lane
+     * @param[in] index to be changed
+     * @param[in] undoList The undolist in which to mark changes
+     */
+    bool addGreenVergeLane(GNEEdge* edge, int index, GNEUndoList* undoList);
+
     /**@brief remove restricted lane
      * @param[in] vclass vehicle class to restrict
      * @param[in] edge the edge in which remove sidewalk
@@ -241,9 +283,9 @@ public:
     /**@brief split edge at position by inserting a new junction
      * @param[in] edge The edge to be split
      * @param[in] pos The position on which to insert the new junction
-     * @return The new junction
+     * @return The new junction and the new edge
      */
-    GNEJunction* splitEdge(GNEEdge* edge, const Position& pos, GNEUndoList* undoList, GNEJunction* newJunction = 0);
+    std::pair<GNEJunction*, GNEEdge*> splitEdge(GNEEdge* edge, const Position& pos, GNEUndoList* undoList, GNEJunction* newJunction = 0);
 
     /**@brief split all edges at position by inserting one new junction
      * @param[in] edge The edge to be split
@@ -259,9 +301,10 @@ public:
 
     /**@brief add reversed edge
      * @param[in] edge The edge of which to add the reverse
+     * @param[in] disconnected add edge reversed or disconnected parallel
      * @return Return the new edge or 0
      */
-    GNEEdge* addReversedEdge(GNEEdge* edge, GNEUndoList* undoList);
+    GNEEdge* addReversedEdge(GNEEdge* edge, const bool disconnected, GNEUndoList* undoList);
 
     /**@brief merge the given junctions
      * edges between the given junctions will be deleted
@@ -269,7 +312,7 @@ public:
      * @param[in] target The junction that will be enlarged
      * @param[in] undoList The undo list with which to register changes
      */
-    void mergeJunctions(GNEJunction* moved, GNEJunction* target, GNEUndoList* undoList);
+    void mergeJunctions(GNEJunction* moved, const GNEJunction* target, GNEUndoList* undoList);
 
     /// @brief select all roundabout edges and junctions for the current roundabout
     void selectRoundabout(GNEJunction* junction, GNEUndoList* undoList);
@@ -277,137 +320,14 @@ public:
     /// @brief transform the given junction into a roundabout
     void createRoundabout(GNEJunction* junction, GNEUndoList* undoList);
 
-    /**@brief get junction by id
-     * @param[in] id The id of the desired junction
-     * @param[in] failHard Whether attempts to retrieve a nonexisting junction should result in an exception
-     * @throws UnknownElement
-     */
-    GNEJunction* retrieveJunction(const std::string& id, bool failHard = true) const;
+    /// @brief save the network
+    void saveNetwork();
 
-    /**@brief get edge type by id
-     * @param[in] id The id of the desired edge type
-     * @param[in] failHard Whether attempts to retrieve a nonexisting edge type should result in an exception
-     * @throws UnknownElement
-     */
-    GNEEdgeType* retrieveEdgeType(const std::string& id, bool failHard = true) const;
+    /// @brief save plain xml representation of the network (and nothing else)
+    void savePlain(const std::string& prefix, const OptionsCont& netconvertOptions);
 
-    /**@brief get edge by id
-     * @param[in] id The id of the desired edge
-     * @param[in] failHard Whether attempts to retrieve a nonexisting edge should result in an exception
-     * @throws UnknownElement
-     */
-    GNEEdge* retrieveEdge(const std::string& id, bool failHard = true) const;
-
-    /**@brief get edge by from and to GNEJunction
-     * @param[in] id The id of the desired edge
-     * @param[in] failHard Whether attempts to retrieve a nonexisting edge should result in an exception
-     * @throws UnknownElement
-     */
-    GNEEdge* retrieveEdge(GNEJunction* from, GNEJunction* to, bool failHard = true) const;
-
-    /**@brief get Connection by id
-     * @param[in] id The id of the desired Connection
-     * @param[in] failHard Whether attempts to retrieve a nonexisting Connection should result in an exception
-     * @throws UnknownElement
-     */
-    GNEConnection* retrieveConnection(const std::string& id, bool failHard = true) const;
-
-    /**@brief return all connections
-     * @param[in] onlySelected Whether to return only selected connections
-     */
-    std::vector<GNEConnection*> retrieveConnections(bool onlySelected = false) const;
-
-    /**@brief get Crossing by id
-     * @param[in] id The id of the desired Crossing
-     * @param[in] failHard Whether attempts to retrieve a nonexisting Crossing should result in an exception
-     * @throws UnknownElement
-     */
-    GNECrossing* retrieveCrossing(const std::string& id, bool failHard = true) const;
-
-    /**@brief return all crossings
-     * @param[in] onlySelected Whether to return only selected crossings
-     */
-    std::vector<GNECrossing*> retrieveCrossings(bool onlySelected = false) const;
-
-    /**@brief get a single attribute carrier based on a GLID
-     * @param[in] ids the GL IDs for which to retrive the AC
-     * @param[in] failHard Whether attempts to retrieve a nonexisting AttributeCarrier should result in an exception
-     * @throws InvalidArgument if GL ID doesn't have a associated Attribute Carrier
-     */
-    GNEAttributeCarrier* retrieveAttributeCarrier(const GUIGlID id, bool failHard = true) const;
-
-    /**@brief get the attribute carriers based on Type
-     * @param[in] type The GUI-type of the AC. SUMO_TAG_NOTHING returns all elements (Warning: bottleneck)
-     */
-    std::vector<GNEAttributeCarrier*> retrieveAttributeCarriers(SumoXMLTag type = SUMO_TAG_NOTHING);
-
-    /**@brief return all edges
-     * @param[in] onlySelected Whether to return only selected edges
-     */
-    std::vector<GNEEdge*> retrieveEdges(bool onlySelected = false);
-
-    /**@brief return edges with junction angle between 0 (inclusive) and 180 (exclusive) degrees
-     * @param[in] onlySelected Whether to return only selected edges
-     */
-    std::vector<GNEEdge*> retrieve000180AngleEdges(bool onlySelected = false) const;
-
-    /**@brief return edges with junction angle between 180 (inclusive) and 360 (exclusive) degrees
-     * @param[in] onlySelected Whether to return only selected edges
-     */
-    std::vector<GNEEdge*> retrieve180360AngleEdges(bool onlySelected = false) const;
-
-    /**@brief return all lanes
-     * @param[in] onlySelected Whether to return only selected lanes
-     */
-    std::vector<GNELane*> retrieveLanes(bool onlySelected = false);
-
-    /**@brief get lane by id
-     * @param[in] id The id of the desired lane
-     * @param[in] failHard Whether attempts to retrieve a nonexisting lane should result in an exception
-     * @param[in] checkVolatileChange Used by additionals after recomputing with volatile options.
-     * @throws UnknownElement
-     */
-    GNELane* retrieveLane(const std::string& id, bool failHard = true, bool checkVolatileChange = false);
-
-    /**@brief return all junctions
-     * @param[in] onlySelected Whether to return only selected junctions
-     */
-    std::vector<GNEJunction*> retrieveJunctions(bool onlySelected = false);
-
-    /**@brief return shape by type shapes
-     * @param[in] shapeTag Type of shape.
-     * @param[in] onlySelected Whether to return only selected junctions
-     */
-    std::vector<GNEShape*> retrieveShapes(SumoXMLTag shapeTag, bool onlySelected = false);
-
-    /**@brief return all shapes
-     * @param[in] onlySelected Whether to return only selected junctions
-     */
-    std::vector<GNEShape*> retrieveShapes(bool onlySelected = false);
-
-    /// @brief inform that net has to be saved
-    void requireSaveNet(bool value);
-
-    /// @brief return if net has to be saved
-    bool isNetSaved() const;
-
-    /**@brief save the network
-     * @param[in] oc The OptionsCont which knows how and where to save
-     */
-    void save(OptionsCont& oc);
-
-    /**@brief save plain xml representation of the network (and nothing else)
-     * @param[in] oc The OptionsCont which knows how and where to save
-     */
-    void savePlain(OptionsCont& oc);
-
-    /**@brief save log of joined junctions (and nothing else)
-     * @param[in] oc The OptionsCont which knows how and where to save
-     */
-    void saveJoined(OptionsCont& oc);
-
-    /// @brief Set the net to be notified of network changes
-    void setViewNet(GNEViewNet* viewNet);
+    /// @brief save log of joined junctions (and nothing else)
+    void saveJoined(const std::string& filename);
 
     /// @brief add GL Object into net
     void addGLObjectIntoGrid(GNEAttributeCarrier* AC);
@@ -418,11 +338,6 @@ public:
     /// @brief modifies endpoins of the given edge
     void changeEdgeEndpoints(GNEEdge* edge, const std::string& newSourceID, const std::string& newDestID);
 
-    /// @brief get view net
-    GNEViewNet* getViewNet() const;
-
-    /// @brief get all selected attribute carriers (or only relative to current supermode
-    std::vector<GNEAttributeCarrier*> getSelectedAttributeCarriers(bool ignoreCurrentSupermode);
 
     /// @brief returns the tllcont of the underlying netbuilder
     NBTrafficLightLogicCont& getTLLogicCont();
@@ -434,17 +349,14 @@ public:
     void initGNEConnections();
 
     /// @brief recompute the network and update lane geometries
-    void computeAndUpdate(OptionsCont& oc, bool volatileOptions);
+    void computeAndUpdate(OptionsCont& neteditOptions, bool volatileOptions);
 
     /**@brief trigger full netbuild computation
      * param[in] window The window to inform about delay
      * param[in] force Whether to force recomputation even if not needed
      * param[in] volatileOptions enable or disable volatile options
-     * param[in] additionalPath path in wich additionals were saved before recomputing with volatile options
-     * param[in] demandPath path in wich demand elements were saved before recomputing with volatile options
-     * param[in] dataPath path in wich data elements were saved before recomputing with volatile options
      */
-    void computeNetwork(GNEApplicationWindow* window, bool force = false, bool volatileOptions = false, std::string additionalPath = "", std::string demandPath = "", std::string dataPath = "");
+    void computeNetwork(GNEApplicationWindow* window, bool force = false, bool volatileOptions = false);
 
     /**@brief compute demand elements
      * param[in] window The window to inform about delay
@@ -460,7 +372,7 @@ public:
      * @note difference to mergeJunctions:
      *  - can join more than 2
      *  - connected edges will keep their geometry (big junction shape is created)
-     *  - no hirarchy: if any junction has a traffic light than the resuling junction will
+     *  - no hierarchy: if any junction has a traffic light than the resulting junction will
      */
     bool joinSelectedJunctions(GNEUndoList* undoList);
 
@@ -503,6 +415,9 @@ public:
     /// @brief clear data elements
     void clearDataElements(GNEUndoList* undoList);
 
+    /// @brief clear meanDatas
+    void clearMeanDataElements(GNEUndoList* undoList);
+
     /**@brief trigger recomputation of junction shape and logic
      * param[in] window The window to inform about delay
      */
@@ -511,143 +426,26 @@ public:
     /// @brief inform the net about the need for recomputation
     void requireRecompute();
 
-    /// @brief check if net requiere recomputing
+    /// @brief check if net require recomputing
     bool isNetRecomputed() const;
 
-    /// @brief check if net has GNECrossings
-    bool netHasGNECrossings() const;
-
-    /// @brief get pointer to the main App
-    FXApp* getApp();
-
-    /// @brief get net builder
-    NBNetBuilder* getNetBuilder() const;
-
     /// @brief add edge id to the list of explicit turnarounds
     void addExplicitTurnaround(std::string id);
 
     /// @brief remove edge id from the list of explicit turnarounds
     void removeExplicitTurnaround(std::string id);
 
-    /// @brief generate edgeType id
-    std::string generateEdgeTypeID() const;
+    /// @brief save additional elements
+    bool saveAdditionals();
 
-    /// @name Functions related to Additional Items
-    /// @{
+    /// @brief save JuPedSim elements
+    bool saveJuPedSimElements(const std::string& filename);
 
-    /**@brief Returns the named additional
-     * @param[in] type tag with the type of additional
-     * @param[in] id The id of the additional to return.
-     * @param[in] failHard Whether attempts to retrieve a nonexisting additional should result in an exception
-     */
-    GNEAdditional* retrieveAdditional(SumoXMLTag type, const std::string& id, bool hardFail = true) const;
+    /// @brief save demand element elements of the network
+    bool saveDemandElements();
 
-    /**@brief return all additionals
-     * @param[in] onlySelected Whether to return only selected additionals
-     */
-    std::vector<GNEAdditional*> retrieveAdditionals(bool onlySelected = false) const;
-
-    /**@brief Returns the number of additionals of the net
-     * @param[in] type type of additional to count. SUMO_TAG_NOTHING will count all additionals
-     * @return Number of additionals of the net
-     */
-    int getNumberOfAdditionals(SumoXMLTag type = SUMO_TAG_NOTHING) const;
-
-    /// @brief inform that additionals has to be saved
-    void requireSaveAdditionals(bool value);
-
-    /**@brief save additional elements of the network
-     * @param[in] filename name of the file in wich save additionals
-    */
-    void saveAdditionals(const std::string& filename);
-
-    /// @brief check if additionals are saved
-    bool isAdditionalsSaved() const;
-
-    /// @brief generate additional id
-    std::string generateAdditionalID(SumoXMLTag type) const;
-
-    /// @}
-
-    /// @name Functions related to DemandElement Items
-    /// @{
-
-    /**@brief Returns the named demand element
-     * @param[in] type tag with the type of demand element
-     * @param[in] id The id of the demand element to return.
-     * @param[in] failHard Whether attempts to retrieve a nonexisting demand element should result in an exception
-     */
-    GNEDemandElement* retrieveDemandElement(SumoXMLTag type, const std::string& id, bool hardFail = true) const;
-
-    /**@brief return all demand elements
-     * @param[in] onlySelected Whether to return only selected demand elements
-     */
-    std::vector<GNEDemandElement*> retrieveDemandElements(bool onlySelected = false) const;
-
-    /**@brief Returns the number of demand elements of the net
-     * @param[in] type type of demand element to count. SUMO_TAG_NOTHING will count all demand elements
-     * @return Number of demand elements of the net
-     */
-    int getNumberOfDemandElements(SumoXMLTag type = SUMO_TAG_NOTHING) const;
-
-    /// @brief inform that demand elements has to be saved
-    void requireSaveDemandElements(bool value);
-
-    /**@brief save demand element elements of the network
-     * @param[in] filename name of the file in wich save demand elements
-    */
-    void saveDemandElements(const std::string& filename);
-
-    /// @brief check if demand elements are saved
-    bool isDemandElementsSaved() const;
-
-    /// @brief generate demand element id
-    std::string generateDemandElementID(SumoXMLTag tag) const;
-
-    /// @}
-
-    /// @name Functions related to DataSet Items
-    /// @{
-
-    /**@brief Returns the named data set
-     * @param[in] id The id of the data set to return.
-     * @param[in] failHard Whether attempts to retrieve a nonexisting data set should result in an exception
-     */
-    GNEDataSet* retrieveDataSet(const std::string& id, bool hardFail = true) const;
-
-    ///@brief return all data sets
-    std::vector<GNEDataSet*> retrieveDataSets() const;
-
-    /**@brief return all generic datas
-     * @param[in] onlySelected Whether to return only selected generic datas
-    */
-    std::vector<GNEGenericData*> retrieveGenericDatas(bool onlySelected = false) const;
-
-    /**@brief Returns the number of data sets of the net
-     * @return Number of data sets of the net
-     */
-    int getNumberOfDataSets() const;
-
-    /// @brief inform that data sets has to be saved
-    void requireSaveDataElements(bool value);
-
-    /**@brief save data set elements of the network
-     * @param[in] filename name of the file in wich save data sets
-    */
-    void saveDataElements(const std::string& filename);
-
-    /// @brief check if data sets are saved
-    bool isDataElementsSaved() const;
-
-    /// @brief generate data set id
-    std::string generateDataSetID(const std::string& prefix) const;
-
-    /// @brief return a set of parameters for the given data Interval
-    std::set<std::string> retrieveGenericDataParameters(const std::string& genericDataTag, const double begin, const double end) const;
-
-    /// @brief return a set of parameters for the given dataSet, generic data Type, begin and end
-    std::set<std::string> retrieveGenericDataParameters(const std::string& dataSetID, const std::string& genericDataTag,
-            const std::string& beginStr, const std::string& endStr) const;
+    /// @brief save data set elements of the network
+    bool saveDataElements();
 
     /// @brief get minimum interval
     double getDataSetIntervalMinimumBegin() const;
@@ -655,83 +453,23 @@ public:
     /// @brief get maximum interval
     double getDataSetIntervalMaximumEnd() const;
 
-    /// @}
-
-    /// @name Functions related with Shapes
-    /// @{
-
-    /**@brief Returns the named shape
-     * @param[in] type tag with the type of shape
-     * @param[in] id The id of the shape to return.
-     * @param[in] failHard Whether attempts to retrieve a nonexisting shape should result in an exception
-     */
-    GNEShape* retrieveShape(SumoXMLTag type, const std::string& id, bool hardFail = true) const;
-
-    /**@brief return all shapes
-     * @param[in] onlySelected Whether to return only selected shapes
-     */
-    std::vector<GNEShape*> retrieveShapes(bool onlySelected = false) const;
-
-    /// @brief generate Shape ID
-    std::string generateShapeID(SumoXMLTag shapeTag) const;
-
-    /**@brief Returns the number of shapes of the net
-     * @param[in] type type of shape to count. SUMO_TAG_NOTHING will count all shapes
-     * @return Number of shapes of the net
-     */
-    int getNumberOfShapes(SumoXMLTag type = SUMO_TAG_NOTHING) const;
-    /// @}
-
-    /// @name Functions related with TAZElements
-    /// @{
-
-    /**@brief Returns the named TAZElement
-     * @param[in] type tag with the type of TAZElement
-     * @param[in] id The id of the TAZElement to return.
-     * @param[in] failHard Whether attempts to retrieve a nonexisting TAZElement should result in an exception
-     */
-    GNETAZElement* retrieveTAZElement(SumoXMLTag type, const std::string& id, bool hardFail = true) const;
-
-    /**@brief return all TAZElements
-     * @param[in] onlySelected Whether to return only selected TAZElements
-     */
-    std::vector<GNETAZElement*> retrieveTAZElements(bool onlySelected = false) const;
-
-    /// @brief generate TAZElement ID
-    std::string generateTAZElementID(SumoXMLTag TAZElementTag) const;
-
-    /**@brief Returns the number of TAZElements of the net
-     * @param[in] type type of TAZElement to count. SUMO_TAG_NOTHING will count all TAZElements
-     * @return Number of TAZElements of the net
-     */
-    int getNumberOfTAZElements(SumoXMLTag type = SUMO_TAG_NOTHING) const;
-    /// @}
-
-    /// @name Functions related to TLS Programs
-    /// @{
-    /// @brief inform that TLS Programs has to be saved
-    void requireSaveTLSPrograms();
+    /// @brief save meanData elements of the network
+    bool saveMeanDatas();
 
     /**@brief save TLS Programs elements of the network
-     * @param[in] filename name of the file in wich save TLS Programs
+     * @param[in] filename name of the file in which save TLS Programs
      */
     void saveTLSPrograms(const std::string& filename);
 
     /// @brief get number of TLS Programs
     int getNumberOfTLSPrograms() const;
 
-    /// @}
-
-    /// @name Functions related to TLS Programs
-    /// @{
     /**@brief save edgeTypes elements of the network
-     * @param[in] filename name of the file in wich save edgeTypes
+     * @param[in] filename name of the file in which save edgeTypes
     */
     void saveEdgeTypes(const std::string& filename);
 
-    /// @}
-
-    /// @name Functions related to Enable or disable update geometry of elements after insertio
+    /// @name Functions related to Enable or disable update geometry of elements after insertion
     /// @{
     /// @brief enable update geometry of elements after inserting or removing an element in net
     void enableUpdateGeometry();
@@ -744,7 +482,7 @@ public:
 
     /// @}
 
-    /// @name Functions related to Enable or disable update data of elements after insertio
+    /// @name Functions related to Enable or disable update data of elements after insertion
     /// @{
     /// @brief enable update data elements after inserting or removing an element in net
     void enableUpdateData();
@@ -757,87 +495,151 @@ public:
 
     /// @}
 
+    /// @name get junction id counter
+    unsigned int& getJunctionIDCounter();
+
+    /// @name get edge id counter
+    unsigned int& getEdgeIDCounter();
+
+    /// @brief variable used for write headers in additional, demand and data elements
+    static const std::map<SumoXMLAttr, std::string> EMPTY_HEADER;
+
 protected:
     /// @brief the rtree which contains all GUIGlObjects (so named for historical reasons)
     SUMORTree myGrid;
 
-    /// @brief The net to be notofied of about changes
-    GNEViewNet* myViewNet;
+    /// @brief pointer to application window
+    GNEApplicationWindow* myApplicationWindow = nullptr;
 
     /// @brief The internal netbuilder
-    NBNetBuilder* myNetBuilder;
+    NBNetBuilder* myNetBuilder = nullptr;
 
-    /// @brief AttributeCarriers of net
-    GNENetHelper::AttributeCarriers* myAttributeCarriers;
+    /// @brief attributeCarriers module
+    GNENetHelper::AttributeCarriers* myAttributeCarriers = nullptr;
 
-    /// @brief Path manager
-    GNEPathManager* myPathManager;
+    /// @brief attributeCarriers templates
+    GNENetHelper::ACTemplate* myACTemplates = nullptr;
 
-    /// @name ID Suppliers for newly created edges and junctions
+    /// @brief saving status module
+    GNENetHelper::SavingStatus* mySavingStatus = nullptr;
+
+    /// @brief Network path manager
+    GNEPathManager* myNetworkPathManager = nullptr;
+
+    /// @brief Demand path manager
+    GNEPathManager* myDemandPathManager = nullptr;
+
+    /// @brief Data path manager
+    GNEPathManager* myDataPathManager = nullptr;
+
+    /// @name counters for junction/edge IDs
     // @{
-    IDSupplier myEdgeIDSupplier;
-    IDSupplier myJunctionIDSupplier;
+    unsigned int myJunctionIDCounter = 0;
+    unsigned int myEdgeIDCounter = 0;
     // @}
 
     /// @brief list of edge ids for which turn-arounds must be added explicitly
     std::set<std::string> myExplicitTurnarounds;
 
     /// @brief whether the net needs recomputation
-    bool myNeedRecompute;
-
-    /// @brief Flag to check if net has to be saved
-    bool myNetSaved;
-
-    /// @brief Flag to check if additionals has to be saved
-    bool myAdditionalsSaved;
-
-    /// @brief Flag to check if shapes has to be saved
-    bool myTLSProgramsSaved;
-
-    /// @brief Flag to check if demand elements has to be saved
-    bool myDemandElementsSaved;
-
-    /// @brief Flag to check if data elements has to be saved
-    bool myDataElementsSaved;
+    bool myNeedRecompute = true;
 
     /// @brief Flag to enable or disable update geometry of elements after inserting or removing element in net
-    bool myUpdateGeometryEnabled;
+    bool myUpdateGeometryEnabled = true;
 
     /// @brief Flag to enable or disable update data elements after inserting or removing element in net
-    bool myUpdateDataEnabled;
+    bool myUpdateDataEnabled = true;
 
 private:
     /// @brief Init Junctions and edges
     void initJunctionsAndEdges();
 
-    /// @brief reserve edge ID (To avoid duplicates)
-    void reserveEdgeID(const std::string& id);
-
-    /// @brief reserve junction ID (To avoid duplicates)
-    void reserveJunctionID(const std::string& id);
-
     /// @brief return true if there are already a Junction in the given position, false in other case
     bool checkJunctionPosition(const Position& pos);
 
-    /// @brief save additionals after confirming invalid objects
-    void saveAdditionalsConfirmed(const std::string& filename);
+    /// @brief write additionals demand elements and meanData in output device
+    void writeAdditionalFileElements(OutputDevice& device, const FileBucket* fileBucket);
 
-    /// @brief save demand elements after confirming invalid objects
-    void saveDemandElementsConfirmed(const std::string& filename);
+    /// @brief write additional element by type and sorted by ID
+    void writeAdditionalByType(OutputDevice& device, const FileBucket* fileBucket, const std::vector<SumoXMLTag> tags) const;
 
-    /// @brief save data elements after confirming invalid objects
-    void saveDataElementsConfirmed(const std::string& filename);
+    /// @brief write demand element by type and sorted by ID
+    void writeDemandByType(OutputDevice& device, const FileBucket* fileBucket, SumoXMLTag tag) const;
 
+    /// @brief write route distributions sorted by ID
+    void writeRouteDistributions(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write route sorted by ID
+    void writeRoutes(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write vTypeDistributions sorted by ID
+    void writeVTypeDistributions(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write vTypes sorted by ID
+    void writeVTypes(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write meanData element by type and sorted by ID
+    void writeMeanDatas(OutputDevice& device, const FileBucket* fileBucket, SumoXMLTag tag) const;
+
+    /// @brief write vType comment
+    bool writeVTypeComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write route comment
+    bool writeRouteComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write rerouter comment
+    bool writeRerouterComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write variable speed sign comment
+    bool writeVariableSpeedSignComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write routeProbe comment
+    bool writeRouteProbeComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write vaporizer comment
+    bool writeVaporizerComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write calibrator comment
+    bool writeCalibratorComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write stoppingPlace comment
+    bool writeStoppingPlaceComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write detector comment
+    bool writeDetectorComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write shape comment
+    bool writeShapesComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write JuPedSim comment
+    bool writeJuPedSimComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write TAZ comment
+    bool writeTAZComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write Wire comment
+    bool writeWireComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write meanDataEdge comment
+    bool writeMeanDataEdgeComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief write Wire comment
+    bool writeMeanDataLaneComment(OutputDevice& device, const FileBucket* fileBucket) const;
+
+    /// @brief replace in list attribute
     static void replaceInListAttribute(GNEAttributeCarrier* ac, SumoXMLAttr key, const std::string& which, const std::string& by, GNEUndoList* undoList);
 
     /// @brief the z boundary (stored in the x-coordinate), values of 0 are ignored
     Boundary myZBoundary;
 
+    /// @brief map with the Edges and their number of lanes
+    std::map<std::string, int> myEdgesAndNumberOfLanes;
+
     /// @brief marker for whether the z-boundary is initialized
     static const double Z_INITIALIZED;
 
-    /// @brief map with the Edges and their number of lanes
-    std::map<std::string, int> myEdgesAndNumberOfLanes;
+    /// @brief Invalidated default constructor.
+    GNENet() = delete;
 
     /// @brief Invalidated copy constructor.
     GNENet(const GNENet&) = delete;
