@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2011-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2011-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -17,20 +17,20 @@
 ///
 // The Widget for modifying lane-to-lane connections
 /****************************************************************************/
-#include <config.h>
 
-#include <utils/gui/windows/GUIAppEnum.h>
-#include <utils/gui/div/GUIDesigns.h>
 #include <netedit/changes/GNEChange_Connection.h>
-#include <netedit/GNEViewParent.h>
-#include <netedit/GNEUndoList.h>
-#include <netedit/GNENet.h>
-#include <netedit/GNEViewNet.h>
+#include <netedit/dialogs/basic/GNEWarningBasicDialog.h>
 #include <netedit/elements/network/GNEConnection.h>
-#include <netedit/elements/network/GNELane.h>
-#include <netedit/frames/network/GNEConnectorFrame.h>
 #include <netedit/frames/common/GNESelectorFrame.h>
+#include <netedit/GNEApplicationWindow.h>
+#include <netedit/GNENet.h>
+#include <netedit/GNEUndoList.h>
+#include <netedit/GNEViewParent.h>
+#include <utils/foxtools/MFXDynamicLabel.h>
+#include <utils/gui/div/GUIDesigns.h>
+#include <utils/gui/windows/GUIAppEnum.h>
 
+#include "GNEConnectorFrame.h"
 
 // ===========================================================================
 // FOX callback mapping
@@ -51,8 +51,8 @@ FXDEFMAP(GNEConnectorFrame::ConnectionOperations) ConnectionOperationsMap[] = {
 };
 
 // Object implementation
-FXIMPLEMENT(GNEConnectorFrame::ConnectionModifications, FXGroupBox, ConnectionModificationsMap, ARRAYNUMBER(ConnectionModificationsMap))
-FXIMPLEMENT(GNEConnectorFrame::ConnectionOperations,    FXGroupBox, ConnectionOperationsMap,    ARRAYNUMBER(ConnectionOperationsMap))
+FXIMPLEMENT(GNEConnectorFrame::ConnectionModifications, GNEGroupBoxModule, ConnectionModificationsMap, ARRAYNUMBER(ConnectionModificationsMap))
+FXIMPLEMENT(GNEConnectorFrame::ConnectionOperations,    GNEGroupBoxModule, ConnectionOperationsMap,    ARRAYNUMBER(ConnectionOperationsMap))
 
 
 // ===========================================================================
@@ -64,9 +64,9 @@ FXIMPLEMENT(GNEConnectorFrame::ConnectionOperations,    FXGroupBox, ConnectionOp
 // ---------------------------------------------------------------------------
 
 GNEConnectorFrame::CurrentLane::CurrentLane(GNEConnectorFrame* connectorFrameParent) :
-    FXGroupBox(connectorFrameParent->myContentFrame, "Lane", GUIDesignGroupBoxFrame) {
+    GNEGroupBoxModule(connectorFrameParent, TL("Lane")) {
     // create lane label
-    myCurrentLaneLabel = new FXLabel(this, "No lane selected", 0, GUIDesignLabelLeft);
+    myCurrentLaneLabel = new FXLabel(getCollapsableFrame(), TL("No lane selected"), 0, GUIDesignLabel(JUSTIFY_LEFT));
 }
 
 
@@ -76,9 +76,9 @@ GNEConnectorFrame::CurrentLane::~CurrentLane() {}
 void
 GNEConnectorFrame::CurrentLane::updateCurrentLaneLabel(const std::string& laneID) {
     if (laneID.empty()) {
-        myCurrentLaneLabel->setText("No lane selected");
+        myCurrentLaneLabel->setText(TL("No lane selected"));
     } else {
-        myCurrentLaneLabel->setText((std::string("Current Lane: ") + laneID).c_str());
+        myCurrentLaneLabel->setText((std::string(TL("Current Lane: ")) + laneID).c_str());
     }
 }
 
@@ -87,18 +87,18 @@ GNEConnectorFrame::CurrentLane::updateCurrentLaneLabel(const std::string& laneID
 // ---------------------------------------------------------------------------
 
 GNEConnectorFrame::ConnectionModifications::ConnectionModifications(GNEConnectorFrame* connectorFrameParent) :
-    FXGroupBox(connectorFrameParent->myContentFrame, "Modifications", GUIDesignGroupBoxFrame),
+    GNEGroupBoxModule(connectorFrameParent, TL("Modifications")),
     myConnectorFrameParent(connectorFrameParent) {
 
     // Create "Cancel" button
-    myCancelButton = new FXButton(this, "Cancel\t\tDiscard connection modifications (Esc)",
-                                  GUIIconSubSys::getIcon(GUIIcon::CANCEL), this, MID_CANCEL, GUIDesignButton);
+    myCancelButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Cancel"), "", TL("Discard connection modifications (Esc)"),
+                     GUIIconSubSys::getIcon(GUIIcon::CANCEL), this, MID_CANCEL, GUIDesignButton);
     // Create "OK" button
-    mySaveButton = new FXButton(this, "OK\t\tSave connection modifications (Enter)",
-                                GUIIconSubSys::getIcon(GUIIcon::ACCEPT), this, MID_OK, GUIDesignButton);
+    mySaveButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("OK"), "", TL("Save connection modifications (Enter)"),
+                   GUIIconSubSys::getIcon(GUIIcon::ACCEPT), this, MID_OK, GUIDesignButton);
 
     // Create checkbox for protect routes
-    myProtectRoutesCheckBox = new FXCheckButton(this, "Protect routes", this, MID_GNE_SET_ATTRIBUTE, GUIDesignCheckButton);
+    myProtectRoutesCheckBox = new FXCheckButton(getCollapsableFrame(), TL("Protect routes"), this, MID_GNE_SET_ATTRIBUTE, GUIDesignCheckButton);
 }
 
 
@@ -108,9 +108,9 @@ GNEConnectorFrame::ConnectionModifications::~ConnectionModifications() {}
 long
 GNEConnectorFrame::ConnectionModifications::onCmdCancelModifications(FXObject*, FXSelector, void*) {
     if (myConnectorFrameParent->myCurrentEditedLane != 0) {
-        myConnectorFrameParent->getViewNet()->getUndoList()->p_abort();
+        myConnectorFrameParent->getViewNet()->getUndoList()->abortAllChangeGroups();
         if (myConnectorFrameParent->myNumChanges) {
-            myConnectorFrameParent->getViewNet()->setStatusBarText("Changes reverted");
+            myConnectorFrameParent->getViewNet()->setStatusBarText(TL("Changes reverted"));
         }
         myConnectorFrameParent->cleanup();
         myConnectorFrameParent->getViewNet()->updateViewNet();
@@ -124,22 +124,24 @@ GNEConnectorFrame::ConnectionModifications::onCmdSaveModifications(FXObject*, FX
     if (myConnectorFrameParent->myCurrentEditedLane != 0) {
         // check if routes has to be protected
         if (myProtectRoutesCheckBox->isEnabled() && (myProtectRoutesCheckBox->getCheck() == TRUE)) {
-            for (const auto& i : myConnectorFrameParent->myCurrentEditedLane->getParentEdge()->getChildDemandElements()) {
-                if (!i->isDemandElementValid()) {
-                    FXMessageBox::warning(getApp(), MBOX_OK,
-                                          "Error saving connection operations", "%s",
-                                          ("Connection edition  cannot be saved because route '" + i->getID() + "' is broken.").c_str());
+            for (const auto& demandElement : myConnectorFrameParent->myCurrentEditedLane->getParentEdge()->getChildDemandElements()) {
+                if (demandElement->isDemandElementValid() != GNEDemandElement::Problem::OK) {
+                    // open warning dialog
+                    GNEWarningBasicDialog(myConnectorFrameParent->getViewNet()->getViewParent()->getGNEAppWindows(),
+                                          TL("Error saving connection operations"),
+                                          TLF("Connection edition cannot be saved because route '%s' is broken.", demandElement->getID()));
                     return 1;
                 }
             }
         }
         // finish route editing
-        myConnectorFrameParent->getViewNet()->getUndoList()->p_end();
+        myConnectorFrameParent->getViewNet()->getUndoList()->end();
         if (myConnectorFrameParent->myNumChanges) {
-            myConnectorFrameParent->getViewNet()->setStatusBarText("Changes accepted");
+            myConnectorFrameParent->getViewNet()->setStatusBarText(TL("Changes accepted"));
         }
         myConnectorFrameParent->cleanup();
-        myConnectorFrameParent->getViewNet()->updateViewNet();
+        // mark network for recomputing
+        myConnectorFrameParent->getViewNet()->getNet()->requireRecompute();
     }
     return 1;
 }
@@ -149,27 +151,27 @@ GNEConnectorFrame::ConnectionModifications::onCmdSaveModifications(FXObject*, FX
 // ---------------------------------------------------------------------------
 
 GNEConnectorFrame::ConnectionOperations::ConnectionOperations(GNEConnectorFrame* connectorFrameParent) :
-    FXGroupBox(connectorFrameParent->myContentFrame, "Operations", GUIDesignGroupBoxFrame),
+    GNEGroupBoxModule(connectorFrameParent, TL("Operations")),
     myConnectorFrameParent(connectorFrameParent) {
 
     // Create "Select Dead Ends" button
-    mySelectDeadEndsButton = new FXButton(this, "Select Dead Ends\t\tSelects all lanes that have no outgoing connection (clears previous selection)",
-                                          0, this, MID_GNE_CONNECTORFRAME_SELECTDEADENDS, GUIDesignButton);
+    mySelectDeadEndsButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Select Dead Ends"), "", TL("Selects all lanes that have no outgoing connection (clears previous selection)"),
+                             0, this, MID_GNE_CONNECTORFRAME_SELECTDEADENDS, GUIDesignButton);
     // Create "Select Dead Starts" button
-    mySelectDeadStartsButton = new FXButton(this, "Select Dead Starts\t\tSelects all lanes that have no incoming connection (clears previous selection)",
-                                            0, this, MID_GNE_CONNECTORFRAME_SELECTDEADSTARTS, GUIDesignButton);
+    mySelectDeadStartsButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Select Dead Starts"), "", TL("Selects all lanes that have no incoming connection (clears previous selection)"),
+                               0, this, MID_GNE_CONNECTORFRAME_SELECTDEADSTARTS, GUIDesignButton);
     // Create "Select Conflicts" button
-    mySelectConflictsButton = new FXButton(this, "Select Conflicts\t\tSelects all lanes with more than one incoming connection from the same edge (clears previous selection)",
-                                           0, this, MID_GNE_CONNECTORFRAME_SELECTCONFLICTS, GUIDesignButton);
+    mySelectConflictsButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Select Conflicts"), "", TL("Selects all lanes with more than one incoming connection from the same edge (clears previous selection)"),
+                              0, this, MID_GNE_CONNECTORFRAME_SELECTCONFLICTS, GUIDesignButton);
     // Create "Select Edges which may always pass" button
-    mySelectPassingButton = new FXButton(this, "Select Passing\t\tSelects all lanes with a connection that has has the 'pass' attribute set",
-                                         0, this, MID_GNE_CONNECTORFRAME_SELECTPASS, GUIDesignButton);
+    mySelectPassingButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Select Passing"), "", TL("Selects all lanes with a connection that has the 'pass' attribute set"),
+                            0, this, MID_GNE_CONNECTORFRAME_SELECTPASS, GUIDesignButton);
     // Create "Clear Selected" button
-    myClearSelectedButton = new FXButton(this, "Clear Selected\t\tClears all connections of all selected objects",
-                                         0, this, MID_CHOOSEN_CLEAR, GUIDesignButton);
+    myClearSelectedButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Clear Selected"), "", TL("Clears all connections of all selected objects"),
+                            0, this, MID_CHOOSEN_CLEAR, GUIDesignButton);
     // Create "Reset Selected" button
-    myResetSelectedButton = new FXButton(this, "Reset Selected\t\tRecomputes connections at all selected junctions",
-                                         0, this, MID_CHOOSEN_RESET, GUIDesignButton);
+    myResetSelectedButton = GUIDesigns::buildFXButton(getCollapsableFrame(), TL("Reset Selected"), "", TL("Recomputes connections at all selected junctions"),
+                            0, this, MID_CHOOSEN_RESET, GUIDesignButton);
 }
 
 
@@ -181,15 +183,15 @@ GNEConnectorFrame::ConnectionOperations::onCmdSelectDeadEnds(FXObject*, FXSelect
     // select all lanes that have no successor lane
     std::vector<GNEAttributeCarrier*> deadEnds;
     // every edge knows its outgoing connections so we can look at each edge in isolation
-    const std::vector<GNEEdge*> edges = myConnectorFrameParent->getViewNet()->getNet()->retrieveEdges();
-    for (auto i : edges) {
-        for (auto j : i->getLanes()) {
-            if (i->getNBEdge()->getConnectionsFromLane(j->getIndex()).size() == 0) {
-                deadEnds.push_back(j);
+    for (const auto& edge : myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getEdges()) {
+        for (const auto& lane : edge.second->getChildLanes()) {
+            if (edge.second->getNBEdge()->getConnectionsFromLane(lane->getIndex()).size() == 0) {
+                deadEnds.push_back(lane);
             }
         }
     }
     myConnectorFrameParent->getViewNet()->getViewParent()->getSelectorFrame()->handleIDs(deadEnds, GNESelectorFrame::ModificationMode::Operation::REPLACE);
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -200,25 +202,23 @@ GNEConnectorFrame::ConnectionOperations::onCmdSelectDeadStarts(FXObject*, FXSele
     std::set<GNEAttributeCarrier*> deadStarts;
     GNENet* net = myConnectorFrameParent->getViewNet()->getNet();
     // every edge knows only its outgoing connections so we look at whole junctions
-    const std::vector<GNEJunction*> junctions = myConnectorFrameParent->getViewNet()->getNet()->retrieveJunctions();
-    for (auto i : junctions) {
+    for (const auto& junction : myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getJunctions()) {
         // first collect all outgoing lanes
-        for (auto j : i->getNBNode()->getOutgoingEdges()) {
-            GNEEdge* edge = net->retrieveEdge(j->getID());
-            for (auto k : edge->getLanes()) {
-                deadStarts.insert(k);
+        for (const auto& outgoingEdge : junction.second->getGNEOutgoingEdges()) {
+            for (const auto& lane : outgoingEdge->getChildLanes()) {
+                deadStarts.insert(lane);
             }
         }
         // then remove all approached lanes
-        for (auto j : i->getNBNode()->getIncomingEdges()) {
-            GNEEdge* edge = net->retrieveEdge(j->getID());
-            for (auto k : edge->getNBEdge()->getConnections()) {
-                deadStarts.erase(net->retrieveEdge(k.toEdge->getID())->getLanes()[k.toLane]);
+        for (const auto& incomingEdge : junction.second->getGNEIncomingEdges()) {
+            for (const auto& connection : incomingEdge->getNBEdge()->getConnections()) {
+                deadStarts.erase(net->getAttributeCarriers()->retrieveEdge(connection.toEdge->getID())->getChildLanes()[connection.toLane]);
             }
         }
     }
     std::vector<GNEAttributeCarrier*> selectObjects(deadStarts.begin(), deadStarts.end());
     myConnectorFrameParent->getViewNet()->getViewParent()->getSelectorFrame()->handleIDs(selectObjects, GNESelectorFrame::ModificationMode::Operation::REPLACE);
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -227,22 +227,22 @@ long
 GNEConnectorFrame::ConnectionOperations::onCmdSelectConflicts(FXObject*, FXSelector, void*) {
     std::vector<GNEAttributeCarrier*> conflicts;
     // conflicts happen per edge so we can look at each edge in isolation
-    const std::vector<GNEEdge*> edges = myConnectorFrameParent->getViewNet()->getNet()->retrieveEdges();
-    for (auto i : edges) {
-        const EdgeVector destinations = i->getNBEdge()->getConnectedEdges();
-        for (auto j : destinations) {
-            GNEEdge* dest = myConnectorFrameParent->getViewNet()->getNet()->retrieveEdge(j->getID());
-            for (auto k : dest->getLanes()) {
-                const bool isConflicted = count_if(i->getNBEdge()->getConnections().begin(), i->getNBEdge()->getConnections().end(),
-                                                   NBEdge::connections_toedgelane_finder(j, (int)(k)->getIndex(), -1)) > 1;
+    for (const auto& edge : myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getEdges()) {
+        const EdgeVector destinations = edge.second->getNBEdge()->getConnectedEdges();
+        for (const auto& destination : destinations) {
+            GNEEdge* dest = myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->retrieveEdge(destination->getID());
+            for (const auto& lane : dest->getChildLanes()) {
+                const bool isConflicted = count_if(edge.second->getNBEdge()->getConnections().begin(), edge.second->getNBEdge()->getConnections().end(),
+                                                   NBEdge::connections_toedgelane_finder(destination, (int)lane->getIndex(), -1)) > 1;
                 if (isConflicted) {
-                    conflicts.push_back(k);
+                    conflicts.push_back(lane);
                 }
             }
         }
 
     }
     myConnectorFrameParent->getViewNet()->getViewParent()->getSelectorFrame()->handleIDs(conflicts, GNESelectorFrame::ModificationMode::Operation::REPLACE);
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -250,15 +250,15 @@ GNEConnectorFrame::ConnectionOperations::onCmdSelectConflicts(FXObject*, FXSelec
 long
 GNEConnectorFrame::ConnectionOperations::onCmdSelectPass(FXObject*, FXSelector, void*) {
     std::vector<GNEAttributeCarrier*> pass;
-    const std::vector<GNEEdge*> edges = myConnectorFrameParent->getViewNet()->getNet()->retrieveEdges();
-    for (auto i : edges) {
-        for (auto j : i->getNBEdge()->getConnections()) {
-            if (j.mayDefinitelyPass) {
-                pass.push_back(i->getLanes()[j.fromLane]);
+    for (const auto& edge : myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getEdges()) {
+        for (const auto& connection : edge.second->getNBEdge()->getConnections()) {
+            if (connection.mayDefinitelyPass) {
+                pass.push_back(edge.second->getChildLanes()[connection.fromLane]);
             }
         }
     }
     myConnectorFrameParent->getViewNet()->getViewParent()->getSelectorFrame()->handleIDs(pass, GNESelectorFrame::ModificationMode::Operation::REPLACE);
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -266,26 +266,27 @@ GNEConnectorFrame::ConnectionOperations::onCmdSelectPass(FXObject*, FXSelector,
 long
 GNEConnectorFrame::ConnectionOperations::onCmdClearSelectedConnections(FXObject*, FXSelector, void*) {
     myConnectorFrameParent->myConnectionModifications->onCmdCancelModifications(0, 0, 0);
-    myConnectorFrameParent->getViewNet()->getUndoList()->p_begin("clear connections from selected lanes, edges and " + toString(SUMO_TAG_JUNCTION) + "s");
+    myConnectorFrameParent->getViewNet()->getUndoList()->begin(GUIIcon::CONNECTION, TL("clear connections from selected lanes, edges and junctions"));
     // clear junction's connection
-    auto junctions = myConnectorFrameParent->getViewNet()->getNet()->retrieveJunctions(true);
-    for (auto i : junctions) {
-        i->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList()); // clear connections
-        i->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList(), GNEAttributeCarrier::FEATURE_MODIFIED); // prevent re-guessing
+    const auto selectedJunctions = myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getSelectedJunctions();
+    for (const auto& junction : selectedJunctions) {
+        junction->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList()); // clear connections
+        junction->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList(), GNEAttributeCarrier::FEATURE_MODIFIED); // prevent re-guessing
     }
     // clear edge's connection
-    auto edges = myConnectorFrameParent->getViewNet()->getNet()->retrieveEdges(true);
-    for (auto i : edges) {
-        for (auto j : i->getLanes()) {
-            myConnectorFrameParent->removeConnections(j);
+    const auto selectedEdges = myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getSelectedEdges();
+    for (const auto& edge : selectedEdges) {
+        for (const auto& lane : edge->getChildLanes()) {
+            myConnectorFrameParent->removeConnections(lane);
         }
     }
     // clear lane's connection
-    auto lanes = myConnectorFrameParent->getViewNet()->getNet()->retrieveLanes(true);
-    for (auto i : lanes) {
-        myConnectorFrameParent->removeConnections(dynamic_cast<GNELane*>(i));
+    const auto selectedLanes = myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getSelectedLanes();
+    for (const auto& lane : selectedLanes) {
+        myConnectorFrameParent->removeConnections(lane);
     }
-    myConnectorFrameParent->getViewNet()->getUndoList()->p_end();
+    myConnectorFrameParent->getViewNet()->getUndoList()->end();
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -293,12 +294,18 @@ GNEConnectorFrame::ConnectionOperations::onCmdClearSelectedConnections(FXObject*
 long
 GNEConnectorFrame::ConnectionOperations::onCmdResetSelectedConnections(FXObject*, FXSelector, void*) {
     myConnectorFrameParent->myConnectionModifications->onCmdCancelModifications(0, 0, 0);
-    myConnectorFrameParent->getViewNet()->getUndoList()->p_begin("reset connections from selected lanes");
-    auto junctions = myConnectorFrameParent->getViewNet()->getNet()->retrieveJunctions(true);
-    for (auto i : junctions) {
-        i->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList());
+    myConnectorFrameParent->getViewNet()->getUndoList()->begin(GUIIcon::CONNECTION, TL("reset connections from selected lanes"));
+    const auto selectedJunctions = myConnectorFrameParent->getViewNet()->getNet()->getAttributeCarriers()->getSelectedJunctions();
+    for (const auto& junction : selectedJunctions) {
+        junction->setLogicValid(false, myConnectorFrameParent->getViewNet()->getUndoList());
     }
-    myConnectorFrameParent->getViewNet()->getUndoList()->p_end();
+    myConnectorFrameParent->getViewNet()->getUndoList()->end();
+    if (selectedJunctions.size() > 0) {
+        auto viewNet = myConnectorFrameParent->getViewNet();
+        viewNet->getNet()->requireRecompute();
+        viewNet->getNet()->computeNetwork(viewNet->getViewParent()->getGNEAppWindows());
+    }
+    myConnectorFrameParent->getViewNet()->updateViewNet();
     return 1;
 }
 
@@ -307,10 +314,10 @@ GNEConnectorFrame::ConnectionOperations::onCmdResetSelectedConnections(FXObject*
 // ---------------------------------------------------------------------------
 
 GNEConnectorFrame::ConnectionSelection::ConnectionSelection(GNEConnectorFrame* connectorFrameParent) :
-    FXGroupBox(connectorFrameParent->myContentFrame, "Selection", GUIDesignGroupBoxFrame) {
-    // create Selection Hint
-    myHoldShiftLabel = new FXLabel(this, "Hold <SHIFT> while clicking\nto create unyielding\nconnections (pass=true).", 0, GUIDesignLabelFrameInformation);
-    myHoldControlLabel = new FXLabel(this, "Hold <CTRL> while clicking\nto create conflicting\nconnections (i.e. at zipper\nnodes or with incompatible\npermissions)", 0, GUIDesignLabelFrameInformation);
+    GNEGroupBoxModule(connectorFrameParent, TL("Selection")) {
+    // create label
+    new MFXDynamicLabel(getCollapsableFrame(), (std::string("- ") + TL("Hold <SHIFT> while clicking to create unyielding connections (pass=true).")).c_str(), 0, GUIDesignLabelFrameInformation);
+    new MFXDynamicLabel(getCollapsableFrame(), (std::string("- ") + TL("Hold <CTRL> while clicking to create conflicting connections (i.e. at zipper nodes or with incompatible permissions)")).c_str(), 0, GUIDesignLabelFrameInformation);
 }
 
 
@@ -320,56 +327,56 @@ GNEConnectorFrame::ConnectionSelection::~ConnectionSelection() {}
 // GNEConnectorFrame::ConnectionLegend - methods
 // ---------------------------------------------------------------------------
 
-GNEConnectorFrame::ConnectionLegend::ConnectionLegend(GNEConnectorFrame* connectorFrameParent) :
-    FXGroupBox(connectorFrameParent->myContentFrame, "Legend", GUIDesignGroupBoxFrame) {
+GNEConnectorFrame::Legend::Legend(GNEConnectorFrame* connectorFrameParent) :
+    GNEGroupBoxModule(connectorFrameParent, TL("Information")) {
 
     // create possible target label
-    FXLabel* possibleTargetLabel = new FXLabel(this, "Possible Target", 0, GUIDesignLabelLeft);
+    FXLabel* possibleTargetLabel = new FXLabel(getCollapsableFrame(), TL("Possible Target"), 0, GUIDesignLabel(JUSTIFY_LEFT));
     possibleTargetLabel->setBackColor(MFXUtils::getFXColor(connectorFrameParent->getViewNet()->getVisualisationSettings().candidateColorSettings.possible));
     possibleTargetLabel->setTextColor(MFXUtils::getFXColor(RGBColor::WHITE));
 
     // create source label
-    FXLabel* sourceLabel = new FXLabel(this, "Source lane", 0, GUIDesignLabelLeft);
+    FXLabel* sourceLabel = new FXLabel(getCollapsableFrame(), TL("Source lane"), 0, GUIDesignLabel(JUSTIFY_LEFT));
     sourceLabel->setBackColor(MFXUtils::getFXColor(connectorFrameParent->getViewNet()->getVisualisationSettings().candidateColorSettings.source));
 
     // create target label
-    FXLabel* targetLabel = new FXLabel(this, "Target lane", 0, GUIDesignLabelLeft);
+    FXLabel* targetLabel = new FXLabel(getCollapsableFrame(), TL("Target lane"), 0, GUIDesignLabel(JUSTIFY_LEFT));
     targetLabel->setBackColor(MFXUtils::getFXColor(connectorFrameParent->getViewNet()->getVisualisationSettings().candidateColorSettings.target));
 
     // create target (pass) label
-    FXLabel* targetPassLabel = new FXLabel(this, "Target (pass)", 0, GUIDesignLabelLeft);
+    FXLabel* targetPassLabel = new FXLabel(getCollapsableFrame(), TL("Target (pass)"), 0, GUIDesignLabel(JUSTIFY_LEFT));
     targetPassLabel->setBackColor(MFXUtils::getFXColor(connectorFrameParent->getViewNet()->getVisualisationSettings().candidateColorSettings.special));
 
     // create conflict label
-    FXLabel* conflictLabel = new FXLabel(this, "Conflict", 0, GUIDesignLabelLeft);
+    FXLabel* conflictLabel = new FXLabel(getCollapsableFrame(), TL("Conflict"), 0, GUIDesignLabel(JUSTIFY_LEFT));
     conflictLabel->setBackColor(MFXUtils::getFXColor(connectorFrameParent->getViewNet()->getVisualisationSettings().candidateColorSettings.conflict));
 }
 
 
-GNEConnectorFrame::ConnectionLegend::~ConnectionLegend() {}
+GNEConnectorFrame::Legend::~Legend() {}
 
 // ---------------------------------------------------------------------------
 // GNEConnectorFrame - methods
 // ---------------------------------------------------------------------------
 
-GNEConnectorFrame::GNEConnectorFrame(FXHorizontalFrame* horizontalFrameParent, GNEViewNet* viewNet):
-    GNEFrame(horizontalFrameParent, viewNet, "Edit Connections"),
+GNEConnectorFrame::GNEConnectorFrame(GNEViewParent* viewParent, GNEViewNet* viewNet):
+    GNEFrame(viewParent, viewNet, TL("Edit Connections")),
     myCurrentEditedLane(0),
     myNumChanges(0) {
-    // create current lane modul
+    // create current lane module
     myCurrentLane = new CurrentLane(this);
 
-    // create connection modifications modul
+    // create connection modifications module
     myConnectionModifications = new ConnectionModifications(this);
 
-    // create connection operations modul
+    // create connection operations module
     myConnectionOperations = new ConnectionOperations(this);
 
-    // create connection selection modul
+    // create connection selection module
     myConnectionSelection = new ConnectionSelection(this);
 
-    // create connection legend modul
-    myConnectionLegend = new ConnectionLegend(this);
+    // create connection legend module
+    myLegend = new Legend(this);
 }
 
 
@@ -377,13 +384,13 @@ GNEConnectorFrame::~GNEConnectorFrame() {}
 
 
 void
-GNEConnectorFrame::handleLaneClick(const GNEViewNetHelper::ObjectsUnderCursor& objectsUnderCursor) {
+GNEConnectorFrame::handleLaneClick(const GNEViewNetHelper::ViewObjectsSelector& viewObjects) {
     // get lane front
-    GNELane* clickedLane = objectsUnderCursor.getLaneFront();
+    GNELane* clickedLane = viewObjects.getLaneFrontNonLocked();
     // iterate over lanes
-    for (const auto& lane : objectsUnderCursor.getLanes()) {
+    for (const auto& lane : viewObjects.getLanes()) {
         // if parent edge of lane is front element, update clickedLane
-        if (lane->getParentEdge() == myViewNet->getFrontAttributeCarrier()) {
+        if (lane->getParentEdge()->isMarkedForDrawingFront()) {
             clickedLane = lane;
         }
     }
@@ -403,9 +410,9 @@ GNEConnectorFrame::removeConnections(GNELane* lane) {
     // select lane as current lane
     buildConnection(lane, false, false, true); // select as current lane
     // iterate over all potential targets
-    for (auto i : myPotentialTargets) {
-        // remove connections using the apropiate parameters in function "buildConnection"
-        buildConnection(i, false, false, false);
+    for (const auto& potentialTarget : myPotentialTargets) {
+        // remove connections using the appropiate parameters in function "buildConnection"
+        buildConnection(potentialTarget, false, false, false);
     }
     // save modifications
     myConnectionModifications->onCmdSaveModifications(0, 0, 0);
@@ -419,9 +426,9 @@ GNEConnectorFrame::buildConnection(GNELane* lane, const bool mayDefinitelyPass,
         myCurrentEditedLane->setSpecialColor(&myViewNet->getVisualisationSettings().candidateColorSettings.source);
         initTargets();
         myNumChanges = 0;
-        myViewNet->getUndoList()->p_begin("modify " + toString(SUMO_TAG_CONNECTION) + "s");
+        myViewNet->getUndoList()->begin(GUIIcon::CONNECTION, TL("modify connections"));
     } else if (myPotentialTargets.count(lane)
-               || (allowConflict && lane->getParentEdge()->getParentJunctions().front() == myCurrentEditedLane->getParentEdge()->getParentJunctions().back())) {
+               || (allowConflict && lane->getParentEdge()->getFromJunction() == myCurrentEditedLane->getParentEdge()->getToJunction())) {
         const int fromIndex = myCurrentEditedLane->getIndex();
         GNEEdge* srcEdge = myCurrentEditedLane->getParentEdge();
         GNEEdge* destEdge = lane->getParentEdge();
@@ -453,7 +460,7 @@ GNEConnectorFrame::buildConnection(GNELane* lane, const bool mayDefinitelyPass,
                     } else {
                         lane->setSpecialColor(&myViewNet->getVisualisationSettings().candidateColorSettings.target);
                     }
-                    srcEdge->getParentJunctions().back()->invalidateTLS(myViewNet->getUndoList(), NBConnection::InvalidConnection, newNBCon);
+                    srcEdge->getToJunction()->invalidateTLS(myViewNet->getUndoList(), NBConnection::InvalidConnection, newNBCon);
                 }
                 break;
             case LaneStatus::CONNECTED:
@@ -466,23 +473,26 @@ GNEConnectorFrame::buildConnection(GNELane* lane, const bool mayDefinitelyPass,
                 changed = true;
                 break;
             }
-            case LaneStatus::CONFLICTED:
+            case LaneStatus::CONFLICTED: {
                 SVCPermissions fromPermissions = srcEdge->getNBEdge()->getPermissions(fromIndex);
                 SVCPermissions toPermissions = destEdge->getNBEdge()->getPermissions(lane->getIndex());
                 if ((fromPermissions & toPermissions) == SVC_PEDESTRIAN) {
-                    myViewNet->setStatusBarText("Pedestrian connections are generated automatically");
+                    myViewNet->setStatusBarText(TL("Pedestrian connections are generated automatically"));
                 } else if ((fromPermissions & toPermissions) == 0) {
-                    myViewNet->setStatusBarText("Incompatible vehicle class permissions");
+                    myViewNet->setStatusBarText(TL("Incompatible vehicle class permissions"));
                 } else {
-                    myViewNet->setStatusBarText("Another lane from the same edge already connects to that lane");
+                    myViewNet->setStatusBarText(TL("Another lane from the same edge already connects to that lane"));
                 }
                 break;
+            }
+            default:
+                break;
         }
         if (changed) {
             myNumChanges += 1;
         }
     } else {
-        myViewNet->setStatusBarText("Invalid target for " + toString(SUMO_TAG_CONNECTION));
+        myViewNet->setStatusBarText(TL("Invalid target for connection"));
     }
     myCurrentLane->updateCurrentLaneLabel(myCurrentEditedLane->getID());
 }
@@ -491,11 +501,11 @@ GNEConnectorFrame::buildConnection(GNELane* lane, const bool mayDefinitelyPass,
 void
 GNEConnectorFrame::initTargets() {
     // gather potential targets
-    NBNode* nbn = myCurrentEditedLane->getParentEdge()->getParentJunctions().back()->getNBNode();
-    // get potencial targets
+    NBNode* nbn = myCurrentEditedLane->getParentEdge()->getToJunction()->getNBNode();
+    // get potential targets
     for (const auto& NBEEdge : nbn->getOutgoingEdges()) {
-        GNEEdge* edge = myViewNet->getNet()->retrieveEdge(NBEEdge->getID());
-        for (const auto& lane : edge->getLanes()) {
+        GNEEdge* edge = myViewNet->getNet()->getAttributeCarriers()->retrieveEdge(NBEEdge->getID());
+        for (const auto& lane : edge->getChildLanes()) {
             myPotentialTargets.insert(lane);
         }
     }
