@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -51,27 +51,19 @@
 #include <utils/xml/XMLSubSys.h>
 #include <utils/iodevices/OutputDevice.h>
 
-// ===========================================================================
-// method declaration
-// ===========================================================================
-
-void fillOptions();
-bool checkOptions();
-NGNet* buildNetwork(NBNetBuilder& nb);
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-
 void
 fillOptions() {
     OptionsCont& oc = OptionsCont::getOptions();
-    oc.addCallExample("-c <CONFIGURATION>", "create net from given configuration");
-    oc.addCallExample("--grid [grid-network options] -o <OUTPUTFILE>", "create grid net");
-    oc.addCallExample("--spider [spider-network options] -o <OUTPUTFILE>", "create spider net");
-    oc.addCallExample("--rand [random-network options] -o <OUTPUTFILE>", "create random net");
+    oc.addCallExample("-c <CONFIGURATION>", TL("create net from given configuration"));
+    oc.addCallExample("--grid [grid-network options] -o <OUTPUTFILE>", TL("create grid net"));
+    oc.addCallExample("--spider [spider-network options] -o <OUTPUTFILE>", TL("create spider net"));
+    oc.addCallExample("--rand [random-network options] -o <OUTPUTFILE>", TL("create random net"));
 
-    oc.setAdditionalHelpMessage(" Either \"--grid\", \"--spider\" or \"--rand\" must be supplied.\n  In dependance to these switches other options are used.");
+    oc.setAdditionalHelpMessage(" Either \"--grid\", \"--spider\" or \"--rand\" must be supplied.\n  In dependence to these switches other options are used.");
 
     // insert options sub-topics
     SystemFrame::addConfigurationOptions(oc); // this subtopic is filled here, too
@@ -91,21 +83,28 @@ fillOptions() {
     SystemFrame::addReportOptions(oc); // this subtopic is filled here, too
 
     NGFrame::fillOptions();
-    NBFrame::fillOptions(true);
-    NWFrame::fillOptions(true);
+    NBFrame::fillOptions(oc, true);
+    NWFrame::fillOptions(oc, true);
     oc.doRegister("default-junction-type", 'j', new Option_String());
     oc.addSynonyme("default-junction-type", "junctions");
-    oc.addDescription("default-junction-type", "Building Defaults", "[traffic_light|priority|right_before_left|traffic_light_right_on_red|priority_stop|allway_stop|...] Determines junction type (see wiki/Networks/PlainXML#Node_types)");
-    RandHelper::insertRandOptions();
+    oc.addDescription("default-junction-type", "Building Defaults", TL("[traffic_light|priority|right_before_left|left_before_right|traffic_light_right_on_red|priority_stop|allway_stop|...] Determines junction type (see wiki/Networks/PlainXML#Node_types)"));
+    RandHelper::insertRandOptions(oc);
+
+    oc.doRegister("tls.discard-simple", new Option_Bool(false));
+    oc.addDescription("tls.discard-simple", "TLS Building", "Does not instantiate traffic lights at geometry-like nodes");
+
+    oc.doRegister("railway.signal.permit-unsignalized", new Option_StringVector({"tram", "cable_car"}));
+    oc.addDescription("railway.signal.permit-unsignalized", "TLS Building", TL("List rail classes that may run without rail signals"));
 }
 
 
 bool
 checkOptions() {
+    OptionsCont& oc = OptionsCont::getOptions();
     bool ok = NGFrame::checkOptions();
-    ok &= NBFrame::checkOptions();
-    ok &= NWFrame::checkOptions();
-    ok &= SystemFrame::checkOptions();
+    ok &= NBFrame::checkOptions(oc);
+    ok &= NWFrame::checkOptions(oc);
+    ok &= SystemFrame::checkOptions(oc);
     return ok;
 }
 
@@ -121,14 +120,14 @@ buildNetwork(NBNetBuilder& nb) {
         // check values
         bool hadError = false;
         if (oc.getInt("spider.arm-number") < 3) {
-            WRITE_ERROR("Spider networks need at least 3 arms.");
+            WRITE_ERROR(TL("Spider networks need at least 3 arms."));
             hadError = true;
         }
-        if (oc.getInt("spider.arm-number") > 4 && oc.isDefault("spider.omit-center")) {
-            WRITE_WARNING("Spider networks with many arms should use option ommit-center");
+        if (oc.getInt("spider.arm-number") > 4 && !oc.getBool("spider.omit-center")) {
+            WRITE_WARNING(TL("Spider networks with many arms should use option spider.omit-center"));
         }
         if (oc.getInt("spider.circle-number") < 1) {
-            WRITE_ERROR("Spider networks need at least one circle.");
+            WRITE_ERROR(TL("Spider networks need at least one circle."));
             hadError = true;
         }
         minLength = MAX2(minLength, laneWidth * 2 * MAX2(oc.getInt("spider.arm-number"), 3) / (2 * M_PI));
@@ -136,7 +135,7 @@ buildNetwork(NBNetBuilder& nb) {
             WRITE_ERROR("The radius of spider networks must be at least "  + toString(POSITION_EPS));
             hadError = true;
         } else if (oc.getFloat("spider.space-radius") < minLength) {
-            WRITE_WARNING("The radius of spider networks should be at least " + toString(minLength) + " for the given lanenumber, lanewidth and junction radius");
+            WRITE_WARNINGF(TL("The radius of spider networks should be at least % for the given lanenumber, lanewidth and junction radius"), toString(minLength));
         }
         if (hadError) {
             throw ProcessError();
@@ -144,7 +143,8 @@ buildNetwork(NBNetBuilder& nb) {
         // build if everything's ok
         NGNet* net = new NGNet(nb);
         net->createSpiderWeb(oc.getInt("spider.arm-number"), oc.getInt("spider.circle-number"),
-                             oc.getFloat("spider.space-radius"), !oc.getBool("spider.omit-center"));
+                             oc.getFloat("spider.space-radius"), !oc.getBool("spider.omit-center"),
+                             oc.getFloat("spider.attach-length"));
         return net;
     }
     // grid-net
@@ -154,7 +154,8 @@ buildNetwork(NBNetBuilder& nb) {
         int yNo = oc.getInt("grid.y-number");
         double xLength = oc.getFloat("grid.x-length");
         double yLength = oc.getFloat("grid.y-length");
-        double attachLength = oc.getFloat("grid.attach-length");
+        double xAttachLength = oc.getFloat("grid.x-attach-length");
+        double yAttachLength = oc.getFloat("grid.y-attach-length");
         if (oc.isDefault("grid.x-number") && !oc.isDefault("grid.number")) {
             xNo = oc.getInt("grid.number");
         }
@@ -167,10 +168,16 @@ buildNetwork(NBNetBuilder& nb) {
         if (oc.isDefault("grid.y-length") && !oc.isDefault("grid.length")) {
             yLength = oc.getFloat("grid.length");
         }
+        if (oc.isDefault("grid.x-attach-length") && !oc.isDefault("grid.attach-length")) {
+            xAttachLength = oc.getFloat("grid.attach-length");
+        }
+        if (oc.isDefault("grid.y-attach-length") && !oc.isDefault("grid.attach-length")) {
+            yAttachLength = oc.getFloat("grid.attach-length");
+        }
         // check values
         bool hadError = false;
-        if (xNo < 1 || yNo < 1 || (attachLength == 0 && (xNo < 2 && yNo < 2))) {
-            WRITE_ERROR("The number of nodes must be positive and at least 2 in one direction if there are no attachments.");
+        if (xNo < 1 || yNo < 1 || (xAttachLength == 0 && yAttachLength == 0 && (xNo < 2 && yNo < 2))) {
+            WRITE_ERROR(TL("The number of nodes must be positive and at least 2 in one direction if there are no attachments."));
             hadError = true;
         }
         const double minAttachLength = minLength / 2 + POSITION_EPS / 2;
@@ -178,20 +185,25 @@ buildNetwork(NBNetBuilder& nb) {
             WRITE_ERROR("The distance between nodes must be at least " + toString(POSITION_EPS));
             hadError = true;
         } else if (xLength < minLength || yLength < minLength) {
-            WRITE_WARNING("The distance between nodes should be at least " + toString(minLength) + " for the given lanenumber, lanewidth and junction radius");
+            WRITE_WARNINGF(TL("The distance between nodes should be at least % for the given lanenumber, lanewidth and junction radius"), toString(minLength));
         }
-        if (attachLength != 0.0 && attachLength < POSITION_EPS) {
+        if (xAttachLength != 0.0 && xAttachLength < POSITION_EPS) {
             WRITE_ERROR("The length of attached streets must be at least " + toString(POSITION_EPS));
             hadError = true;
-        } else if (attachLength != 0.0 && attachLength < minAttachLength) {
-            WRITE_WARNING("The length of attached streets should be at least " + toString(minAttachLength) + " for the given lanenumber, lanewidth and junction radius");
+        } else if (xAttachLength != 0.0 && xAttachLength < minAttachLength) {
+            WRITE_WARNINGF(TL("The length of attached streets should be at least % for the given lanenumber, lanewidth and junction radius"), toString(minAttachLength));
+        } else if (yAttachLength != 0.0 && yAttachLength < POSITION_EPS) {
+            WRITE_ERROR("The length of attached streets must be at least " + toString(POSITION_EPS));
+            hadError = true;
+        } else if (yAttachLength != 0.0 && yAttachLength < minAttachLength) {
+            WRITE_WARNINGF(TL("The length of attached streets should be at least % for the given lanenumber, lanewidth and junction radius"), toString(minAttachLength));
         }
         if (hadError) {
             throw ProcessError();
         }
         // build if everything's ok
         NGNet* net = new NGNet(nb);
-        net->createChequerBoard(xNo, yNo, xLength, yLength, attachLength);
+        net->createChequerBoard(xNo, yNo, xLength, yLength, xAttachLength, yAttachLength);
         return net;
     }
     // random net
@@ -219,9 +231,8 @@ buildNetwork(NBNetBuilder& nb) {
 int
 main(int argc, char** argv) {
     OptionsCont& oc = OptionsCont::getOptions();
-    // give some application descriptions
-    oc.setApplicationDescription("Synthetic network generator for the microscopic, multi-modal traffic simulation SUMO.");
-    oc.setApplicationName("netgenerate", "Eclipse SUMO netgenerate Version " VERSION_STRING);
+    oc.setApplicationDescription(TL("Synthetic network generator for the microscopic, multi-modal traffic simulation SUMO."));
+    oc.setApplicationName("netgenerate", "Eclipse SUMO netgenerate " VERSION_STRING);
     int ret = 0;
     try {
         // initialise the application system (messaging, xml, options)
@@ -245,8 +256,8 @@ main(int argc, char** argv) {
         NBNetBuilder nb;
         nb.applyOptions(oc);
         if (oc.isSet("type-files")) {
-            NIXMLTypesHandler* handler = new NIXMLTypesHandler(nb.getTypeCont());
-            NITypeLoader::load(handler, oc.getStringVector("type-files"), "types");
+            NIXMLTypesHandler handler(nb.getTypeCont());
+            NITypeLoader::load(handler, oc.getStringVector("type-files"), toString(SUMO_TAG_TYPES));
         }
         // build the netgen-network description
         NGNet* net = buildNetwork(nb);
@@ -256,9 +267,16 @@ main(int argc, char** argv) {
         net->toNB();
         delete net;
         // report generated structures
-        WRITE_MESSAGE(" Generation done;");
-        WRITE_MESSAGE("   " + toString<int>(nb.getNodeCont().size()) + " nodes generated.");
-        WRITE_MESSAGE("   " + toString<int>(nb.getEdgeCont().size()) + " edges generated.");
+        WRITE_MESSAGE(TL(" Generation done;"));
+        WRITE_MESSAGEF(TL("   % nodes generated."), toString<int>(nb.getNodeCont().size()));
+        WRITE_MESSAGEF(TL("   % edges generated."), toString<int>(nb.getEdgeCont().size()));
+        if (oc.getBool("tls.discard-simple")) {
+            nb.getNodeCont().discardTrafficLights(nb.getTLLogicCont(), true);
+            int removed = nb.getTLLogicCont().getNumExtracted();
+            if (removed > 0) {
+                WRITE_MESSAGEF(TL(" Removed % traffic lights at geometry-like nodes"), toString(removed));
+            }
+        }
         nb.compute(oc);
         nb.getNodeCont().printBuiltNodesStatistics();
         NWFrame::writeNetwork(oc, nb);
