@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -44,6 +44,10 @@
 #include "NIImporter_ArcView.h"
 
 #ifdef HAVE_GDAL
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4435 5219 5220)
+#endif
 #if __GNUC__ > 3
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wpedantic"
@@ -52,6 +56,9 @@
 #if __GNUC__ > 3
 #pragma GCC diagnostic pop
 #endif
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
 #endif
 
 
@@ -67,27 +74,28 @@ NIImporter_ArcView::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
         return;
     }
     // check whether the correct set of entries is given
-    //  and compute both file names
-    std::string dbf_file = oc.getString("shapefile-prefix") + ".dbf";
-    std::string shp_file = oc.getString("shapefile-prefix") + ".shp";
-    std::string shx_file = oc.getString("shapefile-prefix") + ".shx";
-    // check whether the files do exist
-    if (!FileHelpers::isReadable(dbf_file)) {
-        WRITE_ERROR("File not accessible: " + dbf_file);
-    }
-    if (!FileHelpers::isReadable(shp_file)) {
-        WRITE_ERROR("File not accessible: " + shp_file);
-    }
-    if (!FileHelpers::isReadable(shx_file)) {
-        WRITE_ERROR("File not accessible: " + shx_file);
+    //  and compute all file names
+    const std::string dbf_file = oc.getString("shapefile-prefix") + ".dbf";
+    const std::string shp_file = oc.getString("shapefile-prefix") + ".shp";
+    const std::string shx_file = oc.getString("shapefile-prefix") + ".shx";
+    if (!StringUtils::startsWith(shp_file, "/vsi")) {
+        // if we are not using a virtual file system, check whether the files do exist
+        if (!FileHelpers::isReadable(dbf_file)) {
+            WRITE_ERROR("File not accessible: " + dbf_file);
+        }
+        if (!FileHelpers::isReadable(shp_file)) {
+            WRITE_ERROR("File not accessible: " + shp_file);
+        }
+        if (!FileHelpers::isReadable(shx_file)) {
+            WRITE_ERROR("File not accessible: " + shx_file);
+        }
     }
     if (MsgHandler::getErrorInstance()->wasInformed()) {
         return;
     }
     // load the arcview files
-    NIImporter_ArcView loader(oc,
-                              nb.getNodeCont(), nb.getEdgeCont(), nb.getTypeCont(),
-                              dbf_file, shp_file, oc.getBool("speed-in-kmh"));
+    NIImporter_ArcView loader(oc, nb.getNodeCont(), nb.getEdgeCont(), nb.getTypeCont(),
+                              shp_file, oc.getBool("speed-in-kmh"));
     loader.load();
 }
 
@@ -100,7 +108,6 @@ NIImporter_ArcView::NIImporter_ArcView(const OptionsCont& oc,
                                        NBNodeCont& nc,
                                        NBEdgeCont& ec,
                                        NBTypeCont& tc,
-                                       const std::string& dbf_name,
                                        const std::string& shp_name,
                                        bool speedInKMH)
     : myOptions(oc), mySHPName(shp_name),
@@ -109,7 +116,6 @@ NIImporter_ArcView::NIImporter_ArcView(const OptionsCont& oc,
       mySpeedInKMH(speedInKMH),
       myRunningEdgeID(0),
       myRunningNodeID(0) {
-    UNUSED_PARAMETER(dbf_name);
 }
 
 
@@ -128,34 +134,35 @@ NIImporter_ArcView::load() {
     GDALDataset* poDS = (GDALDataset*)GDALOpenEx(mySHPName.c_str(), GDAL_OF_VECTOR | GA_ReadOnly, NULL, NULL, NULL);
 #endif
     if (poDS == NULL) {
-        WRITE_ERROR("Could not open shape description '" + mySHPName + "'.");
+        WRITE_ERRORF(TL("Could not open shape description '%'."), mySHPName);
         return;
     }
 
     // begin file parsing
-    OGRLayer*  poLayer = poDS->GetLayer(0);
+    OGRLayer* poLayer = poDS->GetLayer(0);
     poLayer->ResetReading();
 
     // build coordinate transformation
+#if GDAL_VERSION_MAJOR < 3
     OGRSpatialReference* origTransf = poLayer->GetSpatialRef();
+#else
+    const OGRSpatialReference* origTransf = poLayer->GetSpatialRef();
+#endif
     OGRSpatialReference destTransf;
     // use wgs84 as destination
     destTransf.SetWellKnownGeogCS("WGS84");
 #if GDAL_VERSION_MAJOR > 2
-    if (myOptions.getBool("shapefile.traditional-axis-mapping")) {
+    if (myOptions.getBool("shapefile.traditional-axis-mapping") || origTransf != nullptr) {
         destTransf.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     }
 #endif
-    OGRCoordinateTransformation* poCT = OGRCreateCoordinateTransformation(origTransf, &destTransf);
+    OGRCoordinateTransformation* poCT = origTransf == nullptr ? nullptr : OGRCreateCoordinateTransformation(origTransf, &destTransf);
     if (poCT == nullptr) {
-        if (myOptions.isSet("shapefile.guess-projection")) {
+        if (myOptions.getBool("shapefile.guess-projection")) {
             OGRSpatialReference origTransf2;
             origTransf2.SetWellKnownGeogCS("WGS84");
             poCT = OGRCreateCoordinateTransformation(&origTransf2, &destTransf);
         }
-        if (poCT == nullptr) {
-            WRITE_WARNING("Could not create geocoordinates converter; check whether proj.4 is installed.");
-        }
     }
 
     const bool saveOrigIDs = OptionsCont::getOptions().getBool("output.original-names");
@@ -167,6 +174,7 @@ NIImporter_ArcView::load() {
 
     int featureIndex = 0;
     bool warnNotUnique = true;
+    bool warnMissingProjection = true;
     std::string idPrefix = ""; // prefix for non-unique street-id values
     std::map<std::string, int> idIndex; // running index to make street-id unique
     while ((poFeature = poLayer->GetNextFeature()) != NULL) {
@@ -179,7 +187,7 @@ NIImporter_ArcView::load() {
         std::string from_node;
         std::string to_node;
         if (!getStringEntry(poFeature, "shapefile.street-id", "LINK_ID", true, id)) {
-            WRITE_ERROR("Needed field '" + id + "' (street-id) is missing.");
+            WRITE_ERRORF(TL("Needed field '%' (street-id) is missing."), id);
             id = "";
         }
         if (id == "") {
@@ -190,11 +198,11 @@ NIImporter_ArcView::load() {
         name = StringUtils::replace(name, "&", "&amp;");
 
         if (!getStringEntry(poFeature, "shapefile.from-id", "REF_IN_ID", true, from_node)) {
-            WRITE_ERROR("Needed field '" + from_node + "' (from node id) is missing.");
+            WRITE_ERRORF(TL("Needed field '%' (from node id) is missing."), from_node);
             from_node = "";
         }
         if (!getStringEntry(poFeature, "shapefile.to-id", "NREF_IN_ID", true, to_node)) {
-            WRITE_ERROR("Needed field '" + to_node + "' (to node id) is missing.");
+            WRITE_ERRORF(TL("Needed field '%' (to node id) is missing."), to_node);
             to_node = "";
         }
 
@@ -210,13 +218,14 @@ NIImporter_ArcView::load() {
             type = poFeature->GetFieldAsString("ST_TYP_AFT");
         }
         if ((type != "" || myOptions.isSet("shapefile.type-id")) && !myTypeCont.knows(type)) {
-            WRITE_WARNINGF("Unknown type '%' for edge '%'", type, id);
+            WRITE_WARNINGF(TL("Unknown type '%' for edge '%'"), type, id);
         }
-        double width = myTypeCont.getEdgeTypeWidth(type);
         bool oneway = myTypeCont.knows(type) ? myTypeCont.getEdgeTypeIsOneWay(type) : false;
         double speed = getSpeed(*poFeature, id);
         int nolanes = getLaneNo(*poFeature, id, speed);
         int priority = getPriority(*poFeature, id);
+        double width = getLaneWidth(*poFeature, id, nolanes);
+        double length = getLength(*poFeature, id);
         if (nolanes <= 0 || speed <= 0) {
             if (myOptions.getBool("shapefile.use-defaults-on-failure")) {
                 nolanes = nolanes <= 0 ? myTypeCont.getEdgeTypeNumLanes(type) : nolanes;
@@ -224,14 +233,14 @@ NIImporter_ArcView::load() {
             } else {
                 const std::string lanesField = myOptions.isSet("shapefile.laneNumber") ? myOptions.getString("shapefile.laneNumber") : "nolanes";
                 const std::string speedField = myOptions.isSet("shapefile.speed") ? myOptions.getString("shapefile.speed") : "speed";
-                WRITE_ERROR("Required field '" + lanesField + "' or '" + speedField + "' is missing (add fields or set option --shapefile.use-defaults-on-failure).");
+                WRITE_ERRORF(TL("Required field '%' or '%' is missing (add fields or set option --shapefile.use-defaults-on-failure)."), lanesField, speedField);
                 WRITE_ERROR("Available fields: " + toString(getFieldNames(poFeature)));
                 OGRFeature::DestroyFeature(poFeature);
                 return;
             }
         }
         if (mySpeedInKMH) {
-            speed = speed / (double) 3.6;
+            speed /= 3.6;
         }
 
 
@@ -240,10 +249,25 @@ NIImporter_ArcView::load() {
         OGRwkbGeometryType gtype = poGeometry->getGeometryType();
         if (gtype != wkbLineString && gtype != wkbLineString25D) {
             OGRFeature::DestroyFeature(poFeature);
-            WRITE_ERROR("Road geometry must be of type 'linestring' or 'linestring25D' (found '" + toString(gtype) + "')");
+            WRITE_ERRORF(TL("Road geometry must be of type 'linestring' or 'linestring25D' (found '%')"), toString(gtype));
             return;
         }
         OGRLineString* cgeom = (OGRLineString*) poGeometry;
+        if (poCT == nullptr && warnMissingProjection) {
+            int outOfRange = 0;
+            for (int j = 0; j < cgeom->getNumPoints(); j++) {
+                if (fabs(cgeom->getX(j)) > 180 || fabs(cgeom->getY(j)) > 90) {
+                    outOfRange++;
+                }
+            }
+            if (2 * outOfRange > cgeom->getNumPoints()) {
+                WRITE_WARNING(TL("No coordinate system found and coordinates look already projected."));
+                GeoConvHelper::init("!", GeoConvHelper::getProcessing().getOffset(), GeoConvHelper::getProcessing().getOrigBoundary(), GeoConvHelper::getProcessing().getConvBoundary());
+            } else {
+                WRITE_WARNING(TL("Could not find geo coordinate system, assuming WGS84."));
+            }
+            warnMissingProjection = false;
+        }
         if (poCT != nullptr) {
             // try transform to wgs84
             cgeom->transform(poCT);
@@ -251,22 +275,25 @@ NIImporter_ArcView::load() {
 
         PositionVector shape;
         for (int j = 0; j < cgeom->getNumPoints(); j++) {
-            Position pos((double) cgeom->getX(j), (double) cgeom->getY(j), (double) cgeom->getZ(j));
+            Position pos(cgeom->getX(j), cgeom->getY(j), cgeom->getZ(j));
             if (!NBNetBuilder::transformCoordinate(pos)) {
-                WRITE_WARNINGF("Unable to project coordinates for edge '%'.", id);
+                WRITE_WARNINGF(TL("Unable to project coordinates for edge '%'."), id);
             }
             shape.push_back_noDoublePos(pos);
         }
 
         // build from-node
         NBNode* from = myNodeCont.retrieve(from_node);
-        if (from == 0) {
+        if (from == nullptr) {
             Position from_pos = shape[0];
-            from = myNodeCont.retrieve(from_pos, nodeJoinDist);
-            if (from == 0) {
+            std::vector<NBNode*> cands = myNodeCont.retrieveByPos(from_pos, nodeJoinDist);
+            if (!cands.empty()) {
+                from = cands.front();
+            }
+            if (from == nullptr) {
                 from = new NBNode(from_node, from_pos);
                 if (!myNodeCont.insert(from)) {
-                    WRITE_ERROR("Node '" + from_node + "' could not be added");
+                    WRITE_ERRORF(TL("Node '%' could not be added"), from_node);
                     delete from;
                     continue;
                 }
@@ -274,13 +301,16 @@ NIImporter_ArcView::load() {
         }
         // build to-node
         NBNode* to = myNodeCont.retrieve(to_node);
-        if (to == 0) {
+        if (to == nullptr) {
             Position to_pos = shape[-1];
-            to = myNodeCont.retrieve(to_pos, nodeJoinDist);
-            if (to == 0) {
+            std::vector<NBNode*> cands = myNodeCont.retrieveByPos(to_pos, nodeJoinDist);
+            if (!cands.empty()) {
+                to = cands.front();
+            }
+            if (to == nullptr) {
                 to = new NBNode(to_node, to_pos);
                 if (!myNodeCont.insert(to)) {
-                    WRITE_ERROR("Node '" + to_node + "' could not be added");
+                    WRITE_ERRORF(TL("Node '%' could not be added"), to_node);
                     delete to;
                     continue;
                 }
@@ -288,7 +318,7 @@ NIImporter_ArcView::load() {
         }
 
         if (from == to) {
-            WRITE_WARNINGF("Edge '%' connects identical nodes, skipping.", id);
+            WRITE_WARNINGF(TL("Edge '%' connects identical nodes, skipping."), id);
             continue;
         }
 
@@ -300,13 +330,12 @@ NIImporter_ArcView::load() {
         }
         const std::string origID = saveOrigIDs ? id : "";
         // check for duplicate ids
-        NBEdge* existing = myEdgeCont.retrieve(id);
-        NBEdge* existingReverse = myEdgeCont.retrieve("-" + id);
+        NBEdge* const existing = myEdgeCont.retrieve(id);
+        NBEdge* const existingReverse = myEdgeCont.retrieve("-" + id);
         if (existing != nullptr || existingReverse != nullptr) {
-            std::string duplicateID = existing != nullptr ? id : existingReverse->getID();
-            if ((existing != 0 && existing->getGeometry() == shape)
-                    || (existingReverse != 0 && existingReverse->getGeometry() == shape.reverse())) {
-                WRITE_ERROR("Edge '" + duplicateID + " is not unique");
+            if ((existing != nullptr && existing->getGeometry() == shape)
+                    || (existingReverse != nullptr && existingReverse->getGeometry() == shape.reverse())) {
+                WRITE_ERRORF(TL("Edge '%' is not unique."), (existing != nullptr ? id : existingReverse->getID()));
             } else {
                 if (idIndex.count(id) == 0) {
                     idIndex[id] = 0;
@@ -315,7 +344,7 @@ NIImporter_ArcView::load() {
                 idPrefix = id;
                 id += "#" + toString(idIndex[id]);
                 if (warnNotUnique) {
-                    WRITE_WARNING("street-id '" + idPrefix + "' is not unique. Renaming subsequent edge to '" + id + "'");
+                    WRITE_WARNINGF(TL("Edge '%' is not unique. Renaming subsequent edge to '%'."), idPrefix, id);
                     warnNotUnique = false;
                 }
             }
@@ -324,26 +353,34 @@ NIImporter_ArcView::load() {
         if (dir == "B" || dir == "F" || dir == "" || myOptions.getBool("shapefile.all-bidirectional")) {
             if (myEdgeCont.retrieve(id) == 0) {
                 LaneSpreadFunction spread = dir == "B" || dir == "FALSE" ? LaneSpreadFunction::RIGHT : LaneSpreadFunction::CENTER;
-                NBEdge* edge = new NBEdge(id, from, to, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape, spread, name, origID);
+                if (spread == LaneSpreadFunction::RIGHT && OptionsCont::getOptions().getString("default.spreadtype") == toString(LaneSpreadFunction::ROADCENTER)) {
+                    spread = LaneSpreadFunction::ROADCENTER;
+                }
+                NBEdge* edge = new NBEdge(id, from, to, type, speed, NBEdge::UNSPECIFIED_FRICTION, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape, spread, name, origID);
                 edge->setPermissions(myTypeCont.getEdgeTypePermissions(type));
+                edge->setLoadedLength(length);
                 myEdgeCont.insert(edge);
                 checkSpread(edge);
                 addParams(edge, poFeature, params);
             } else {
-                WRITE_ERROR("Could not create edge '" + id + "'. An edge with the same id already exists");
+                WRITE_ERRORF(TL("Could not create edge '%'. An edge with the same id already exists."), id);
             }
         }
         // add negative direction if wanted
         if ((dir == "B" || dir == "T" || myOptions.getBool("shapefile.all-bidirectional")) && !oneway) {
             if (myEdgeCont.retrieve("-" + id) == 0) {
                 LaneSpreadFunction spread = dir == "B" || dir == "FALSE" ? LaneSpreadFunction::RIGHT : LaneSpreadFunction::CENTER;
-                NBEdge* edge = new NBEdge("-" + id, to, from, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(), spread, name, origID);
+                if (spread == LaneSpreadFunction::RIGHT && OptionsCont::getOptions().getString("default.spreadtype") == toString(LaneSpreadFunction::ROADCENTER)) {
+                    spread = LaneSpreadFunction::ROADCENTER;
+                }
+                NBEdge* edge = new NBEdge("-" + id, to, from, type, speed, NBEdge::UNSPECIFIED_FRICTION, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(), spread, name, origID);
                 edge->setPermissions(myTypeCont.getEdgeTypePermissions(type));
+                edge->setLoadedLength(length);
                 myEdgeCont.insert(edge);
                 checkSpread(edge);
                 addParams(edge, poFeature, params);
             } else {
-                WRITE_ERROR("Could not create edge '-" + id + "'. An edge with the same id already exists");
+                WRITE_ERRORF(TL("Could not create edge '-%'. An edge with the same id already exists."), id);
             }
         }
         //
@@ -357,7 +394,7 @@ NIImporter_ArcView::load() {
 #endif
     PROGRESS_DONE_MESSAGE();
 #else
-    WRITE_ERROR("SUMO was compiled without GDAL support.");
+    WRITE_ERROR(TL("SUMO was compiled without GDAL support."));
 #endif
 }
 
@@ -370,7 +407,7 @@ NIImporter_ArcView::getSpeed(OGRFeature& poFeature, const std::string& edgeid) {
             const double speed = poFeature.GetFieldAsDouble(index);
             if (speed <= 0) {
                 WRITE_WARNING("invalid value for field: '"
-                              + myOptions.getString("shapefile.laneNumber")
+                              + myOptions.getString("shapefile.speed")
                               + "': '" + std::string(poFeature.GetFieldAsString(index)) + "'");
             } else {
                 return speed;
@@ -400,6 +437,50 @@ NIImporter_ArcView::getSpeed(OGRFeature& poFeature, const std::string& edgeid) {
 }
 
 
+double
+NIImporter_ArcView::getLaneWidth(OGRFeature& poFeature, const std::string& edgeid, int laneNumber) {
+    if (myOptions.isSet("shapefile.width")) {
+        int index = poFeature.GetDefnRef()->GetFieldIndex(myOptions.getString("shapefile.width").c_str());
+        if (index >= 0 && poFeature.IsFieldSet(index)) {
+            const double width = poFeature.GetFieldAsDouble(index);
+            if (width <= 0) {
+                WRITE_WARNING("invalid value for field: '"
+                              + myOptions.getString("shapefile.width")
+                              + "' of edge '" + edgeid
+                              + "': '" + std::string(poFeature.GetFieldAsString(index)) + "'");
+            } else {
+                return width / laneNumber;
+            }
+        }
+    }
+    if (myOptions.isSet("shapefile.type-id")) {
+        return myTypeCont.getEdgeTypeWidth(poFeature.GetFieldAsString((char*)(myOptions.getString("shapefile.type-id").c_str())));
+    }
+    return NBEdge::UNSPECIFIED_WIDTH;
+}
+
+
+
+double
+NIImporter_ArcView::getLength(OGRFeature& poFeature, const std::string& edgeid) {
+    if (myOptions.isSet("shapefile.length")) {
+        int index = poFeature.GetDefnRef()->GetFieldIndex(myOptions.getString("shapefile.length").c_str());
+        if (index >= 0 && poFeature.IsFieldSet(index)) {
+            const double length = poFeature.GetFieldAsDouble(index);
+            if (length <= 0) {
+                WRITE_WARNING("invalid value for field: '"
+                              + myOptions.getString("shapefile.length")
+                              + "' of edge '" + edgeid
+                              + "': '" + std::string(poFeature.GetFieldAsString(index)) + "'");
+            } else {
+                return length;
+            }
+        }
+    }
+    return NBEdge::UNSPECIFIED_LOADED_LENGTH;
+}
+
+
 int
 NIImporter_ArcView::getLaneNo(OGRFeature& poFeature, const std::string& edgeid,
                               double speed) {
