@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -55,6 +55,10 @@ public:
     /// @brief Destructor
     ~NIXMLTrafficLightsHandler();
 
+    const std::set<std::string>& getSeenIDs() {
+        return mySeenIDs;
+    }
+
 
 protected:
     /// @name inherited from GenericSAXHandler
@@ -115,6 +119,8 @@ private:
     /// @brief whether definitions for unknown traffic lights shall be silently ignored
     bool myIgnoreUnknown;
 
+    /// @brief list of traffic light logics that were found
+    std::set<std::string> mySeenIDs;
 
 private:
     /// @brief invalidated copy constructor
