@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -44,7 +44,6 @@ public:
 //    static std::vector<int> getWithin(const AbstractPoly &p, double off=0.0);
 //    static void buildNodeClusters();
 //    static void dict_assignConnectionsToNodes();
-    static int dictSize();
     static void clearDict();
     static int getMaxID();
 protected:
