@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -64,13 +64,19 @@
 NLDetectorBuilder::E3DetectorDefinition::E3DetectorDefinition(const std::string& id,
         const std::string& device, double haltingSpeedThreshold,
         SUMOTime haltingTimeThreshold, SUMOTime splInterval,
-        const std::string& vTypes, bool openEntry) :
+        const std::string name, const std::string& vTypes,
+        const std::string& nextEdges,
+        int detectPersons, bool openEntry, bool expectArrival) :
     myID(id), myDevice(device),
     myHaltingSpeedThreshold(haltingSpeedThreshold),
     myHaltingTimeThreshold(haltingTimeThreshold),
     mySampleInterval(splInterval),
+    myName(name),
     myVehicleTypes(vTypes),
-    myOpenEntry(openEntry) {
+    myNextEdges(nextEdges),
+    myDetectPersons(detectPersons),
+    myOpenEntry(openEntry),
+    myExpectArrival(expectArrival) {
 }
 
 
@@ -89,44 +95,64 @@ NLDetectorBuilder::~NLDetectorBuilder() {
 }
 
 
-void
+Parameterised*
 NLDetectorBuilder::buildInductLoop(const std::string& id,
-                                   const std::string& lane, double pos, SUMOTime splInterval,
+                                   const std::string& lane, double pos, double length, SUMOTime splInterval,
                                    const std::string& device, bool friendlyPos,
-                                   const std::string& vTypes) {
+                                   const std::string name,
+                                   const std::string& vTypes,
+                                   const std::string& nextEdges,
+                                   int detectPersons) {
     checkSampleInterval(splInterval, SUMO_TAG_E1DETECTOR, id);
     // get and check the lane
     MSLane* clane = getLaneChecking(lane, SUMO_TAG_E1DETECTOR, id);
     // get and check the position
-    pos = getPositionChecking(pos, clane, friendlyPos, id);
+    pos = getPositionChecking(pos, clane, friendlyPos, SUMO_TAG_E1DETECTOR, id);
+    if (length < 0) {
+        throw InvalidArgument("The length of " + toString(SUMO_TAG_E1DETECTOR) + " '" + id + "' cannot be negative");
+    } else if (length > 0 && pos + length > clane->getLength()) {
+        if (friendlyPos) {
+            length = MIN2(length, clane->getLength());
+            pos = clane->getLength() - length;
+        } else {
+            throw InvalidArgument("The length of " + toString(SUMO_TAG_E1DETECTOR) + " '" + id + "' puts it beyond the lane's '" + clane->getID() + "' end.");
+        }
+    }
     // build the loop
-    MSDetectorFileOutput* loop = createInductLoop(id, clane, pos, vTypes);
+    MSDetectorFileOutput* loop = createInductLoop(id, clane, pos, length, name, vTypes, nextEdges, detectPersons, true);
     // add the file output
     myNet.getDetectorControl().add(SUMO_TAG_INDUCTION_LOOP, loop, device, splInterval);
+    return loop;
 }
 
 
-void
+Parameterised*
 NLDetectorBuilder::buildInstantInductLoop(const std::string& id,
         const std::string& lane, double pos,
         const std::string& device, bool friendlyPos,
-        const std::string& vTypes) {
+        const std::string name,
+        const std::string& vTypes,
+        const std::string& nextEdges,
+        int detectPersons) {
     // get and check the lane
     MSLane* clane = getLaneChecking(lane, SUMO_TAG_INSTANT_INDUCTION_LOOP, id);
     // get and check the position
-    pos = getPositionChecking(pos, clane, friendlyPos, id);
+    pos = getPositionChecking(pos, clane, friendlyPos, SUMO_TAG_INSTANT_INDUCTION_LOOP, id);
     // build the loop
-    MSDetectorFileOutput* loop = createInstantInductLoop(id, clane, pos, device, vTypes);
+    MSDetectorFileOutput* loop = createInstantInductLoop(id, clane, pos, device, name, vTypes, nextEdges, detectPersons);
     // add the file output
     myNet.getDetectorControl().add(SUMO_TAG_INSTANT_INDUCTION_LOOP, loop);
+    return loop;
 }
 
 
-void
+Parameterised*
 NLDetectorBuilder::buildE2Detector(const std::string& id, MSLane* lane, double pos, double endPos, double length,
                                    const std::string& device, SUMOTime frequency,
                                    SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
-                                   const std::string& vTypes, bool friendlyPos, bool showDetector,
+                                   const std::string name, const std::string& vTypes,
+                                   const std::string& nextEdges,
+                                   int detectPersons, bool friendlyPos, bool showDetector,
                                    MSTLLogicControl::TLSLogicVariants* tlls, MSLane* toLane) {
 
     bool tlsGiven = tlls != nullptr;
@@ -175,7 +201,7 @@ NLDetectorBuilder::buildE2Detector(const std::string& id, MSLane* lane, double p
     MSE2Collector* det = nullptr;
     if (tlsGiven) {
         // Detector connected to TLS
-        det =  createE2Detector(id, DU_USER_DEFINED, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes, showDetector);
+        det =  createE2Detector(id, DU_USER_DEFINED, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons, showDetector);
         myNet.getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, det);
         // add the file output (XXX: Where's the corresponding delete?)
         if (toLaneGiven) {
@@ -195,17 +221,20 @@ NLDetectorBuilder::buildE2Detector(const std::string& id, MSLane* lane, double p
     } else {
         // User specified detector for xml-output
         checkSampleInterval(frequency, SUMO_TAG_E2DETECTOR, id);
-        det =  createE2Detector(id, DU_USER_DEFINED, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes, showDetector);
+        det = createE2Detector(id, DU_USER_DEFINED, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons, showDetector);
         myNet.getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, det, device, frequency);
     }
-
+    return det;
 }
 
-void
+
+Parameterised*
 NLDetectorBuilder::buildE2Detector(const std::string& id, std::vector<MSLane*> lanes, double pos, double endPos,
                                    const std::string& device, SUMOTime frequency,
                                    SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
-                                   const std::string& vTypes, bool friendlyPos, bool showDetector,
+                                   const std::string name, const std::string& vTypes,
+                                   const std::string& nextEdges,
+                                   int detectPersons, bool friendlyPos, bool showDetector,
                                    MSTLLogicControl::TLSLogicVariants* tlls, MSLane* toLane) {
 
     bool tlsGiven = tlls != nullptr;
@@ -252,7 +281,7 @@ NLDetectorBuilder::buildE2Detector(const std::string& id, std::vector<MSLane*> l
     MSE2Collector* det = nullptr;
     if (tlsGiven) {
         // Detector connected to TLS
-        det = createE2Detector(id, DU_USER_DEFINED, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes, showDetector);
+        det = createE2Detector(id, DU_USER_DEFINED, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons, showDetector);
         myNet.getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, det);
         // add the file output (XXX: Where's the corresponding delete?)
         if (toLaneGiven) {
@@ -273,22 +302,26 @@ NLDetectorBuilder::buildE2Detector(const std::string& id, std::vector<MSLane*> l
         // User specified detector for xml-output
         checkSampleInterval(frequency, SUMO_TAG_E2DETECTOR, id);
 
-        det = createE2Detector(id, DU_USER_DEFINED, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes, showDetector);
+        det = createE2Detector(id, DU_USER_DEFINED, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons, showDetector);
         myNet.getDetectorControl().add(SUMO_TAG_LANE_AREA_DETECTOR, det, device, frequency);
     }
-
+    return det;
 }
 
 
 
-void
+Parameterised*
 NLDetectorBuilder::beginE3Detector(const std::string& id,
                                    const std::string& device, SUMOTime splInterval,
                                    double haltingSpeedThreshold,
                                    SUMOTime haltingTimeThreshold,
-                                   const std::string& vTypes, bool openEntry) {
+                                   const std::string name,
+                                   const std::string& vTypes,
+                                   const std::string& nextEdges,
+                                   int detectPersons, bool openEntry, bool expectArrival) {
     checkSampleInterval(splInterval, SUMO_TAG_E3DETECTOR, id);
-    myE3Definition = new E3DetectorDefinition(id, device, haltingSpeedThreshold, haltingTimeThreshold, splInterval, vTypes, openEntry);
+    myE3Definition = new E3DetectorDefinition(id, device, haltingSpeedThreshold, haltingTimeThreshold, splInterval, name, vTypes, nextEdges, detectPersons, openEntry, expectArrival);
+    return myE3Definition;
 }
 
 
@@ -300,7 +333,7 @@ NLDetectorBuilder::addE3Entry(const std::string& lane,
     }
     MSLane* clane = getLaneChecking(lane, SUMO_TAG_E3DETECTOR, myE3Definition->myID);
     // get and check the position
-    pos = getPositionChecking(pos, clane, friendlyPos, myE3Definition->myID);
+    pos = getPositionChecking(pos, clane, friendlyPos, SUMO_TAG_DET_ENTRY, myE3Definition->myID);
     // build and save the entry
     myE3Definition->myEntries.push_back(MSCrossSection(clane, pos));
 }
@@ -314,7 +347,7 @@ NLDetectorBuilder::addE3Exit(const std::string& lane,
     }
     MSLane* clane = getLaneChecking(lane, SUMO_TAG_E3DETECTOR, myE3Definition->myID);
     // get and check the position
-    pos = getPositionChecking(pos, clane, friendlyPos, myE3Definition->myID);
+    pos = getPositionChecking(pos, clane, friendlyPos, SUMO_TAG_DET_EXIT, myE3Definition->myID);
     // build and save the exit
     myE3Definition->myExits.push_back(MSCrossSection(clane, pos));
 }
@@ -339,15 +372,21 @@ NLDetectorBuilder::endE3Detector() {
         // create E3 detector
         MSDetectorFileOutput* det = createE3Detector(myE3Definition->myID,
                                     myE3Definition->myEntries, myE3Definition->myExits,
-                                    myE3Definition->myHaltingSpeedThreshold, myE3Definition->myHaltingTimeThreshold, myE3Definition->myVehicleTypes,
-                                    myE3Definition->myOpenEntry);
+                                    myE3Definition->myHaltingSpeedThreshold, myE3Definition->myHaltingTimeThreshold,
+                                    myE3Definition->myName,
+                                    myE3Definition->myVehicleTypes,
+                                    myE3Definition->myNextEdges,
+                                    myE3Definition->myDetectPersons,
+                                    myE3Definition->myOpenEntry,
+                                    myE3Definition->myExpectArrival);
+        det->updateParameters(myE3Definition->getParametersMap());
         // add to net
         myNet.getDetectorControl().add(SUMO_TAG_ENTRY_EXIT_DETECTOR, det, myE3Definition->myDevice, myE3Definition->mySampleInterval);
-    } else
+    } else {
         WRITE_WARNING(toString(SUMO_TAG_E3DETECTOR) + " with id = '" + myE3Definition->myID + "' will not be created because is empty (no " + toString(SUMO_TAG_DET_ENTRY) + " or " + toString(SUMO_TAG_DET_EXIT) + " was defined)")
-
-        // clean up
-        delete myE3Definition;
+    }
+    // clean up
+    delete myE3Definition;
     myE3Definition = nullptr;
 }
 
@@ -373,22 +412,30 @@ NLDetectorBuilder::buildRouteProbe(const std::string& id, const std::string& edg
     myNet.getDetectorControl().add(SUMO_TAG_ROUTEPROBE, probe, device, frequency, begin);
 }
 
+
 MSDetectorFileOutput*
 NLDetectorBuilder::createInductLoop(const std::string& id,
                                     MSLane* lane, double pos,
-                                    const std::string& vTypes, bool) {
+                                    double length,
+                                    const std::string name,
+                                    const std::string& vTypes,
+                                    const std::string& nextEdges,
+                                    int detectPersons,
+                                    bool /*show*/) {
     if (MSGlobals::gUseMesoSim) {
-        return new MEInductLoop(id, MSGlobals::gMesoNet->getSegmentForEdge(lane->getEdge(), pos), pos, vTypes);
+        return new MEInductLoop(id, MSGlobals::gMesoNet->getSegmentForEdge(lane->getEdge(), pos), pos, name, vTypes, nextEdges, detectPersons);
     }
-    return new MSInductLoop(id, lane, pos, vTypes, false);
+    return new MSInductLoop(id, lane, pos, length, name, vTypes, nextEdges, detectPersons, false);
 }
 
 
 MSDetectorFileOutput*
 NLDetectorBuilder::createInstantInductLoop(const std::string& id,
         MSLane* lane, double pos, const std::string& od,
-        const std::string& vTypes) {
-    return new MSInstantInductLoop(id, OutputDevice::getDevice(od), lane, pos, vTypes);
+        const std::string name, const std::string& vTypes,
+        const std::string& nextEdges,
+        int detectPersons) {
+    return new MSInstantInductLoop(id, OutputDevice::getDevice(od), lane, pos, name, vTypes, nextEdges, detectPersons);
 }
 
 
@@ -396,16 +443,20 @@ MSE2Collector*
 NLDetectorBuilder::createE2Detector(const std::string& id,
                                     DetectorUsage usage, MSLane* lane, double pos, double endPos, double length,
                                     SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
-                                    const std::string& vTypes, bool /* showDetector */) {
-    return new MSE2Collector(id, usage, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes);
+                                    const std::string name, const std::string& vTypes,
+                                    const std::string& nextEdges,
+                                    int detectPersons, bool /* showDetector */) {
+    return new MSE2Collector(id, usage, lane, pos, endPos, length, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons);
 }
 
 MSE2Collector*
 NLDetectorBuilder::createE2Detector(const std::string& id,
                                     DetectorUsage usage, std::vector<MSLane*> lanes, double pos, double endPos,
                                     SUMOTime haltingTimeThreshold, double haltingSpeedThreshold, double jamDistThreshold,
-                                    const std::string& vTypes, bool /* showDetector */) {
-    return new MSE2Collector(id, usage, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, vTypes);
+                                    const std::string name, const std::string& vTypes,
+                                    const std::string& nextEdges,
+                                    int detectPersons, bool /* showDetector */) {
+    return new MSE2Collector(id, usage, lanes, pos, endPos, haltingTimeThreshold, haltingSpeedThreshold, jamDistThreshold, name, vTypes, nextEdges, detectPersons);
 }
 
 MSDetectorFileOutput*
@@ -414,14 +465,18 @@ NLDetectorBuilder::createE3Detector(const std::string& id,
                                     const CrossSectionVector& exits,
                                     double haltingSpeedThreshold,
                                     SUMOTime haltingTimeThreshold,
-                                    const std::string& vTypes,
-                                    bool openEntry) {
-    return new MSE3Collector(id, entries, exits, haltingSpeedThreshold, haltingTimeThreshold, vTypes, openEntry);
+                                    const std::string name, const std::string& vTypes,
+                                    const std::string& nextEdges,
+                                    int detectPersons,
+                                    bool openEntry,
+                                    bool expectArrival) {
+    return new MSE3Collector(id, entries, exits, haltingSpeedThreshold, haltingTimeThreshold, name, vTypes, nextEdges, detectPersons, openEntry, expectArrival);
 }
 
 
 double
 NLDetectorBuilder::getPositionChecking(double pos, MSLane* lane, bool friendlyPos,
+                                       SumoXMLTag tag,
                                        const std::string& detid) {
     // check whether it is given from the end
     if (pos < 0) {
@@ -432,14 +487,14 @@ NLDetectorBuilder::getPositionChecking(double pos, MSLane* lane, bool friendlyPo
         if (friendlyPos) {
             pos = lane->getLength();
         } else {
-            throw InvalidArgument("The position of detector '" + detid + "' lies beyond the lane's '" + lane->getID() + "' end.");
+            throw InvalidArgument("The position of " + toString(tag) + " '" + detid + "' lies beyond the lane's '" + lane->getID() + "' end.");
         }
     }
     if (pos < 0) {
         if (friendlyPos) {
             pos = 0.;
         } else {
-            throw InvalidArgument("The position of detector '" + detid + "' lies before the lane's '" + lane->getID() + "' begin.");
+            throw InvalidArgument("The position of " + toString(tag) + "  '" + detid + "' lies before the lane's '" + lane->getID() + "' begin.");
         }
     }
     return pos;
@@ -454,6 +509,8 @@ NLDetectorBuilder::createEdgeLaneMeanData(const std::string& id, SUMOTime freque
         const double maxTravelTime, const double minSamples,
         const double haltSpeed, const std::string& vTypes,
         const std::string& writeAttributes,
+        std::vector<MSEdge*> edges,
+        AggregateType aggregate,
         const std::string& device) {
     if (begin < 0) {
         throw InvalidArgument("Negative begin time for meandata dump '" + id + "'.");
@@ -466,21 +523,23 @@ NLDetectorBuilder::createEdgeLaneMeanData(const std::string& id, SUMOTime freque
     }
     checkStepLengthMultiple(begin, " for meandata dump '" + id + "'");
     MSMeanData* det = nullptr;
-    if (type == "" || type == "performance" || type == "traffic") {
+    if ((type == SUMOXMLDefinitions::MeanDataTypes.getString(MeanDataType::DEFAULT)) ||
+            (type == SUMOXMLDefinitions::MeanDataTypes.getString(MeanDataType::TRAFFIC)) ||
+            (type == "performance")) {
         det = new MSMeanData_Net(id, begin, end, useLanes, withEmpty,
-                                 printDefaults, withInternal, trackVehicles, detectPersons, maxTravelTime, minSamples, haltSpeed, vTypes, writeAttributes);
-    } else if (type == "emissions" || type == "hbefa") {
+                                 printDefaults, withInternal, trackVehicles, detectPersons, maxTravelTime, minSamples, haltSpeed, vTypes, writeAttributes, edges, aggregate);
+    } else if ((type == SUMOXMLDefinitions::MeanDataTypes.getString(MeanDataType::EMISSIONS)) || (type == "hbefa")) {
         if (type == "hbefa") {
-            WRITE_WARNING("The netstate type 'hbefa' is deprecated. Please use the type 'emissions' instead.");
+            WRITE_WARNING(TL("The netstate type 'hbefa' is deprecated. Please use the type 'emissions' instead."));
         }
         det = new MSMeanData_Emissions(id, begin, end, useLanes, withEmpty,
-                                       printDefaults, withInternal, trackVehicles, maxTravelTime, minSamples, vTypes, writeAttributes);
-    } else if (type == "harmonoise") {
+                                       printDefaults, withInternal, trackVehicles, maxTravelTime, minSamples, vTypes, writeAttributes, edges, aggregate);
+    } else if (type == SUMOXMLDefinitions::MeanDataTypes.getString(MeanDataType::HARMONOISE)) {
         det = new MSMeanData_Harmonoise(id, begin, end, useLanes, withEmpty,
-                                        printDefaults, withInternal, trackVehicles, maxTravelTime, minSamples, vTypes, writeAttributes);
-    } else if (type == "amitran") {
+                                        printDefaults, withInternal, trackVehicles, maxTravelTime, minSamples, vTypes, writeAttributes, edges, aggregate);
+    } else if (type == SUMOXMLDefinitions::MeanDataTypes.getString(MeanDataType::AMITRAN)) {
         det = new MSMeanData_Amitran(id, begin, end, useLanes, withEmpty,
-                                     printDefaults, withInternal, trackVehicles, detectPersons, maxTravelTime, minSamples, haltSpeed, vTypes, writeAttributes);
+                                     printDefaults, withInternal, trackVehicles, detectPersons, maxTravelTime, minSamples, haltSpeed, vTypes, writeAttributes, edges, aggregate);
     } else {
         throw InvalidArgument("Invalid type '" + type + "' for meandata dump '" + id + "'.");
     }
