@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -80,7 +80,7 @@ public:
                                 bool includeInternal, ConnectionStyle style = SUMONET, bool geoAccuracy = false);
 
     /// @brief writes the given prohibitions
-    static void writeProhibitions(OutputDevice& into, const NBConnectionProhibits& prohibitions);
+    static void writeProhibitions(OutputDevice& into, const NBConnectionProhibits& prohibitions, const NBEdgeCont& ec);
 
     /// @brief writes the traffic light logics to the given device
     static void writeTrafficLights(OutputDevice& into, const NBTrafficLightLogicCont& tllCont);
@@ -96,10 +96,9 @@ public:
     static void writeRoundabouts(OutputDevice& into, const std::set<EdgeSet>& roundabouts,
                                  const NBEdgeCont& ec);
 
-
     /** @brief Write a stopOffset element into output device
      */
-    static void writeStopOffsets(OutputDevice& into, const std::map<SVCPermissions, double>& stopOffsets);
+    static void writeStopOffsets(OutputDevice& into, const StopOffset& stopOffset);
 
     /** @brief Writes a district
      * @param[in] into The device to write the edge into
@@ -120,13 +119,17 @@ private:
     static bool writeInternalEdges(OutputDevice& into, const NBEdgeCont& ec, const NBNode& n);
 
 
+    /// @brief retrieve bidi edge id for internal corresponding to the given connection
+    static std::string getInternalBidi(const NBEdge* e, const NBEdge::Connection& k, double& length);
+
+
     /** @brief Writes an edge (<edge ...)
      * @param[in] into The device to write the edge into
      * @param[in] e The edge to write
      * @param[in] noNames Whether names shall be ignored
      * @see writeLane()
      */
-    static void writeEdge(OutputDevice& into, const NBEdge& e, bool noNames);
+    static void writeEdge(OutputDevice& into, const NBEdge& e, bool noNames, LaneSpreadFunction defaultSpread);
 
 
     /** @brief Writes a lane (<lane ...) of an edge
@@ -140,14 +143,16 @@ private:
      * @param[in] customShape whether this lane has a custom shape
      */
     static void writeLane(OutputDevice& into, const std::string& lID,
-                          double speed, SVCPermissions permissions, SVCPermissions preferred,
+                          double speed, double friction,
+                          SVCPermissions permissions, SVCPermissions preferred,
                           SVCPermissions changeLeft, SVCPermissions changeRight,
                           double startOffset, double endOffset,
-                          std::map<SVCPermissions, double> stopOffsets, double width, PositionVector shape,
+                          const StopOffset& stopOffset, double width, PositionVector shape,
                           const Parameterised* params, double length, int index,
                           const std::string& oppositeID, const std::string& type,
                           bool accelRamp = false,
-                          bool customShape = false);
+                          bool customShape = false,
+                          const PositionVector& outlineShape = PositionVector());
 
 
     /** @brief Writes a junction (<junction ...)
@@ -182,12 +187,13 @@ private:
                                         int fromLane, int toLane, const std::string& via,
                                         LinkDirection dir = LinkDirection::STRAIGHT,
                                         const std::string& tlID = "",
-                                        int linkIndex = NBConnection::InvalidTlIndex);
+                                        int linkIndex = NBConnection::InvalidTlIndex,
+                                        bool minor = false,
+                                        double visibility = NBEdge::UNSPECIFIED_VISIBILITY_DISTANCE);
 
     /// @brief writes a SUMOTime as int if possible, otherwise as a float
     static std::string writeSUMOTime(SUMOTime time);
 
-
     /// @brief the attribute value for a prohibition
     static std::string prohibitionConnection(const NBConnection& c);
 
