@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -23,24 +23,24 @@
 #include <config.h>
 #include <algorithm>
 #include <utils/common/MsgHandler.h>
+#include <utils/common/ToString.h>
+#include <utils/common/StringUtils.h>
+#include <utils/options/OptionsCont.h>
+#include <utils/iodevices/OutputDevice.h>
+#include <utils/geom/GeoConvHelper.h>
 #include <netbuild/NBEdge.h>
 #include <netbuild/NBEdgeCont.h>
 #include <netbuild/NBNode.h>
 #include <netbuild/NBNodeCont.h>
 #include <netbuild/NBNetBuilder.h>
 #include <netbuild/NBPTLineCont.h>
+#include <netbuild/NBPTStop.h>
 #include <netbuild/NBParking.h>
-#include <utils/common/ToString.h>
-#include <utils/common/StringUtils.h>
-#include <utils/options/OptionsCont.h>
-#include <utils/iodevices/OutputDevice.h>
-#include <utils/geom/GeoConvHelper.h>
 #include "NWFrame.h"
 #include "NWWriter_SUMO.h"
 #include "NWWriter_XML.h"
 
 
-
 // ===========================================================================
 // method definitions
 // ===========================================================================
@@ -48,51 +48,81 @@
 // static methods
 // ---------------------------------------------------------------------------
 void
-NWWriter_XML::writeNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
+NWWriter_XML::writeNetwork(const OptionsCont& oc, const std::string& prefix, NBNetBuilder& nb) {
     // check whether plain-output files shall be generated
-    if (oc.isSet("plain-output-prefix")) {
-        writeNodes(oc, nb.getNodeCont());
-        if (nb.getTypeCont().size() > 0) {
-            writeTypes(oc, nb.getTypeCont());
+    if (prefix != "") {
+        const bool haveTypes = nb.getTypeCont().size() > 0;
+        writeNodes(oc, prefix, nb.getNodeCont());
+        if (haveTypes) {
+            writeTypes(prefix, nb.getEdgeCont(), nb.getTypeCont());
         }
-        writeEdgesAndConnections(oc, nb.getNodeCont(), nb.getEdgeCont());
-        writeTrafficLights(oc, nb.getTLLogicCont(), nb.getEdgeCont());
+        writeEdgesAndConnections(oc, prefix, nb.getNodeCont(), nb.getEdgeCont());
+        writeTrafficLights(prefix, nb.getTLLogicCont(), nb.getEdgeCont());
+        writeConfig(oc, prefix, haveTypes);
     }
-    if (oc.isSet("junctions.join-output")) {
-        writeJoinedJunctions(oc, nb.getNodeCont());
+    if (oc.isSet("junctions.join-output") && (oc.getString("junctions.join-output").size() > 0)) {
+        writeJoinedJunctions(oc.getString("junctions.join-output"), nb.getNodeCont());
     }
-    if (oc.isSet("street-sign-output")) {
+    if (oc.isSet("street-sign-output") && (oc.getString("street-sign-output").size() > 0)) {
         writeStreetSigns(oc, nb.getEdgeCont());
     }
-    if (oc.exists("ptstop-output") && oc.isSet("ptstop-output")) {
+    if (oc.exists("ptstop-output") && oc.isSet("ptstop-output") && (oc.getString("ptstop-output").size() > 0)) {
         writePTStops(oc, nb.getPTStopCont());
     }
-    if (oc.exists("ptline-output") && oc.isSet("ptline-output")) {
-        writePTLines(oc, nb.getPTLineCont(), nb.getEdgeCont());
+    if (oc.exists("ptline-output") && oc.isSet("ptline-output") && (oc.getString("ptline-output").size() > 0)) {
+        writePTLines(oc, nb.getPTLineCont());
     }
-
-    if (oc.exists("parking-output") && oc.isSet("parking-output")) {
+    if (oc.exists("parking-output") && oc.isSet("parking-output") && (oc.getString("parking-output").size() > 0)) {
         writeParkingAreas(oc, nb.getParkingCont(), nb.getEdgeCont());
     }
-    if (oc.exists("taz-output") && oc.isSet("taz-output")) {
+    if (oc.exists("taz-output") && oc.isSet("taz-output") && (oc.getString("taz-output").size() > 0)) {
         writeDistricts(oc, nb.getDistrictCont());
     }
 }
 
 
 void
-NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
+NWWriter_XML::writeConfig(const OptionsCont& oc, const std::string& prefix, bool haveTypes) {
+    if (!oc.exists("node-files")) {
+        // do not write configuration for netgen
+        return;
+    }
+    OptionsCont* tmp = oc.clone();
+    tmp->set("node-files", prefix + ".nod.xml");
+    tmp->set("edge-files", prefix + ".edg.xml");
+    tmp->set("connection-files", prefix + ".con.xml");
+    tmp->set("tllogic-files", prefix + ".tll.xml");
+    if (haveTypes) {
+        tmp->set("type-files", prefix + ".typ.xml");
+    }
+    tmp->setDefault("sumo-net-file", "");
+    tmp->setDefault("plain-output-prefix", "");
+
+    const std::string configPath = prefix + ".netccfg";
+    std::ofstream out(configPath.c_str());
+    if (!out.good()) {
+        delete tmp;
+        throw ProcessError(TLF("Could not save configuration to '%'", configPath));
+    } else {
+        tmp->writeConfiguration(out, true, false, false);
+    }
+    delete tmp;
+}
+
+
+void
+NWWriter_XML::writeNodes(const OptionsCont& oc, const std::string& prefix, NBNodeCont& nc) {
     const GeoConvHelper& gch = GeoConvHelper::getFinal();
     bool useGeo = oc.exists("proj.plain-geo") && oc.getBool("proj.plain-geo");
     if (useGeo && !gch.usingGeoProjection()) {
-        WRITE_WARNING("Ignoring option \"proj.plain-geo\" because no geo-conversion has been defined");
+        WRITE_WARNING(TL("Ignoring option \"proj.plain-geo\" because no geo-conversion has been defined"));
         useGeo = false;
     }
     const bool geoAccuracy = useGeo || gch.usingInverseGeoProjection();
 
-    OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".nod.xml");
+    OutputDevice& device = OutputDevice::getDevice(prefix + ".nod.xml");
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION);
     device.writeXMLHeader("nodes", "nodes_file.xsd", attrs);
 
     // write network offsets and projection to allow reconstruction of original coordinates
@@ -101,6 +131,7 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
     }
 
     // write nodes
+    TrafficLightType tlsDefaultType = SUMOXMLDefinitions::TrafficLightTypes.get(oc.getString("tls.default-type"));
     for (std::map<std::string, NBNode*>::const_iterator i = nc.begin(); i != nc.end(); ++i) {
         NBNode* n = (*i).second;
         device.openTag(SUMO_TAG_NODE);
@@ -120,19 +151,25 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
 
         device.writeAttr(SUMO_ATTR_TYPE, toString(n->getType()));
         if (n->isTLControlled()) {
-            const std::set<NBTrafficLightDefinition*>& tlss = n->getControllingTLS();
             // set may contain multiple programs for the same id.
             // make sure ids are unique and sorted
             std::set<std::string> tlsIDs;
             std::set<std::string> controlledInnerEdges;
-            for (std::set<NBTrafficLightDefinition*>::const_iterator it_tl = tlss.begin(); it_tl != tlss.end(); it_tl++) {
-                tlsIDs.insert((*it_tl)->getID());
-                std::vector<std::string> cie = (*it_tl)->getControlledInnerEdges();
+            std::string tlType = "";
+            for (NBTrafficLightDefinition* tl : n->getControllingTLS()) {
+                tlsIDs.insert(tl->getID());
+                std::vector<std::string> cie = tl->getControlledInnerEdges();
                 controlledInnerEdges.insert(cie.begin(), cie.end());
+                if (tl->getType() != tlsDefaultType) {
+                    tlType = toString(tl->getType());
+                }
             }
             std::vector<std::string> sortedIDs(tlsIDs.begin(), tlsIDs.end());
             sort(sortedIDs.begin(), sortedIDs.end());
             device.writeAttr(SUMO_ATTR_TLID, sortedIDs);
+            if (tlType != "") {
+                device.writeAttr(SUMO_ATTR_TLTYPE, tlType);
+            }
             if (controlledInnerEdges.size() > 0) {
                 std::vector<std::string> sortedCIEs(controlledInnerEdges.begin(), controlledInnerEdges.end());
                 sort(sortedCIEs.begin(), sortedCIEs.end());
@@ -145,7 +182,7 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
         if (n->getRadius() != NBNode::UNSPECIFIED_RADIUS) {
             device.writeAttr(SUMO_ATTR_RADIUS, n->getRadius());
         }
-        if (n->getKeepClear() == false) {
+        if (!n->getKeepClear()) {
             device.writeAttr<bool>(SUMO_ATTR_KEEP_CLEAR, n->getKeepClear());
         }
         if (n->getRightOfWay() != RightOfWay::DEFAULT) {
@@ -154,8 +191,11 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
         if (n->getFringeType() != FringeType::DEFAULT) {
             device.writeAttr<std::string>(SUMO_ATTR_FRINGE, toString(n->getFringeType()));
         }
+        if (n->getRoundaboutType() != RoundaboutType::DEFAULT) {
+            device.writeAttr<std::string>(SUMO_ATTR_ROUNDABOUT, toString(n->getRoundaboutType()));
+        }
         if (n->getName() != "") {
-            device.writeAttr<std::string>(SUMO_ATTR_NAME, n->getName());
+            device.writeAttr<std::string>(SUMO_ATTR_NAME, StringUtils::escapeXML(n->getName()));
         }
         n->writeParams(device);
         device.closeTag();
@@ -165,29 +205,36 @@ NWWriter_XML::writeNodes(const OptionsCont& oc, NBNodeCont& nc) {
 
 
 void
-NWWriter_XML::writeTypes(const OptionsCont& oc, NBTypeCont& tc) {
-    OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".typ.xml");
+NWWriter_XML::writeTypes(const std::string& prefix, NBEdgeCont& ec, NBTypeCont& tc) {
+    OutputDevice& device = OutputDevice::getDevice(prefix + ".typ.xml");
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
-    device.writeXMLHeader("types", "types_file.xsd", attrs);
-    tc.writeEdgeTypes(device);
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION);
+    device.writeXMLHeader(toString(SUMO_TAG_TYPES), "types_file.xsd", attrs);
+    std::set<std::string> usedTypes = ec.getUsedTypes();
+    tc.writeEdgeTypes(device, usedTypes);
     device.close();
 }
 
 
 void
-NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NBEdgeCont& ec) {
+NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, const std::string& prefix, NBNodeCont& nc, NBEdgeCont& ec) {
     const GeoConvHelper& gch = GeoConvHelper::getFinal();
     bool useGeo = oc.exists("proj.plain-geo") && oc.getBool("proj.plain-geo");
     const bool geoAccuracy = useGeo || gch.usingInverseGeoProjection();
 
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
-    OutputDevice& edevice = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".edg.xml");
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION);
+    OutputDevice& edevice = OutputDevice::getDevice(prefix + ".edg.xml");
     edevice.writeXMLHeader("edges", "edges_file.xsd", attrs);
-    OutputDevice& cdevice = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".con.xml");
+    OutputDevice& cdevice = OutputDevice::getDevice(prefix + ".con.xml");
     cdevice.writeXMLHeader("connections", "connections_file.xsd", attrs);
     const bool writeNames = oc.getBool("output.street-names");
+    const bool writeLanes = oc.getBool("plain-output.lanes");
+
+    // write network offsets and projection to allow reconstruction of original coordinates at least for geo-referenced networks
+    if (!useGeo && gch.usingGeoProjection()) {
+        GeoConvHelper::writeLocation(edevice);
+    }
     LaneSpreadFunction defaultSpread = SUMOXMLDefinitions::LaneSpreadFunctions.get(oc.getString("default.spreadtype"));
     for (std::map<std::string, NBEdge*>::const_iterator i = ec.begin(); i != ec.end(); ++i) {
         // write the edge itself to the edges-files
@@ -204,10 +251,18 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
         if (e->getTypeID() != "") {
             edevice.writeAttr(SUMO_ATTR_TYPE, e->getTypeID());
         }
+        if (e->getRoutingType() != "") {
+            edevice.writeAttr(SUMO_ATTR_ROUTINGTYPE, e->getRoutingType());
+        }
         edevice.writeAttr(SUMO_ATTR_NUMLANES, e->getNumLanes());
         if (!e->hasLaneSpecificSpeed()) {
             edevice.writeAttr(SUMO_ATTR_SPEED, e->getSpeed());
         }
+        if (!e->hasLaneSpecificFriction()) {
+            if (e->getFriction() != NBEdge::UNSPECIFIED_FRICTION) {
+                edevice.writeAttr(SUMO_ATTR_FRICTION, e->getFriction());
+            }
+        }
         // write non-default geometry
         if (!e->hasDefaultGeometry()) {
             writeShape(edevice, gch, e->getGeometry(), SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
@@ -230,37 +285,40 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
         if (!e->hasLaneSpecificPermissions()) {
             writePermissions(edevice, e->getPermissions(0));
         }
-        if (!e->hasLaneSpecificStopOffsets() && e->getStopOffsets().size() != 0) {
-            NWWriter_SUMO::writeStopOffsets(edevice, e->getStopOffsets());
+        if (!e->hasLaneSpecificStopOffsets() && e->getEdgeStopOffset().isDefined()) {
+            NWWriter_SUMO::writeStopOffsets(edevice, e->getEdgeStopOffset());
         }
         if (e->getDistance() != 0) {
             edevice.writeAttr(SUMO_ATTR_DISTANCE, e->getDistance());
         }
-        if (e->needsLaneSpecificOutput()) {
+        if (e->getBidiEdge() != 0) {
+            edevice.writeAttr(SUMO_ATTR_BIDI, e->getBidiEdge()->getID());
+        }
+        if (e->needsLaneSpecificOutput() || writeLanes) {
             int idx = 0;
             for (const NBEdge::Lane& lane : e->getLanes()) {
                 edevice.openTag(SUMO_TAG_LANE);
                 edevice.writeAttr(SUMO_ATTR_INDEX, idx++);
                 // write allowed lanes
-                if (e->hasLaneSpecificPermissions()) {
+                if (e->hasLaneSpecificPermissions() || writeLanes) {
                     writePermissions(edevice, lane.permissions);
                 }
                 writePreferences(edevice, lane.preferred);
                 // write other attributes
-                if (lane.width != NBEdge::UNSPECIFIED_WIDTH && e->hasLaneSpecificWidth()) {
+                if (lane.width != NBEdge::UNSPECIFIED_WIDTH && (e->hasLaneSpecificWidth() || writeLanes)) {
                     edevice.writeAttr(SUMO_ATTR_WIDTH, lane.width);
                 }
-                if (lane.endOffset != NBEdge::UNSPECIFIED_OFFSET && e->hasLaneSpecificEndOffset()) {
+                if (lane.endOffset != NBEdge::UNSPECIFIED_OFFSET && (e->hasLaneSpecificEndOffset() || writeLanes)) {
                     edevice.writeAttr(SUMO_ATTR_ENDOFFSET, lane.endOffset);
                 }
-                if (e->hasLaneSpecificSpeed()) {
+                if (e->hasLaneSpecificSpeed() || writeLanes) {
                     edevice.writeAttr(SUMO_ATTR_SPEED, lane.speed);
                 }
                 if (lane.accelRamp) {
                     edevice.writeAttr(SUMO_ATTR_ACCELERATION, lane.accelRamp);
                 }
-                if (lane.customShape.size() > 0) {
-                    writeShape(edevice, gch, lane.customShape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
+                if (lane.customShape.size() > 0 || writeLanes) {
+                    writeShape(edevice, gch, lane.customShape.size() > 0 ? lane.customShape : lane.shape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
                 }
                 if (lane.type != "") {
                     edevice.writeAttr(SUMO_ATTR_TYPE, lane.type);
@@ -277,7 +335,7 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
                     edevice.closeTag();
                 }
                 lane.writeParams(edevice);
-                NWWriter_SUMO::writeStopOffsets(edevice, lane.stopOffsets);
+                NWWriter_SUMO::writeStopOffsets(edevice, lane.laneStopOffset);
                 edevice.closeTag();
             }
         }
@@ -321,7 +379,7 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
 
     // write loaded prohibitions to the connections-file
     for (std::map<std::string, NBNode*>::const_iterator i = nc.begin(); i != nc.end(); ++i) {
-        NWWriter_SUMO::writeProhibitions(cdevice, i->second->getProhibitions());
+        NWWriter_SUMO::writeProhibitions(cdevice, i->second->getProhibitions(), ec);
     }
     // write pedestrian crossings to the connections-file
     for (std::map<std::string, NBNode*>::const_iterator it_node = nc.begin(); it_node != nc.end(); ++it_node) {
@@ -343,6 +401,10 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
             if (c->customShape.size() != 0) {
                 writeShape(cdevice, gch, c->customShape, SUMO_ATTR_SHAPE, useGeo, geoAccuracy);
             }
+            if (c->outlineShape.size() != 0) {
+                writeShape(cdevice, gch, c->outlineShape, SUMO_ATTR_OUTLINESHAPE, useGeo, geoAccuracy);
+            }
+            c->writeParams(cdevice);
             cdevice.closeTag();
         }
     }
@@ -368,10 +430,10 @@ NWWriter_XML::writeEdgesAndConnections(const OptionsCont& oc, NBNodeCont& nc, NB
 
 
 void
-NWWriter_XML::writeTrafficLights(const OptionsCont& oc, NBTrafficLightLogicCont& tc, NBEdgeCont& ec) {
+NWWriter_XML::writeTrafficLights(const std::string& prefix, NBTrafficLightLogicCont& tc, NBEdgeCont& ec) {
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
-    OutputDevice& device = OutputDevice::getDevice(oc.getString("plain-output-prefix") + ".tll.xml");
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION);
+    OutputDevice& device = OutputDevice::getDevice(prefix + ".tll.xml");
     device.writeXMLHeader("tlLogics", "tllogic_file.xsd", attrs);
     NWWriter_SUMO::writeTrafficLights(device, tc);
     // we also need to remember the associations between tlLogics and connections
@@ -391,10 +453,10 @@ NWWriter_XML::writeTrafficLights(const OptionsCont& oc, NBTrafficLightLogicCont&
 
 
 void
-NWWriter_XML::writeJoinedJunctions(const OptionsCont& oc, NBNodeCont& nc) {
+NWWriter_XML::writeJoinedJunctions(const std::string& filename, NBNodeCont& nc) {
     std::map<SumoXMLAttr, std::string> attrs;
-    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION, 1);
-    OutputDevice& device = OutputDevice::getDevice(oc.getString("junctions.join-output"));
+    attrs[SUMO_ATTR_VERSION] = toString(NETWORK_VERSION);
+    OutputDevice& device = OutputDevice::getDevice(filename);
     device.writeXMLHeader("nodes", "nodes_file.xsd", attrs);
     const std::vector<std::set<std::string> >& clusters = nc.getJoinedClusters();
     for (std::vector<std::set<std::string> >::const_iterator it = clusters.begin(); it != clusters.end(); it++) {
@@ -427,24 +489,29 @@ NWWriter_XML::writeStreetSigns(const OptionsCont& oc, NBEdgeCont& ec) {
     }
     device.close();
 }
+
+
 void
 NWWriter_XML::writePTStops(const OptionsCont& oc, NBPTStopCont& sc) {
     OutputDevice& device = OutputDevice::getDevice(oc.getString("ptstop-output"));
     device.writeXMLHeader("additional", "additional_file.xsd");
-    for (std::map<std::string, NBPTStop*>::const_iterator i = sc.begin(); i != sc.end(); ++i) {
-        i->second->write(device);
+    for (const auto& stopIt : sc.getStops()) {
+        stopIt.second->write(device);
     }
     device.close();
 }
-void NWWriter_XML::writePTLines(const OptionsCont& oc, NBPTLineCont& lc, NBEdgeCont& ec) {
+
+
+void NWWriter_XML::writePTLines(const OptionsCont& oc, NBPTLineCont& lc) {
     OutputDevice& device = OutputDevice::getDevice(oc.getString("ptline-output"));
     device.writeXMLHeader("ptLines", "ptlines_file.xsd");
     for (const auto& item : lc.getLines()) {
-        item.second->write(device, ec);
+        item.second->write(device);
     }
     device.close();
 }
 
+
 void NWWriter_XML::writeParkingAreas(const OptionsCont& oc, NBParkingCont& pc, NBEdgeCont& ec) {
     OutputDevice& device = OutputDevice::getDevice(oc.getString("parking-output"));
     device.writeXMLHeader("additional", "additional_file.xsd");
@@ -454,6 +521,7 @@ void NWWriter_XML::writeParkingAreas(const OptionsCont& oc, NBParkingCont& pc, N
     device.close();
 }
 
+
 void
 NWWriter_XML::writeDistricts(const OptionsCont& oc, NBDistrictCont& dc) {
     OutputDevice& device = OutputDevice::getDevice(oc.getString("taz-output"));
@@ -463,6 +531,7 @@ NWWriter_XML::writeDistricts(const OptionsCont& oc, NBDistrictCont& dc) {
     }
 }
 
+
 void
 NWWriter_XML::writeShape(OutputDevice& out, const GeoConvHelper& gch, PositionVector shape, SumoXMLAttr attr, bool useGeo, bool geoAccuracy) {
     if (useGeo) {
@@ -479,7 +548,5 @@ NWWriter_XML::writeShape(OutputDevice& out, const GeoConvHelper& gch, PositionVe
     }
 }
 
-/****************************************************************************/
-
 
 /****************************************************************************/
