@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -45,13 +45,15 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-RONetHandler::RONetHandler(RONet& net, ROAbstractEdgeBuilder& eb, const bool ignoreInternal, const double minorPenalty) :
+RONetHandler::RONetHandler(RONet& net, ROAbstractEdgeBuilder& eb, const bool ignoreInternal, const double minorPenalty, double tlsPenalty, double turnaroundPenalty) :
     SUMOSAXHandler("sumo-network"),
     myNet(net),
-    myNetworkVersion(0),
+    myNetworkVersion(0, 0),
     myEdgeBuilder(eb), myIgnoreInternal(ignoreInternal),
     myCurrentName(), myCurrentEdge(nullptr), myCurrentStoppingPlace(nullptr),
-    myMinorPenalty(minorPenalty)
+    myMinorPenalty(minorPenalty),
+    myTLSPenalty(tlsPenalty),
+    myTurnaroundPenalty(turnaroundPenalty)
 {}
 
 
@@ -67,7 +69,7 @@ RONetHandler::myStartElement(int element,
             break;
         case SUMO_TAG_NET: {
             bool ok;
-            myNetworkVersion = attrs.get<double>(SUMO_ATTR_VERSION, nullptr, ok, false);
+            myNetworkVersion = StringUtils::toVersion(attrs.get<std::string>(SUMO_ATTR_VERSION, nullptr, ok, false));
             break;
         }
         case SUMO_TAG_EDGE:
@@ -115,10 +117,81 @@ RONetHandler::myStartElement(int element,
             const SUMOVehicleClass svc = getVehicleClassID(attrs.get<std::string>(SUMO_ATTR_VCLASS, myCurrentTypeID.c_str(), ok));
             const double speed = attrs.get<double>(SUMO_ATTR_SPEED, myCurrentTypeID.c_str(), ok);
             if (ok) {
-                myNet.addRestriction(myCurrentTypeID, svc, speed);
+                myNet.addSpeedRestriction(myCurrentTypeID, svc, speed);
             }
             break;
         }
+        case SUMO_TAG_PREFERENCE: {
+            bool ok = true;
+            const std::string routingType = attrs.get<std::string>(SUMO_ATTR_ROUTINGTYPE, nullptr, ok);
+            const double prio = attrs.get<double>(SUMO_ATTR_PRIORITY, routingType.c_str(), ok);
+            if (prio <= 0) {
+                throw InvalidArgument("In preference for routingType '" + routingType + "', priority must be positve");
+            }
+            if (attrs.hasAttribute(SUMO_ATTR_VCLASSES)) {
+                StringTokenizer st(attrs.get<std::string>(SUMO_ATTR_VCLASSES, routingType.c_str(), ok));
+                for (std::string className : st.getVector()) {
+                    myNet.addPreference(routingType, getVehicleClassID(className), prio);
+                }
+            } else if (!attrs.hasAttribute(SUMO_ATTR_VTYPES)) {
+                // general preferenze applying to all types and vClasses
+                myNet.addPreference(routingType, "", prio);
+            }
+            if (attrs.hasAttribute(SUMO_ATTR_VTYPES)) {
+                StringTokenizer st(attrs.get<std::string>(SUMO_ATTR_VTYPES, routingType.c_str(), ok));
+                for (std::string typeName : st.getVector()) {
+                    myNet.addPreference(routingType, typeName, prio);
+                }
+            }
+            break;
+        }
+        case SUMO_TAG_REROUTER: {
+            bool ok = true;
+            myRerouterID = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
+            break;
+        }
+        case SUMO_TAG_INTERVAL:
+            if (myRerouterID != "") {
+                bool ok = true;
+                myIntervalBegin = attrs.getOptSUMOTimeReporting(SUMO_ATTR_BEGIN, myRerouterID.c_str(), ok, -1);
+                myIntervalEnd = attrs.getOptSUMOTimeReporting(SUMO_ATTR_END, myRerouterID.c_str(), ok, SUMOTime_MAX);
+                if (myIntervalEnd <= myIntervalBegin) {
+                    throw ProcessError(TLF("Invalid rerouter interval from % to %", time2string(myIntervalBegin), time2string(myIntervalEnd)));
+                }
+            }
+            break;
+        case SUMO_TAG_CLOSING_REROUTE: {
+            const std::string& closed_id = attrs.getStringSecure(SUMO_ATTR_ID, "");
+            ROEdge* closedEdge = myNet.getEdge(closed_id);
+            if (closedEdge == nullptr) {
+                throw ProcessError(TLF("rerouter '%': Edge '%' to close is not known.", myRerouterID, closed_id));
+            }
+            bool ok;
+            const std::string allow = attrs.getOpt<std::string>(SUMO_ATTR_ALLOW, myRerouterID.c_str(), ok, "", false);
+            const std::string disallow = attrs.getOpt<std::string>(SUMO_ATTR_DISALLOW, myRerouterID.c_str(), ok, "");
+            RouterProhibition prohibition;
+            prohibition.permissions = parseVehicleClasses(allow, disallow);
+            prohibition.begin = STEPS2TIME(myIntervalBegin);
+            prohibition.end = STEPS2TIME(attrs.getOptSUMOTimeReporting(SUMO_ATTR_UNTIL, nullptr, ok, myIntervalEnd));
+            myNet.addProhibition(closedEdge, prohibition);
+            break;
+        }
+        case SUMO_TAG_CLOSING_LANE_REROUTE: {
+            const std::string& closed_id = attrs.getStringSecure(SUMO_ATTR_ID, "");
+            ROLane* closedLane = myNet.getLane(closed_id);
+            if (closedLane == nullptr) {
+                throw ProcessError(TLF("rerouter '%': Lane '%' to close is not known.", myRerouterID, closed_id));
+            }
+            bool ok;
+            const std::string allow = attrs.getOpt<std::string>(SUMO_ATTR_ALLOW, myRerouterID.c_str(), ok, "", false);
+            const std::string disallow = attrs.getOpt<std::string>(SUMO_ATTR_DISALLOW, myRerouterID.c_str(), ok, "");
+            RouterProhibition prohibition;
+            prohibition.permissions = parseVehicleClasses(allow, disallow);
+            prohibition.begin = STEPS2TIME(myIntervalBegin);
+            prohibition.end = STEPS2TIME(attrs.getOptSUMOTimeReporting(SUMO_ATTR_UNTIL, nullptr, ok, myIntervalEnd));
+            myNet.addLaneProhibition(closedLane, prohibition);
+            break;
+        }
         case SUMO_TAG_PARAM:
             addParam(attrs);
             break;
@@ -131,10 +204,13 @@ RONetHandler::myStartElement(int element,
 void
 RONetHandler::myEndElement(int element) {
     switch (element) {
+        case SUMO_TAG_REROUTER:
+            myRerouterID = "";
+            break;
         case SUMO_TAG_NET:
             // build junction graph
             for (std::set<std::string>::const_iterator it = myUnseenNodeIDs.begin(); it != myUnseenNodeIDs.end(); ++it) {
-                WRITE_ERROR("Unknown node '" + *it + "'.");
+                WRITE_ERRORF(TL("Unknown node '%'."), *it);
             }
             break;
         default:
@@ -164,9 +240,8 @@ RONetHandler::parseEdge(const SUMOSAXAttributes& attrs) {
     if (!ok) {
         throw ProcessError();
     }
-    const SumoXMLEdgeFunc func = attrs.getEdgeFunc(ok);
+    const SumoXMLEdgeFunc func = attrs.getOpt<SumoXMLEdgeFunc>(SUMO_ATTR_FUNCTION, myCurrentName.c_str(), ok, SumoXMLEdgeFunc::NORMAL);
     if (!ok) {
-        WRITE_ERROR("Edge '" + myCurrentName + "' has an unknown type.");
         return;
     }
     // get the edge
@@ -176,10 +251,10 @@ RONetHandler::parseEdge(const SUMOSAXAttributes& attrs) {
     myCurrentEdge = nullptr;
     if (func == SumoXMLEdgeFunc::INTERNAL || func == SumoXMLEdgeFunc::CROSSING || func == SumoXMLEdgeFunc::WALKINGAREA) {
         assert(myCurrentName[0] == ':');
-        const std::string junctionID = myCurrentName.substr(1, myCurrentName.rfind('_') - 1);
+        const std::string junctionID = SUMOXMLDefinitions::getJunctionIDFromInternalEdge(myCurrentName);
         from = junctionID;
         to = junctionID;
-        priority = 0;
+        priority = -1;
     } else {
         from = attrs.get<std::string>(SUMO_ATTR_FROM, myCurrentName.c_str(), ok);
         to = attrs.get<std::string>(SUMO_ATTR_TO, myCurrentName.c_str(), ok);
@@ -200,10 +275,10 @@ RONetHandler::parseEdge(const SUMOSAXAttributes& attrs) {
         toNode = new RONode(to);
         myNet.addNode(toNode);
     }
+    const std::string type = attrs.getOpt<std::string>(SUMO_ATTR_TYPE, myCurrentName.c_str(), ok, "");
+    const std::string routingType = attrs.getOpt<std::string>(SUMO_ATTR_ROUTINGTYPE, myCurrentName.c_str(), ok, "");
     // build the edge
-    myCurrentEdge = myEdgeBuilder.buildEdge(myCurrentName, fromNode, toNode, priority);
-    // set the type
-    myCurrentEdge->setRestrictions(myNet.getRestrictions(attrs.getOpt<std::string>(SUMO_ATTR_TYPE, myCurrentName.c_str(), ok, "")));
+    myCurrentEdge = myEdgeBuilder.buildEdge(myCurrentName, fromNode, toNode, priority, type, routingType);
     myCurrentEdge->setFunction(func);
 
     if (myNet.addEdge(myCurrentEdge)) {
@@ -241,7 +316,7 @@ RONetHandler::parseLane(const SUMOSAXAttributes& attrs) {
         return;
     }
     if (shape.size() < 2) {
-        WRITE_ERROR("Ignoring lane '" + id + "' with broken shape.");
+        WRITE_ERRORF(TL("Ignoring lane '%' with broken shape."), id);
         return;
     }
     // get the length
@@ -264,7 +339,8 @@ RONetHandler::parseJunction(const SUMOSAXAttributes& attrs) {
     bool ok = true;
     // get the id, report an error if not given or empty...
     std::string id = attrs.get<std::string>(SUMO_ATTR_ID, nullptr, ok);
-    if (attrs.getNodeType(ok) == SumoXMLNodeType::INTERNAL) {
+    const SumoXMLNodeType type = attrs.get<SumoXMLNodeType>(SUMO_ATTR_TYPE, id.c_str(), ok);
+    if (type == SumoXMLNodeType::INTERNAL) {
         return;
     }
     myUnseenNodeIDs.erase(id);
@@ -277,7 +353,7 @@ RONetHandler::parseJunction(const SUMOSAXAttributes& attrs) {
     }
     RONode* n = myNet.getNode(id);
     if (n == nullptr) {
-        WRITE_WARNING("Skipping isolated junction '" + id + "'.");
+        WRITE_WARNINGF(TL("Skipping isolated junction '%'."), id);
     } else {
         n->setPosition(Position(x, y, z));
     }
@@ -293,13 +369,14 @@ RONetHandler::parseConnection(const SUMOSAXAttributes& attrs) {
     const int toLane = attrs.get<int>(SUMO_ATTR_TO_LANE, nullptr, ok);
     std::string dir = attrs.get<std::string>(SUMO_ATTR_DIR, nullptr, ok);
     std::string viaID = attrs.getOpt<std::string>(SUMO_ATTR_VIA, nullptr, ok, "");
+    std::string tlID = attrs.getOpt<std::string>(SUMO_ATTR_TLID, nullptr, ok, "");
     ROEdge* from = myNet.getEdge(fromID);
     ROEdge* to = myNet.getEdge(toID);
     if (from == nullptr) {
-        throw ProcessError("unknown from-edge '" + fromID + "' in connection");
+        throw ProcessError(TLF("unknown from-edge '%' in connection", fromID));
     }
     if (to == nullptr) {
-        throw ProcessError("unknown to-edge '" + toID + "' in connection");
+        throw ProcessError(TLF("unknown to-edge '%' in connection", toID));
     }
     if ((int)from->getLanes().size() <= fromLane) {
         throw ProcessError("invalid fromLane '" + toString(fromLane) + "' in connection from '" + fromID + "'.");
@@ -308,12 +385,26 @@ RONetHandler::parseConnection(const SUMOSAXAttributes& attrs) {
         throw ProcessError("invalid toLane '" + toString(toLane) + "' in connection to '" + toID + "'.");
     }
     if (myIgnoreInternal || viaID == "") {
-        from->getLanes()[fromLane]->addOutgoingLane(to->getLanes()[toLane]);
+        std::string allow = attrs.getOpt<std::string>(SUMO_ATTR_ALLOW, nullptr, ok, "");
+        std::string disallow = attrs.getOpt<std::string>(SUMO_ATTR_DISALLOW, nullptr, ok, "");
+        ROEdge* dummyVia = nullptr;
+        SVCPermissions permissions;
+        if (allow == "" && disallow == "") {
+            permissions = SVC_UNSPECIFIED;
+        } else {
+            myNet.setPermissionsFound();
+            // dummyVia is only needed to hold permissions
+            permissions = parseVehicleClasses(allow, disallow);
+            dummyVia = new ROEdge("dummyVia_" + from->getLanes()[fromLane]->getID() + "->" + to->getLanes()[toLane]->getID(),
+                                  from->getToJunction(), from->getToJunction(), permissions);
+            myNet.addEdge(dummyVia);
+        }
+        from->getLanes()[fromLane]->addOutgoingLane(to->getLanes()[toLane], dummyVia);
         from->addSuccessor(to, nullptr, dir);
-    }  else {
-        ROEdge* const via = myNet.getEdge(viaID.substr(0, viaID.rfind('_')));
+    } else {
+        ROEdge* const via = myNet.getEdge(SUMOXMLDefinitions::getEdgeIDFromLane(viaID));
         if (via == nullptr) {
-            throw ProcessError("unknown via-edge '" + viaID + "' in connection");
+            throw ProcessError(TLF("unknown via-edge '%' in connection", viaID));
         }
         from->getLanes()[fromLane]->addOutgoingLane(to->getLanes()[toLane], via);
         from->addSuccessor(to, via, dir);
@@ -322,6 +413,15 @@ RONetHandler::parseConnection(const SUMOSAXAttributes& attrs) {
         if (state == LINKSTATE_MINOR || state == LINKSTATE_EQUAL || state == LINKSTATE_STOP || state == LINKSTATE_ALLWAY_STOP) {
             via->setTimePenalty(myMinorPenalty);
         }
+        if (dir == toString(LinkDirection::TURN) || dir == toString(LinkDirection::TURN_LEFTHAND)) {
+            via->setTimePenalty(myTurnaroundPenalty);
+        }
+        if (tlID != "") {
+            via->setTimePenalty(myTLSPenalty);
+        }
+    }
+    if (to->isCrossing()) {
+        to->setTimePenalty(myTLSPenalty);
     }
 }
 
@@ -365,11 +465,12 @@ RONetHandler::parseAccess(const SUMOSAXAttributes& attrs) {
         throw InvalidArgument("Unknown lane '" + lane + "' for access.");
     }
     if ((edge->getPermissions() & SVC_PEDESTRIAN) == 0) {
-        WRITE_WARNING("Ignoring invalid access from non-pedestrian edge '" + edge->getID() + "'.");
+        WRITE_WARNINGF(TL("Ignoring invalid access from non-pedestrian edge '%'."), edge->getID());
         return;
     }
-    double pos = attrs.getOpt<double>(SUMO_ATTR_POSITION, "access", ok, 0.);
-    const double length = attrs.getOpt<double>(SUMO_ATTR_LENGTH, "access", ok, -1);
+    const bool random = attrs.getOpt<std::string>(SUMO_ATTR_POSITION, "access", ok) == "random";
+    double pos = random ? edge->getLength() / 2. : attrs.getOpt<double>(SUMO_ATTR_POSITION, "access", ok, 0.);
+    double length = attrs.getOpt<double>(SUMO_ATTR_LENGTH, "access", ok, -1);
     const bool friendlyPos = attrs.getOpt<bool>(SUMO_ATTR_FRIENDLY_POS, "access", ok, false);
     if (!ok || (SUMORouteHandler::checkStopPos(pos, pos, edge->getLength(), 0., friendlyPos) != SUMORouteHandler::StopPos::STOPPOS_VALID)) {
         throw InvalidArgument("Invalid position " + toString(pos) + " for access on lane '" + lane + "'.");
@@ -377,6 +478,12 @@ RONetHandler::parseAccess(const SUMOSAXAttributes& attrs) {
     if (!ok) {
         throw ProcessError();
     }
+    if (length < 0) {
+        const Position accPos = myNet.getLane(lane)->geometryPositionAtOffset(pos);
+        const double stopCenter = (myCurrentStoppingPlace->startPos + myCurrentStoppingPlace->endPos) / 2;
+        const Position stopPos = myNet.getLane(myCurrentStoppingPlace->lane)->geometryPositionAtOffset(stopCenter);
+        length  = accPos.distanceTo(stopPos);
+    }
     myCurrentStoppingPlace->accessPos.push_back(std::make_tuple(lane, pos, length));
 }
 
@@ -389,12 +496,14 @@ RONetHandler::parseDistrict(const SUMOSAXAttributes& attrs) {
     if (!ok) {
         return;
     }
-    myNet.addDistrict(myCurrentName, myEdgeBuilder.buildEdge(myCurrentName + "-source", nullptr, nullptr, 0), myEdgeBuilder.buildEdge(myCurrentName + "-sink", nullptr, nullptr, 0));
+    ROEdge* const sink = myEdgeBuilder.buildEdge(myCurrentName + "-sink", nullptr, nullptr, 0, "", "");
+    ROEdge* const source = myEdgeBuilder.buildEdge(myCurrentName + "-source", nullptr, nullptr, 0, "", "");
+    myNet.addDistrict(myCurrentName, source, sink);
     if (attrs.hasAttribute(SUMO_ATTR_EDGES)) {
-        std::vector<std::string> desc = attrs.getStringVector(SUMO_ATTR_EDGES);
-        for (std::vector<std::string>::const_iterator i = desc.begin(); i != desc.end(); ++i) {
-            myNet.addDistrictEdge(myCurrentName, *i, true);
-            myNet.addDistrictEdge(myCurrentName, *i, false);
+        const std::vector<std::string>& desc = attrs.get<std::vector<std::string> >(SUMO_ATTR_EDGES, myCurrentName.c_str(), ok);
+        for (const std::string& eID : desc) {
+            myNet.addDistrictEdge(myCurrentName, eID, true);
+            myNet.addDistrictEdge(myCurrentName, eID, false);
         }
     }
 }
