@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -43,47 +43,18 @@
 // ===========================================================================
 // method definitions
 // ===========================================================================
-bool
-TraCIServerAPI_Edge::processGet(TraCIServer& server, tcpip::Storage& inputStorage,
-                                tcpip::Storage& outputStorage) {
-    const int variable = inputStorage.readUnsignedByte();
-    const std::string id = inputStorage.readString();
-    server.initWrapper(libsumo::RESPONSE_GET_EDGE_VARIABLE, variable, id);
-    try {
-        if (!libsumo::Edge::handleVariable(id, variable, &server, &inputStorage)) {
-            switch (variable) {
-                case libsumo::VAR_EDGE_TRAVELTIME: {
-                    const double time = StoHelp::readTypedDouble(inputStorage, "The message must contain the time definition.");
-                    StoHelp::writeTypedDouble(server.getWrapperStorage(), libsumo::Edge::getAdaptedTraveltime(id, time));
-                    break;
-                }
-                case libsumo::VAR_EDGE_EFFORT: {
-                    const double time = StoHelp::readTypedDouble(inputStorage, "The message must contain the time definition.");
-                    StoHelp::writeTypedDouble(server.getWrapperStorage(), libsumo::Edge::getEffort(id, time));
-                    break;
-                }
-                default:
-                    return server.writeErrorStatusCmd(libsumo::CMD_GET_EDGE_VARIABLE,
-                                                      "Get Edge Variable: unsupported variable " + toHex(variable, 2)
-                                                      + " specified", outputStorage);
-            }
-        }
-    } catch (libsumo::TraCIException& e) {
-        return server.writeErrorStatusCmd(libsumo::CMD_GET_EDGE_VARIABLE, e.what(), outputStorage);
-    }
-    server.writeStatusCmd(libsumo::CMD_GET_EDGE_VARIABLE, libsumo::RTYPE_OK, "", outputStorage);
-    server.writeResponseWithLength(outputStorage, server.getWrapperStorage());
-    return true;
-}
-
-
 bool
 TraCIServerAPI_Edge::processSet(TraCIServer& server, tcpip::Storage& inputStorage,
                                 tcpip::Storage& outputStorage) {
     std::string warning; // additional description for response
     // variable
     int variable = inputStorage.readUnsignedByte();
-    if (variable != libsumo::VAR_EDGE_TRAVELTIME && variable != libsumo::VAR_EDGE_EFFORT && variable != libsumo::VAR_MAXSPEED
+    if (variable != libsumo::VAR_EDGE_TRAVELTIME
+            && variable != libsumo::VAR_EDGE_EFFORT
+            && variable != libsumo::VAR_MAXSPEED
+            && variable != libsumo::LANE_ALLOWED
+            && variable != libsumo::LANE_DISALLOWED
+            && variable != libsumo::VAR_FRICTION
             && variable != libsumo::VAR_PARAMETER) {
         return server.writeErrorStatusCmd(libsumo::CMD_SET_EDGE_VARIABLE,
                                           "Change Edge State: unsupported variable " + toHex(variable, 2)
@@ -97,13 +68,13 @@ TraCIServerAPI_Edge::processSet(TraCIServer& server, tcpip::Storage& inputStorag
             case libsumo::LANE_ALLOWED: {
                 // read and set allowed vehicle classes
                 const std::vector<std::string> classes = StoHelp::readTypedStringList(inputStorage, "Allowed vehicle classes must be given as a list of strings.");
-                libsumo::Edge::setAllowedVehicleClasses(id, classes);
+                libsumo::Edge::setAllowed(id, classes);
                 break;
             }
             case libsumo::LANE_DISALLOWED: {
                 // read and set disallowed vehicle classes
                 const std::vector<std::string> classes = StoHelp::readTypedStringList(inputStorage, "Not allowed vehicle classes must be given as a list of strings.");
-                libsumo::Edge::setDisallowedVehicleClasses(id, classes);
+                libsumo::Edge::setDisallowed(id, classes);
                 break;
             }
             case libsumo::VAR_EDGE_TRAVELTIME: {
@@ -152,6 +123,12 @@ TraCIServerAPI_Edge::processSet(TraCIServer& server, tcpip::Storage& inputStorag
                 libsumo::Edge::setMaxSpeed(id, value);
                 break;
             }
+            case libsumo::VAR_FRICTION: {
+                // read and set friction for entire edge
+                const double value = StoHelp::readTypedDouble(inputStorage, "The friction must be given as a double.");
+                libsumo::Edge::setFriction(id, value);
+                break;
+            }
             case libsumo::VAR_PARAMETER: {
                 // read and check item number
                 StoHelp::readCompound(inputStorage, 2, "A compound object of size 2 is needed for setting a parameter.");
