@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -31,6 +31,7 @@
 #include <utils/common/SUMOVehicleClass.h>
 #include <utils/common/Named.h>
 #include <utils/distribution/Distribution_Parameterized.h>
+#include <utils/vehicle/SUMOVTypeParameter.h>
 #include "StdDefs.h"
 
 
@@ -107,6 +108,12 @@ inline std::string toString<LaneSpreadFunction>(const LaneSpreadFunction& lsf, s
     return SUMOXMLDefinitions::LaneSpreadFunctions.getString(lsf);
 }
 
+template <>
+inline std::string toString<ParkingType>(const ParkingType& pt, std::streamsize accuracy) {
+    UNUSED_PARAMETER(accuracy);
+    return SUMOXMLDefinitions::ParkingTypes.getString(pt);
+}
+
 template <>
 inline std::string toString<RightOfWay>(const RightOfWay& row, std::streamsize accuracy) {
     UNUSED_PARAMETER(accuracy);
@@ -119,6 +126,12 @@ inline std::string toString<FringeType>(const FringeType& fringeType, std::strea
     return SUMOXMLDefinitions::FringeTypeValues.getString(fringeType);
 }
 
+template <>
+inline std::string toString<RoundaboutType>(const RoundaboutType& roundaboutType, std::streamsize accuracy) {
+    UNUSED_PARAMETER(accuracy);
+    return SUMOXMLDefinitions::RoundaboutTypeValues.getString(roundaboutType);
+}
+
 template <>
 inline std::string toString<PersonMode>(const PersonMode& personMode, std::streamsize accuracy) {
     UNUSED_PARAMETER(accuracy);
@@ -153,6 +166,13 @@ inline std::string toString<TrafficLightLayout>(const TrafficLightLayout& layout
 }
 
 
+template <>
+inline std::string toString<InsertionCheck>(const InsertionCheck& check, std::streamsize accuracy) {
+    UNUSED_PARAMETER(accuracy);
+    return SUMOXMLDefinitions::InsertionChecks.getString(check);
+}
+
+
 template <>
 inline std::string toString<LaneChangeModel>(const LaneChangeModel& model, std::streamsize accuracy) {
     UNUSED_PARAMETER(accuracy);
@@ -160,9 +180,26 @@ inline std::string toString<LaneChangeModel>(const LaneChangeModel& model, std::
 }
 
 template <>
-inline std::string toString<LateralAlignment>(const LateralAlignment& latA, std::streamsize accuracy) {
+inline std::string toString<LatAlignmentDefinition>(const LatAlignmentDefinition& lad, std::streamsize accuracy) {
     UNUSED_PARAMETER(accuracy);
-    return SUMOXMLDefinitions::LateralAlignments.getString(latA);
+    switch (lad) {
+        case LatAlignmentDefinition::RIGHT:
+            return "right";
+        case LatAlignmentDefinition::CENTER:
+            return "center";
+        case LatAlignmentDefinition::ARBITRARY:
+            return "arbitrary";
+        case LatAlignmentDefinition::NICE:
+            return "nice";
+        case LatAlignmentDefinition::COMPACT:
+            return "compact";
+        case LatAlignmentDefinition::LEFT:
+            return "left";
+        case LatAlignmentDefinition::GIVEN:
+        case LatAlignmentDefinition::DEFAULT:
+        default:
+            return "";
+    }
 }
 
 template <>
@@ -194,6 +231,7 @@ inline std::string toString(const std::vector<V*>& v, std::streamsize accuracy =
     return toString<V>(v.begin(), v.end(), accuracy);
 }
 
+
 template <typename V>
 inline std::string toString(const typename std::vector<V*>::const_iterator& b, const typename std::vector<V*>::const_iterator& e, std::streamsize accuracy = gPrecision) {
     UNUSED_PARAMETER(accuracy);
@@ -280,6 +318,15 @@ inline std::string joinNamedToStringSorting(const std::set<T*>& ns, const T_BETW
     return joinToStringSorting(ids, between);
 }
 
+template <typename T, typename T_BETWEEN>
+inline std::string joinNamedToStringSorting(const std::set<T*, ComparatorIdLess>& ns, const T_BETWEEN& between) {
+    std::vector<std::string> ids;
+    for (T* n : ns) {
+        ids.push_back(Named::getIDSecure(n));
+    }
+    return joinToStringSorting(ids, between);
+}
+
 
 template <typename T, typename C, typename T_BETWEEN>
 inline std::string joinNamedToString(const std::set<T*, C>& ns, const T_BETWEEN& between) {
@@ -291,6 +338,22 @@ inline std::string joinNamedToString(const std::set<T*, C>& ns, const T_BETWEEN&
 }
 
 
+template <typename KEY, typename VAL, typename T_BETWEEN, typename T_BETWEEN_KEYVAL>
+inline std::string joinNamedToString(const std::map<KEY, VAL, ComparatorIdLess>& s, const T_BETWEEN& between, const T_BETWEEN_KEYVAL& between_keyval, std::streamsize accuracy = gPrecision) {
+    std::ostringstream oss;
+    bool connect = false;
+    for (typename std::map<KEY, VAL>::const_iterator it = s.begin(); it != s.end(); ++it) {
+        if (connect) {
+            oss << toString(between, accuracy);
+        } else {
+            connect = true;
+        }
+        oss << Named::getIDSecure(it->first) << between_keyval << toString(it->second, accuracy);
+    }
+    return oss.str();
+}
+
+
 template <typename V>
 inline std::string toString(const std::set<V*>& v, std::streamsize accuracy = gPrecision) {
     UNUSED_PARAMETER(accuracy);
@@ -302,6 +365,17 @@ inline std::string toString(const std::set<V*>& v, std::streamsize accuracy = gP
 }
 
 
+template <typename V>
+inline std::string toString(const std::set<V*, ComparatorNumericalIdLess>& v, std::streamsize accuracy = gPrecision) {
+    UNUSED_PARAMETER(accuracy);
+    std::vector<std::string> ids;
+    for (typename std::set<V*, ComparatorNumericalIdLess>::const_iterator it = v.begin(); it != v.end(); ++it) {
+        ids.push_back((*it)->getID());
+    }
+    return joinToStringSorting(ids, " ");
+}
+
+
 template <>
 inline std::string toString(const std::vector<int>& v, std::streamsize accuracy) {
     return joinToString(v, " ", accuracy);
@@ -320,6 +394,23 @@ inline std::string toString(const std::vector<double>& v, std::streamsize accura
 }
 
 
+template <typename V, typename W>
+inline std::string toString(const std::vector<std::pair<V, W> >& v, std::streamsize accuracy = gPrecision, const std::string& between = ";", const std::string& between2 = ",") {
+    std::ostringstream oss;
+    oss << std::setprecision(accuracy);
+    bool connect = false;
+    for (auto it : v) {
+        if (connect) {
+            oss << toString(between, accuracy);
+        } else {
+            connect = true;
+        }
+        oss << toString(it.first) << between2 << toString(it.second);
+    }
+    return oss.str();
+}
+
+
 template <typename T, typename T_BETWEEN>
 inline std::string joinToString(const std::set<T>& s, const T_BETWEEN& between, std::streamsize accuracy = gPrecision) {
     std::ostringstream oss;
@@ -365,6 +456,12 @@ inline std::string joinToString(const std::map<KEY, VAL>& s, const T_BETWEEN& be
 
 
 template <>
-inline std::string toString(const std::map<std::string, std::string>& v, std::streamsize) {
+inline std::string toString(const Parameterised::Map& v, std::streamsize) {
     return joinToString(v, ", ", ":");
 }
+
+template <>
+inline std::string toString(const MMVersion& v, std::streamsize) {
+    // we only need higher accuracy on the minor version for hotfix releases
+    return toString(v.first) + "." + toString(v.second, 0);
+}
