@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -47,10 +47,10 @@ public:
     *
     * A random sample is drawn according to the assigned probabilities.
     *
-    * @param[in] which The random number generator to use; the static one will be used if 0 is passed
+    * @param[in] which The random number generator to use; the static one will be used if nullptr is passed
     * @return the drawn member
     */
-    double sample(std::mt19937* which = 0) const {
+    double sample(SumoRNG* which = nullptr) const {
         return get(which);
     }
 
