@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2006-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2006-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -20,8 +20,9 @@
 #include <config.h>
 
 #include <utils/common/RGBColor.h>
-#include "MFXUtils.h"
+#include <utils/common/StringTokenizer.h>
 
+#include "MFXUtils.h"
 
 // ===========================================================================
 // method definitions
@@ -42,7 +43,7 @@ MFXUtils::userPermitsOverwritingWhenFileExists(FXWindow* const parent,
         return TRUE;
     }
     int answer =
-        FXMessageBox::question(parent, MBOX_YES_NO, "File Exists", "Overwrite '%s'?", file.text());
+        FXMessageBox::question(parent, MBOX_YES_NO, TL("File Exists"), TL("Overwrite '%s'?"), file.text());
     if (answer == MBOX_CLICKED_NO) {
         return FALSE;
     }
@@ -66,39 +67,71 @@ MFXUtils::getTitleText(const FXString& appname, FXString filename) {
 
 
 FXString
-MFXUtils::assureExtension(const FXString& filename, const FXString& defaultExtension) {
-    FXString ext = FXPath::extension(filename);
-    if (ext == "") {
-        if (filename.rfind('.') == filename.length() - 1) {
-            return filename + defaultExtension;
+MFXUtils::assureExtension(const FXFileDialog& openDialog) {
+    const auto extensions = parseExtensions(openDialog.getPatternText(openDialog.getCurrentPattern()));
+    const auto filename = openDialog.getFilename();
+    // iterate over all extension to check if is the same extension
+    for (const auto& extension : extensions) {
+        if (extension.length() < filename.length()) {
+            bool sameExtension = true;
+            for (auto i = 0; i < extension.length(); i++) {
+                if (filename[i + filename.length() - extension.length()] != extension[i]) {
+                    sameExtension = false;
+                }
+            }
+            if (sameExtension) {
+                return filename;
+            }
         }
-        return filename + "." + defaultExtension;
     }
-    return filename;
+    // in this point, we have to give an extension (if exist)
+    if (extensions.size() > 0) {
+        return filename + "." + extensions.front();
+    } else {
+        return filename;
+    }
+}
+
+
+std::vector<FXString>
+MFXUtils::parseExtensions(FXString patternText) {
+    std::vector<FXString> extensions;
+    // first take elementes between parentheses
+    patternText = patternText.after('(');
+    patternText = patternText.before(')');
+    // check files extension
+    if (patternText != "*") {
+        // split extensions
+        const auto extensionsStr = StringTokenizer(patternText.text(), ",").getVector();
+        for (const auto& extensionStr : extensionsStr) {
+            FXString extension = extensionStr.c_str();
+            extensions.push_back(extension.after('.'));
+        }
+    }
+    return extensions;
 }
 
 
 FXString
-MFXUtils::getFilename2Write(FXWindow* parent,
-                            const FXString& header, const FXString& extension,
+MFXUtils::getFilename2Write(FXWindow* parent, const FXString& header, const FXString& extensions,
                             FXIcon* icon, FXString& currentFolder) {
     // get the new file name
     FXFileDialog opendialog(parent, header);
     opendialog.setIcon(icon);
     opendialog.setSelectMode(SELECTFILE_ANY);
-    opendialog.setPatternList("*" + extension);
+    opendialog.setPatternList(extensions);
     if (currentFolder.length() != 0) {
         opendialog.setDirectory(currentFolder);
     }
     if (!opendialog.execute()) {
         return "";
     }
-    FXString file = assureExtension(opendialog.getFilename(), extension.after('.')).text();
-    if (!userPermitsOverwritingWhenFileExists(parent, file)) {
+    const auto filename = assureExtension(opendialog);
+    if (!userPermitsOverwritingWhenFileExists(parent, filename)) {
         return "";
     }
     currentFolder = opendialog.getDirectory();
-    return file;
+    return filename;
 }
 
 
