@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -29,6 +29,7 @@
 #include <utils/foxtools/fxheader.h>
 #include <utils/common/ValueSource.h>
 #include <utils/common/SUMOTime.h>
+#include <utils/gui/div/GUIPersistentWindowPos.h>
 #include "GUIParameterTableItem.h"
 
 
@@ -57,7 +58,7 @@ class Parameterised;
  *
  * Each row is represented using an instance of GUIParameterTableItemInterface.
  */
-class GUIParameterTableWindow : public FXMainWindow {
+class GUIParameterTableWindow : public FXMainWindow, public GUIPersistentWindowPos {
     FXDECLARE(GUIParameterTableWindow)
 public:
     /** @brief Constructor
@@ -65,7 +66,7 @@ public:
      * @param[in] app The application this window belongs to
      * @param[in] o The gl-object this table describes
      */
-    GUIParameterTableWindow(GUIMainWindow& app, GUIGlObject& o);
+    GUIParameterTableWindow(GUIMainWindow& app, GUIGlObject& o, const std::string& title = "");
 
 
     /// @brief Destructor
@@ -83,6 +84,8 @@ public:
     void closeBuilding(const Parameterised* p = 0);
 
 
+    /// @brief ensure that the font covers the given text
+    void checkFont(const std::string& text);
 
     /** @brief Lets this window know the object shown is being deleted
      * @param[in] o The deleted (shown) object
