@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -27,6 +27,7 @@
 #include <map>
 #include <utils/common/RGBColor.h>
 #include <utils/common/ToString.h>
+
 #include "GUIPropertySchemeStorage.h"
 
 
@@ -48,7 +49,7 @@ class GUIGlObject;
 struct GUIVisualizationTextSettings {
 
     /// @brief constructor
-    GUIVisualizationTextSettings(bool _show, double _size, RGBColor _color, RGBColor _bgColor = RGBColor(128, 0, 0, 0), bool _constSize = true);
+    GUIVisualizationTextSettings(bool _show, double _size, RGBColor _color, RGBColor _bgColor = RGBColor(128, 0, 0, 0), bool _constSize = true, bool _onlySelected = false);
 
     /// @brief equality comparator
     bool operator==(const GUIVisualizationTextSettings& other);
@@ -62,8 +63,11 @@ struct GUIVisualizationTextSettings {
     /// @brief get scale size
     double scaledSize(double scale, double constFactor = 0.1) const;
 
+    /// @brief whether to show the text
+    bool show(const GUIGlObject* o) const;
+
     /// @brief flag show
-    bool show;
+    bool showText;
 
     /// @brief text size
     double size;
@@ -74,8 +78,47 @@ struct GUIVisualizationTextSettings {
     /// @brief background text color
     RGBColor bgColor;
 
-    /// @brif flag to avoid size changes
+    /// @brief flag to avoid size changes
     bool constSize;
+
+    /// @brief whether only selected objects shall have text drawn
+    bool onlySelected;
+};
+
+
+struct GUIVisualizationRainbowSettings {
+
+    /// @brief constructor
+    GUIVisualizationRainbowSettings(bool _hideMin, double _minThreshold, bool _hideMax, double _maxThreshold, bool _setNeutral,
+                                    double _neutralThreshold, bool _fixRange, int _rainboScheme);
+
+    /// @brief equality comparator
+    bool operator==(const GUIVisualizationRainbowSettings& other);
+
+    /// @brief inequality comparator
+    bool operator!=(const GUIVisualizationRainbowSettings& other);
+
+    /// @brief print values in output device
+    void print(OutputDevice& dev, const std::string& name) const;
+
+    /// @brief whether data below threshold should not be colored
+    bool hideMin;
+    /// @brief threshold below which value should not be colored
+    double minThreshold;
+    /// @brief whether data above threshold should not be colored
+    bool hideMax;
+    /// @brief threshold above which value should not be colored
+    double maxThreshold;
+    /// @brief whether the scale should be centered at a specific value
+    bool setNeutral;
+    /// @brief neutral point of scale
+    double neutralThreshold;
+    /// @brief whether the color scale should be fixed to the given min/max values
+    bool fixRange;
+    /// @brief index in the list of color schemes
+    int rainbowScheme;
+    /// @brief color steps for the rainbow;
+    std::vector<RGBColor> colors;
 };
 
 
@@ -150,50 +193,108 @@ struct GUIVisualizationColorSettings {
     /// @brief vehicle selection color
     RGBColor selectedVehicleColor;
 
-    /// @brief person  selection color
+    /// @brief person selection color
     RGBColor selectedPersonColor;
 
-    /// @brief person plan selection color (Rides, Walks, personStops...)
+    /// @brief person plan selection color (Rides, Walks, stopPersons...)
     RGBColor selectedPersonPlanColor;
 
+    /// @brief container selection color
+    RGBColor selectedContainerColor;
+
+    /// @brief container plan selection color (Rides, Walks, containerStops...)
+    RGBColor selectedContainerPlanColor;
+
     /// @brief edge data selection color
     RGBColor selectedEdgeDataColor;
 
+    /// @brief color for busStops
+    RGBColor busStopColor;
+
+    /// @brief color for busStops signs
+    RGBColor busStopColorSign;
+
+    /// @brief color for trainStops
+    RGBColor trainStopColor;
+
+    /// @brief color for trainStops signs
+    RGBColor trainStopColorSign;
+
+    /// @brief color for containerStops
+    RGBColor containerStopColor;
+
+    /// @brief color for containerStop signs
+    RGBColor containerStopColorSign;
+
+    /// @brief color for chargingStations
+    RGBColor chargingStationColor;
+
+    /// @brief color for chargingStation sign
+    RGBColor chargingStationColorSign;
+
+    /// @brief color for chargingStation during charging
+    RGBColor chargingStationColorCharge;
+
+    /// @brief color for parkingAreas
+    RGBColor parkingAreaColor;
+
+    /// @brief color for parkingArea sign
+    RGBColor parkingAreaColorSign;
+
+    /// @brief color for parkingSpace contour
+    RGBColor parkingSpaceColorContour;
+
+    /// @brief color for parkingSpace innen
+    RGBColor parkingSpaceColor;
+
+    /// @brief color for Stops
+    RGBColor stopColor;
+
+    /// @brief color for Waypoints
+    RGBColor waypointColor;
+
+    /// @brief color for vehicle trips
+    RGBColor vehicleTripColor;
+
+    /// @brief color for stopPersons
+    RGBColor stopPersonColor;
+
+    /// @brief color for stopPersons
+    RGBColor personTripColor;
+
+    /// @brief color for walks
+    RGBColor walkColor;
+
+    /// @brief color for rides
+    RGBColor rideColor;
+
+    /// @brief color for containerStops
+    RGBColor stopContainerColor;
+
+    /// @brief color for transport
+    RGBColor transportColor;
+
+    /// @brief color for tranships
+    RGBColor transhipColor;
+
     /// @brief color for highlighthing deadends
     static const RGBColor SUMO_color_DEADEND_SHOW;
 
     /// @brief color for child connections between parents and child elements
-    static const RGBColor childConnections;
+    static const RGBColor childConnectionColor;
 
     /// @brief color for edited shapes (Junctions, crossings and connections)
-    static const RGBColor editShape;
+    static const RGBColor editShapeColor;
 
     /// @brief color for crossings
-    static const RGBColor crossing;
+    static const RGBColor crossingColor;
 
     /// @brief color for priority crossing
-    static const RGBColor crossingPriority;
+    static const RGBColor crossingPriorityColor;
 
     /// @brief color for invalid crossing
-    static const RGBColor crossingInvalid;
+    static const RGBColor crossingInvalidColor;
 
-    /// @brief color for Stops
-    static const RGBColor stops;
-
-    /// @brief color for vehicle trips
-    static const RGBColor vehicleTrips;
-
-    /// @brief color for personStops
-    static const RGBColor personStops;
-
-    /// @brief color for personStops
-    static const RGBColor personTrip;
-
-    /// @brief color for walks
-    static const RGBColor walk;
-
-    /// @brief color for rides
-    static const RGBColor ride;
 };
 
 
@@ -214,12 +315,15 @@ struct GUIVisualizationCandidateColorSettings {
 
     /// @brief color for selected conflict candidate element (Usually selected using ctrl+click)
     static const RGBColor conflict;
+
+    /// @brief color for invalid elements
+    static const RGBColor invalid;
 };
 
 /// @brief struct for connection settings
 struct GUIVisualizationNeteditSizeSettings {
 
-    /// @brief junction buuble radius
+    /// @brief junction bubble radius
     static const double junctionBubbleRadius;
 
     /// @brief moving junction geometry point radius
@@ -228,6 +332,9 @@ struct GUIVisualizationNeteditSizeSettings {
     /// @brief moving edge geometry point radius
     static const double edgeGeometryPointRadius;
 
+    /// @brief moving lane geometry point radius
+    static const double laneGeometryPointRadius;
+
     /// @brief moving connection geometry point radius
     static const double connectionGeometryPointRadius;
 
@@ -242,6 +349,9 @@ struct GUIVisualizationNeteditSizeSettings {
 
     /// @brief poly line width
     static const double polylineWidth;
+
+    /// @brief moving additional geometry point radius
+    static const double additionalGeometryPointRadius;
 };
 
 /// @brief struct for connection settings
@@ -287,6 +397,39 @@ struct GUIVisualizationAdditionalSettings {
 
     /// @brief Vaporizer size
     static const double vaporizerSize;
+
+    /// @brief stopEdges size
+    static const double stopEdgeSize;
+
+    /// @brief connection color
+    static const RGBColor connectionColor;
+
+    /// @brief connection color selected
+    static const RGBColor connectionColorSelected;
+
+    /// @brief tractionSubstation size
+    static const double tractionSubstationSize;
+
+    /// @brief overhead wire color top
+    static const RGBColor overheadWireColorTop;
+
+    /// @brief overhead wire color bot
+    static const RGBColor overheadWireColorBot;
+
+    /// @brief overhead wire color selected
+    static const RGBColor overheadWireColorSelected;
+
+    /// @brief arrow width
+    static const double arrowWidth;
+
+    /// @brief arrow length
+    static const double arrowLength;
+
+    /// @brief arrow offset
+    static const double arrowOffset;
+
+    /// @brief connection color between E1/junctions and TLS
+    static const RGBColor TLSConnectionColor;
 };
 
 
@@ -336,51 +479,29 @@ struct GUIVisualizationDetectorSettings {
 
 /// @brief struct for stoppingPlace settings
 struct GUIVisualizationStoppingPlaceSettings {
-
     /// @brief busStop offset
     static const double stoppingPlaceSignOffset;
 
     /// @brief busStop width
     static const double busStopWidth;
 
-    /// @brief color for busStops
-    static const RGBColor busStopColor;
-
-    /// @brief color for busStops signs
-    static const RGBColor busStopColorSign;
+    /// @brief trainStop width
+    static const double trainStopWidth;
 
     /// @brief containerStop width
     static const double containerStopWidth;
 
-    /// @brief color for containerStops
-    static const RGBColor containerStopColor;
-
-    /// @brief color for containerStop signs
-    static const RGBColor containerStopColorSign;
-
     /// @brief chargingStation width
     static const double chargingStationWidth;
 
-    /// @brief color for chargingStations
-    static const RGBColor chargingStationColor;
+    /// @brief symbol external radius
+    static const double symbolExternalRadius;
 
-    /// @brief color for chargingStation sign
-    static const RGBColor chargingStationColorSign;
+    /// @brief symbol internal radius
+    static const double symbolInternalRadius;
 
-    /// @brief color for chargingStation during charging
-    static const RGBColor chargingStationColorCharge;
-
-    /// @brief color for parkingAreas
-    static const RGBColor parkingAreaColor;
-
-    /// @brief color for parkingArea sign
-    static const RGBColor parkingAreaColorSign;
-
-    /// @brief color for parkingSpace contour
-    static const RGBColor parkingSpaceColorContour;
-
-    /// @brief color for parkingSpace innen
-    static const RGBColor parkingSpaceColor;
+    /// @brief symbol internal text size
+    static const double symbolInternalTextSize;
 };
 
 
@@ -390,6 +511,12 @@ struct GUIVisualizationDottedContourSettings {
     /// @brief width of dotted contour segments
     static const double segmentWidth;
 
+    /// @brief width of small dotted contour segments
+    static const double segmentWidthSmall;
+
+    /// @brief width of large dotted contour segments
+    static const double segmentWidthLarge;
+
     /// @brief length of dotted contour segments
     static const double segmentLength;
 
@@ -410,23 +537,38 @@ struct GUIVisualizationDottedContourSettings {
 /// @brief struct for width settings
 struct GUIVisualizationWidthSettings {
 
-    /// @brief width for routes
-    static const double route;
+    /// @brief constructor
+    GUIVisualizationWidthSettings();
 
-    /// @brief width for embeddedroutes
-    static const double embeddedRoute;
+    /// @brief equality comparator
+    bool operator==(const GUIVisualizationWidthSettings& other);
+
+    /// @brief inequality comparator
+    bool operator!=(const GUIVisualizationWidthSettings& other);
 
     /// @brief width for trips
-    static const double trip;
+    double tripWidth;
 
     /// @brief width for person trips
-    static const double personTrip;
+    double personTripWidth;
 
     /// @brief width for walks
-    static const double walk;
+    double walkWidth;
 
     /// @brief width for rides
-    static const double ride;
+    double rideWidth;
+
+    /// @brief width for rides
+    double transportWidth;
+
+    /// @brief width for rides
+    double transhipWidth;
+
+    /// @brief width for routes
+    static const double routeWidth;
+
+    /// @brief width for embeddedroutes
+    static const double embeddedRouteWidth;
 };
 
 
@@ -434,64 +576,7 @@ struct GUIVisualizationWidthSettings {
 struct GUIVisualizationDetailSettings {
 
     /// @brief draw connections in demand mode
-    static const double connectionsDemandMode;
-
-    /// @brief details for lane textures
-    static const double laneTextures;
-
-    /// @brief lock icons
-    static const double lockIcon;
-
-    /// @brief details for additional textures
-    static const double additionalTextures;
-
-    /// @brief details for Geometry Points
-    static const double geometryPointsDetails;
-
-    /// @brief details for Geometry Points Texts
-    static const double geometryPointsText;
-
-    /// @brief details for stopping places
-    static const double stoppingPlaceDetails;
-
-    /// @brief details for stopping place texts
-    static const double stoppingPlaceText;
-
-    /// @brief details for detectors
-    static const double detectorDetails;
-
-    /// @brief details for detector texts
-    static const double detectorText;
-
-    /// @brief details for calibrator text
-    static const double calibratorText;
-
-    /// @brief details for stops
-    static const double stopsDetails;
-
-    /// @brief details for stop texts
-    static const double stopsText;
-
-    /// @brief details for draw person as triangles
-    static const double vehicleTriangles;
-
-    /// @brief details for draw person as boxes
-    static const double vehicleBoxes;
-
-    /// @brief details for draw person as shapes
-    static const double vehicleShapes;
-
-    /// @brief details for draw person as triangles
-    static const double personTriangles;
-
-    /// @brief details for draw person as circles
-    static const double personCircles;
-
-    /// @brief details for draw person as person shapes
-    static const double personShapes;
-
-    /// @brief Exaggeration for persons (only used in NETEDIT)
-    static const double personExaggeration;
+    static const double tmp;
 };
 
 
@@ -502,8 +587,74 @@ struct GUIVisualizationDetailSettings {
 class GUIVisualizationSettings {
 
 public:
+
+    enum class Detail : int {
+        Level0 = 0,
+        CircleResolution32 = 0,     // circle resolution = 32
+        DrawPolygonTesselation = 0, // draw polygons tesselated
+        LaneDetails = 0,            // offset, icons, indicators...
+        Text = 0,                   // draw text (E2, routes...)
+        VehiclePoly = 0,            // draw vehicles as polygons
+        JunctionElementDetails = 0, // draw junction elements with high detail (crossings, connections..)
+        LockedIcons = 0,            // draw lock icons
+
+        Level1 = 1,
+        CircleResolution16 = 1,         // circle resolution = 16
+        VehicleBox = 1,                 // vehicles as boxes
+        AdditionalDetails = 1,          // stoppingPlace signs, EntryExit arrows...
+        GeometryPoint = 1,              // draw geometry points
+        JunctionElement = 1,            // crossings, walking area, connections and internal lanes
+        DottedContoursResampled = 1,    // resample dotted contours
+        PreciseSelection = 1,           // precise selection using boundaries
+        DottedContours = 1,             // draw dotted contours
+
+        Level2 = 2,
+        CircleResolution8 = 2,  // circle resolution = 8
+        DrawPolygonSquare = 2,  // draw polygons as squares
+        VehicleTriangle = 2,    // draw vehicles as triangles
+        Additionals = 2,        // draw additional elements
+        GeometryBoxLines = 2,   // draw lines instead boxes in GUIGeometry::drawGeometry
+
+        Level3 = 3,
+        CircleResolution4 = 3,  // draw circle resolution as squares
+        TLSIcon = 3,            // draw TLS icons
+
+        Level4 = 4,
+        GeometryBoxSimpleLine = 4,  // draw lines with width = 1 instead boxes in GUIGeometry::drawGeometry
+    };
+
     /// @brief constructor
-    GUIVisualizationSettings(bool _netedit = false);
+    GUIVisualizationSettings(const std::string& _name, bool _netedit = false);
+
+    /// @brief check if draw junction
+    bool checkDrawJunction(const Boundary& b, const bool selected) const;
+
+    /// @brief check if draw edge
+    bool checkDrawEdge(const Boundary& b) const;
+
+    /// @brief update ignore hide by zoom (call BEFORE drawing all elements).
+    void updateIgnoreHideByZoom();
+
+    /// @brief check if draw additionals
+    bool checkDrawAdditional(Detail d, const bool selected) const;
+
+    /// @brief check if draw polygon
+    bool checkDrawPoly(const Boundary& b, const bool selected) const;
+
+    /// @brief check if draw POI
+    bool checkDrawPOI(const double w, const double h, const Detail d, const bool selected) const;
+
+    /// @brief check if draw vehicle
+    bool checkDrawVehicle(Detail d, const bool selected) const;
+
+    /// @brief check if draw person
+    bool checkDrawPerson(Detail d, const bool selected) const;
+
+    /// @brief check if draw container
+    bool checkDrawContainer(Detail d, const bool selected) const;
+
+    /// @brief copy all content from another GUIVisualizationSettings (note: DON'T USE in DrawGL functions!)
+    void copy(const GUIVisualizationSettings& s);
 
     /// @brief init default settings
     void initNeteditDefaults();
@@ -514,22 +665,22 @@ public:
      */
     void save(OutputDevice& dev) const;
 
-    /** @brief Returns the number of the active lane (edge) coloring schme
+    /** @brief Returns the number of the active lane (edge) coloring scheme
      * @return number of the active scheme
      */
     int getLaneEdgeMode() const;
 
-    /** @brief Returns the number of the active lane (edge) scaling schme
+    /** @brief Returns the number of the active lane (edge) scaling scheme
      * @return number of the active scheme
      */
     int getLaneEdgeScaleMode() const;
 
-    /** @brief Returns the current lane (edge) coloring schme
+    /** @brief Returns the current lane (edge) coloring scheme
      * @return current scheme
      */
     GUIColorScheme& getLaneEdgeScheme();
 
-    /** @brief Returns the current lane (edge) scaling schme
+    /** @brief Returns the current lane (edge) scaling scheme
      * @return current scheme
      */
     GUIScaleScheme& getLaneEdgeScaleScheme();
@@ -538,7 +689,7 @@ public:
     bool operator==(const GUIVisualizationSettings& vs2);
 
     /// @brief map from LinkState to color constants
-    static const RGBColor& getLinkColor(const LinkState& ls);
+    static const RGBColor& getLinkColor(const LinkState& ls, bool realistic = false);
 
     /// @brief return an angle that is suitable for reading text aligned with the given angle (degrees)
     double getTextAngle(double objectAngle) const;
@@ -546,18 +697,12 @@ public:
     /// @brief return wether the text was flipped for reading at the given angle
     bool flippedTextAngle(double objectAngle) const;
 
-    /// @brief check if additionals must be drawn
-    bool drawAdditionals(const double exaggeration) const;
+    /// @brief return the detail level
+    Detail getDetailLevel(const double exaggeration) const;
 
-    /// @brief check if details can be drawn for the given GUIVisualizationDetailSettings and current scale and exxageration
+    /// @brief check if details can be drawn for the given GUIVisualizationDetailSettings and current scale and exaggeration
     bool drawDetail(const double detail, const double exaggeration) const;
 
-    /// @brief function to calculate circle resolution for all circles drawn in drawGL(...) functions
-    int getCircleResolution() const;
-
-    /// @brief check if dotted contour can be drawn
-    bool drawDottedContour() const;
-
     /// @brief check if moving geometry point can be draw
     bool drawMovingGeometryPoint(const double exaggeration, const double radius) const;
 
@@ -576,6 +721,9 @@ public:
     /// @brief Information whether frames-per-second should be drawn
     bool fps;
 
+    /// @brief drawl all objects according to their z data
+    bool trueZ;
+
     /// @name Background visualization settings
     /// @{
 
@@ -589,7 +737,6 @@ public:
     double gridXSize, gridYSize;
     /// @}
 
-
     /// @name lane visualization settings
     /// @{
 
@@ -617,16 +764,19 @@ public:
     /// @brief Information whether link textures (arrows) shall be drawn
     bool showLinkDecals;
 
+    /// @brief Information whether link rules (colored bars) shall be drawn with a realistic color scheme
+    bool realisticLinkRules;
+
     /// @brief Information whether link rules (colored bars) shall be drawn
     bool showLinkRules;
 
     /// @brief Information whether rails shall be drawn
     bool showRails;
 
-    // Setting bundles for optional drawing names with size and color
-    GUIVisualizationTextSettings edgeName, internalEdgeName, cwaEdgeName, streetName, edgeValue;
+    /// @brief Setting bundles for optional drawing names with size and color
+    GUIVisualizationTextSettings edgeName, internalEdgeName, cwaEdgeName, streetName, edgeValue, edgeScaleValue;
 
-    /// @brief flag to show or hidde connectors
+    /// @brief flag to show or hide connectors
     bool hideConnectors;
 
     /// @brief The lane exaggeration (upscale thickness)
@@ -644,19 +794,27 @@ public:
     /// @brief Whether to improve visualisation of superposed (rail) edges
     bool spreadSuperposed;
 
+    /// @brief disable hide by zoom
+    bool disableHideByZoom;
+
     /// @brief key for coloring by edge parameter
     std::string edgeParam, laneParam;
     /// @brief key for coloring by vehicle parameter
     std::string vehicleParam;
+    /// @brief key for scaling by vehicle parameter
+    std::string vehicleScaleParam;
     /// @brief key for rendering vehicle textual parameter
     std::string vehicleTextParam;
 
     /// @brief key for coloring by edgeData
     std::string edgeData;
+    /// @brief id for coloring by live edgeData
+    std::string edgeDataID;
+    /// @brief key for scaling by edgeData
+    std::string edgeDataScaling;
 
-    /// @brief value below which edge data value should not be rendered
-    bool edgeValueHideCheck;
-    double edgeValueHideThreshold;
+    /// @brief checks and thresholds for rainbow coloring
+    GUIVisualizationRainbowSettings edgeValueRainBow;
     /// @}
 
     /// @name vehicle visualization settings
@@ -665,6 +823,9 @@ public:
     /// @brief The vehicle colorer
     GUIColorer vehicleColorer;
 
+    /// @brief The size scaling settings for vehicles
+    GUIScaler vehicleScaler;
+
     /// @brief The quality of vehicle drawing
     int vehicleQuality;
 
@@ -689,12 +850,22 @@ public:
     /// @brief Whether vehicle length shall be scaled with length/geometry factor
     bool scaleLength;
 
+    /// @brief Whether to draw reversed vehicles in their reversed state
+    bool drawReversed;
+
+    /// @brief Set whether parking related information should be shown
+    bool showParkingInfo;
+
+    /// @brief Set whether the charging search related information should be shown
+    bool showChargingInfo;
+
     // Setting bundles for controling the size of the drawn vehicles
     GUIVisualizationSizeSettings vehicleSize;
 
     // Setting bundles for optional drawing vehicle names or color value
-    GUIVisualizationTextSettings vehicleName, vehicleValue, vehicleText;
+    GUIVisualizationTextSettings vehicleName, vehicleValue, vehicleScaleValue, vehicleText;
 
+    GUIVisualizationRainbowSettings vehicleValueRainBow;
     /// @}
 
 
@@ -712,6 +883,12 @@ public:
 
     // Setting bundles for optional drawing person names
     GUIVisualizationTextSettings personName, personValue;
+
+    /// @brief Flag for visualizing the pedestrian network generated for JuPedSim
+    bool showPedestrianNetwork;
+
+    /// @brief The color of the pedestrian network generated for JuPedSim
+    RGBColor pedestrianNetworkColor;
     /// @}
 
 
@@ -745,10 +922,12 @@ public:
     bool showLane2Lane;
     /// @brief whether the shape of the junction should be drawn
     bool drawJunctionShape;
-    /// @brief whether crosings and walkingareas shall be drawn
+    /// @brief whether crossings and walkingareas shall be drawn
     bool drawCrossingsAndWalkingareas;
     // Setting bundles for controling the size of the drawn junction
     GUIVisualizationSizeSettings junctionSize;
+
+    GUIVisualizationRainbowSettings junctionValueRainBow;
     /// @}
 
 
@@ -756,7 +935,7 @@ public:
     /// @{
 
     /// @brief The additional structures visualization scheme
-    // @todo decouple addExageration for POIs, Polygons, Triggers etc
+    // @todo decouple addExaggeration for POIs, Polygons, Triggers etc
     int addMode;
     // Setting bundles for controling the size of additional items
     GUIVisualizationSizeSettings addSize;
@@ -776,6 +955,9 @@ public:
     // Setting bundles for controling the size of the drawn POIs
     GUIVisualizationSizeSettings poiSize;
 
+    /// @brief The detail level for drawing POIs
+    int poiDetail;
+
     // Setting bundles for optional drawing poi names
     GUIVisualizationTextSettings poiName;
 
@@ -788,6 +970,12 @@ public:
     /// @brief key for rendering poi textual parameter
     std::string poiTextParam;
 
+    /// @brief whether the rendering layer of POIs should be overriden
+    bool poiUseCustomLayer;
+
+    /// @brief the custom layer for POIs
+    double poiCustomLayer;
+
     /// @brief The polygon colorer
     GUIColorer polyColorer;
 
@@ -799,8 +987,69 @@ public:
 
     // Setting bundles for optional drawing polygon types
     GUIVisualizationTextSettings polyType;
+
+    /// @brief whether the rendering layer of polygons should be overriden
+    bool polyUseCustomLayer;
+
+    /// @brief the custom layer for polygons
+    double polyCustomLayer;
     /// @}
 
+
+    /// @name data mode visualization settings
+    /// @{
+    /// @brief the edgeRelation / tazRelation colorer
+    GUIColorer dataColorer;
+
+    /// @brief The size scaling settings for data elements
+    GUIScaler dataScaler;
+
+    GUIVisualizationTextSettings dataValue;
+
+    /// @brief The tazRelation exaggeration (upscale thickness)
+    double tazRelWidthExaggeration;
+
+    /// @brief The edgeRelation exaggeration (upscale thickness)
+    double edgeRelWidthExaggeration;
+
+    /// @brief key for coloring by edgeRelation / tazRelation attribute
+    std::string relDataAttr;
+
+    /// @brief key for scaling by edgeRelation / tazRelation attribute
+    std::string relDataScaleAttr;
+
+    /// @brief value below which edgeData and edgeRelation data value should not be rendered
+    GUIVisualizationRainbowSettings dataValueRainBow;
+    /// @}
+
+
+    /// @name 3D visualization settings
+    /// @{
+    /// @brief whether the coloring schemes of vehicles should be ignored
+    bool ignoreColorSchemeFor3DVehicles;
+
+    /// @brief whether the TLS link markers should be drawn
+    bool show3DTLSLinkMarkers;
+
+    /// @brief whether the semi-transparent domes around 3D TL models should be drawn
+    bool show3DTLSDomes;
+
+    /// @brief whether 3D TLS models should be generated automatically
+    bool generate3DTLSModels;
+
+    /// @brief whether to draw the head up display items
+    bool show3DHeadUpDisplay;
+
+    /// @brief 3D material light components
+    RGBColor ambient3DLight;
+    RGBColor diffuse3DLight;
+
+    /// @brief sky background color
+    RGBColor skyColor;
+
+    /// @}
+
+
     /// @brief Information whether the size legend shall be drawn
     bool showSizeLegend;
 
@@ -819,23 +1068,26 @@ public:
     /// @brief enable or disable draw boundaries
     bool drawBoundaries;
 
-    /// @brief the current selection scaling in NETEDIT (set in SelectorFrame)
+    /// @brief the current selection scaling in netedit (set in SelectorFrame)
     double selectorFrameScale;
 
-    /// @brief whether drawing is performed for the purpose of selecting objects with a single click
-    bool drawForPositionSelection;
+    /// @brief whether drawing is performed for the purpose of selecting objects in view using ViewObjectsHandler
+    bool drawForViewObjectsHandler;
 
     /// @brief whether drawing is performed for the purpose of selecting objects using a rectangle
     bool drawForRectangleSelection;
 
-    /// @brief flag to force draw for position selection (see drawForPositionSelection)
-    bool forceDrawForPositionSelection;
-
     /// @brief flag to force draw for rectangle selection (see drawForRectangleSelection)
     bool forceDrawForRectangleSelection;
 
-    /// @brief flag to force draw dotted contour
-    bool forceDrawDottedContour;
+    /// @brief flag for disable dotted contours in netedit
+    bool disableDottedContours;
+
+    // Setting bundles for optional drawing geometry point indices
+    GUIVisualizationTextSettings geometryIndices;
+
+    /// @brief whether secondary lane shape shall be drawn
+    bool secondaryShape;
 
     /**@brief whether drawing is performed in left-hand networks
      * @note used to avoid calls to OptionsCont::getOptions() in every drawgl(...) function, and
@@ -854,11 +1106,16 @@ public:
     static const std::string SCHEME_NAME_LANE_PARAM_NUMERICAL;
     static const std::string SCHEME_NAME_PARAM_NUMERICAL;
     static const std::string SCHEME_NAME_EDGEDATA_NUMERICAL;
+    static const std::string SCHEME_NAME_DATA_ATTRIBUTE_NUMERICAL;
     static const std::string SCHEME_NAME_SELECTION;
     static const std::string SCHEME_NAME_TYPE;
     static const std::string SCHEME_NAME_PERMISSION_CODE;
+    static const std::string SCHEME_NAME_EDGEDATA_LIVE;
 
     static const double MISSING_DATA;
+    static RGBColor COL_MISSING_DATA;
+
+    static std::map<std::string, std::vector<RGBColor> > RAINBOW_SCHEMES;
 
     /// @brief color settings
     GUIVisualizationColorSettings colorSettings;
@@ -889,4 +1146,18 @@ public:
 
     /// @brief detail settings
     GUIVisualizationDetailSettings detailSettings;
+
+    /// @brief constant for boundary size drawing (20 for slow computers, 10 for quick computers)
+    double BoundarySizeDrawing = 15;
+
+protected:
+    /// @brief flag for ignore hide by zoom (used if we're drawing elements with constant size, their ID/name/etc. texts, etc.)
+    bool myIgnoreHideByZoom;
+
+private:
+    /// @brief set copy constructor private
+    GUIVisualizationSettings(const GUIVisualizationSettings&) = default;
+
+    /// @brief set assignment operator private
+    GUIVisualizationSettings& operator=(const GUIVisualizationSettings&) = default;
 };
