@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -40,48 +40,70 @@ GUIPerspectiveChanger::~GUIPerspectiveChanger() {
 
 void
 GUIPerspectiveChanger::onLeftBtnPress(void*) {
+    // reimplement in child
 }
 
 
 bool
 GUIPerspectiveChanger::onLeftBtnRelease(void*) {
+    // reimplement in child
+    return false;
+}
+
+
+void
+GUIPerspectiveChanger::onMiddleBtnPress(void*) {
+    // reimplement in child
+}
+
+
+bool
+GUIPerspectiveChanger::onMiddleBtnRelease(void*) {
+    // reimplement in child
     return false;
 }
 
 
 void
 GUIPerspectiveChanger::onRightBtnPress(void*) {
+    // reimplement in child
 }
 
 
 bool
 GUIPerspectiveChanger::onRightBtnRelease(void*) {
+    // reimplement in child
     return false;
 }
 
 void
 GUIPerspectiveChanger::onDoubleClicked(void*) {
+    // reimplement in child
 }
 
 
 void
 GUIPerspectiveChanger::onMouseWheel(void*) {
+    // reimplement in child
 }
 
 
 void
 GUIPerspectiveChanger::onMouseMove(void*) {
+    // reimplement in child
 }
 
 
 long
 GUIPerspectiveChanger::onKeyPress(void*) {
+    // reimplement in child
     return 0;
 }
 
 
 long
 GUIPerspectiveChanger::onKeyRelease(void*) {
+    // reimplement in child
     return 0;
 }
 
