@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2004-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2004-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -26,6 +26,11 @@
 #include <sstream>
 #include <string>
 #include <iomanip>
+#ifdef WIN32
+#define NOMINMAX
+#include <windows.h>
+#undef NOMINMAX
+#endif
 #include "OutputDevice.h"
 #include "OutputDevice_File.h"
 #include "OutputDevice_COUT.h"
@@ -38,24 +43,42 @@
 #include <utils/common/ToString.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/options/OptionsCont.h>
+#include <utils/options/OptionsIO.h>
 
 
 // ===========================================================================
 // static member definitions
 // ===========================================================================
 std::map<std::string, OutputDevice*> OutputDevice::myOutputDevices;
+int OutputDevice::myPrevConsoleCP = -1;
 
 
 // ===========================================================================
 // static method definitions
 // ===========================================================================
 OutputDevice&
-OutputDevice::getDevice(const std::string& name) {
+OutputDevice::getDevice(const std::string& name, bool usePrefix) {
+#ifdef WIN32
+    // fix the windows console output on first call
+    if (myPrevConsoleCP == -1) {
+        myPrevConsoleCP = GetConsoleOutputCP();
+        SetConsoleOutputCP(CP_UTF8);
+    }
+#endif
     // check whether the device has already been aqcuired
     if (myOutputDevices.find(name) != myOutputDevices.end()) {
         return *myOutputDevices[name];
     }
     // build the device
+    const OptionsCont& oc = OptionsCont::getOptions();
+    const int len = (int)name.length();
+    bool isParquet = (oc.exists("output.format") && oc.getString("output.format") == "parquet") || (len > 8 && name.substr(len - 8) == ".parquet");
+#ifndef HAVE_PARQUET
+    if (isParquet) {
+        WRITE_WARNING("Compiled without Parquet support, falling back to XML.")
+        isParquet = false;
+    }
+#endif
     OutputDevice* dev = nullptr;
     // check whether the device shall print to stdout
     if (name == "stdout") {
@@ -64,33 +87,55 @@ OutputDevice::getDevice(const std::string& name) {
         dev = OutputDevice_CERR::getDevice();
     } else if (FileHelpers::isSocket(name)) {
         try {
-            int port = StringUtils::toInt(name.substr(name.find(":") + 1));
-            dev = new OutputDevice_Network(name.substr(0, name.find(":")), port);
+            const bool ipv6 = name[0] == '[';  // IPv6 addresses may be written like '[::1]:8000'
+            const size_t sepIndex = name.find(":", ipv6 ? name.find("]") : 0);
+            const int port = StringUtils::toInt(name.substr(sepIndex + 1));
+            dev = new OutputDevice_Network(ipv6 ? name.substr(1, sepIndex - 2) : name.substr(0, sepIndex), port);
         } catch (NumberFormatException&) {
             throw IOError("Given port number '" + name.substr(name.find(":") + 1) + "' is not numeric.");
         } catch (EmptyData&) {
-            throw IOError("No port number given.");
+            throw IOError(TL("No port number given."));
         }
     } else {
-        const int len = (int)name.length();
-        std::string name2 = name;
-        if (OptionsCont::getOptions().isSet("output-prefix") && name != "/dev/null") {
-            std::string prefix = OptionsCont::getOptions().getString("output-prefix");
+        std::string name2 = (name == "nul" || name == "NUL") ? "/dev/null" : name;
+        if (usePrefix && oc.isSet("output-prefix") && name2 != "/dev/null") {
+            std::string prefix = oc.getString("output-prefix");
             const std::string::size_type metaTimeIndex = prefix.find("TIME");
             if (metaTimeIndex != std::string::npos) {
-                time_t rawtime;
+                const time_t rawtime = std::chrono::system_clock::to_time_t(OptionsIO::getLoadTime());
                 char buffer [80];
-                time(&rawtime);
                 struct tm* timeinfo = localtime(&rawtime);
                 strftime(buffer, 80, "%Y-%m-%d-%H-%M-%S", timeinfo);
-                prefix.replace(metaTimeIndex, 4, std::string(buffer));
+                prefix.replace(metaTimeIndex, 4, buffer);
             }
             name2 = FileHelpers::prependToLastPathComponent(prefix, name);
         }
-        dev = new OutputDevice_File(name2, len > 3 && name.substr(len - 3) == ".gz");
+        if (usePrefix && oc.isSet("output-suffix") && name2 != "/dev/null") {
+            std::string suffix = oc.getString("output-suffix");
+            const std::string::size_type metaTimeIndex = suffix.find("TIME");
+            if (metaTimeIndex != std::string::npos) {
+                const time_t rawtime = std::chrono::system_clock::to_time_t(OptionsIO::getLoadTime());
+                char buffer [80];
+                struct tm* timeinfo = localtime(&rawtime);
+                strftime(buffer, 80, "%Y-%m-%d-%H-%M-%S", timeinfo);
+                suffix.replace(metaTimeIndex, 4, buffer);
+            }
+            name2 = FileHelpers::appendBeforeExtension(name2, suffix);
+        }
+        name2 = StringUtils::substituteEnvironment(name2, &OptionsIO::getLoadTime());
+        dev = new OutputDevice_File(name2, isParquet);
     }
+    if ((oc.exists("output.format") && oc.getString("output.format") == "csv") || (len > 4 && name.substr(len - 4) == ".csv") || (len > 7 && name.substr(len - 7) == ".csv.gz")) {
+        dev->setFormatter(new CSVFormatter(oc.getString("output.column-header"), oc.getString("output.column-separator")[0]));
+    }
+#ifdef HAVE_PARQUET
+    if (isParquet) {
+        dev->setFormatter(new ParquetFormatter(oc.getString("output.column-header"), oc.getString("output.compression")));
+    }
+#endif
     dev->setPrecision();
     dev->getOStream() << std::setiosflags(std::ios::fixed);
+    dev->myWriteMetadata = oc.exists("write-metadata") && oc.getBool("write-metadata");
     myOutputDevices[name] = dev;
     return *dev;
 }
@@ -115,12 +160,20 @@ OutputDevice&
 OutputDevice::getDeviceByOption(const std::string& optionName) {
     std::string devName = OptionsCont::getOptions().getString(optionName);
     if (myOutputDevices.find(devName) == myOutputDevices.end()) {
-        throw InvalidArgument("Device '" + devName + "' has not been created.");
+        throw InvalidArgument("Output device '" + devName + "' for option '" + optionName + "' has not been created.");
     }
     return OutputDevice::getDevice(devName);
 }
 
 
+void
+OutputDevice::flushAll() {
+    for (auto item : myOutputDevices) {
+        item.second->flush();
+    }
+}
+
+
 void
 OutputDevice::closeAll(bool keepErrorRetrievers) {
     std::vector<OutputDevice*> errorDevices;
@@ -132,52 +185,37 @@ OutputDevice::closeAll(bool keepErrorRetrievers) {
             nonErrorDevices.push_back(i->second);
         }
     }
-    for (std::vector<OutputDevice*>::iterator i = nonErrorDevices.begin(); i != nonErrorDevices.end(); ++i) {
+    for (OutputDevice* const dev : nonErrorDevices) {
         try {
-            //std::cout << "  close '" << (*i)->getFilename() << "'\n";
-            (*i)->close();
+            dev->close();
         } catch (const IOError& e) {
-            WRITE_ERROR("Error on closing output devices.");
+            WRITE_ERROR(TL("Error on closing output devices."));
             WRITE_ERROR(e.what());
         }
     }
     if (!keepErrorRetrievers) {
-        for (std::vector<OutputDevice*>::iterator i = errorDevices.begin(); i != errorDevices.end(); ++i) {
+        for (OutputDevice* const dev : errorDevices) {
             try {
-                (*i)->close();
+                dev->close();
             } catch (const IOError& e) {
                 std::cerr << "Error on closing error output devices." << std::endl;
                 std::cerr << e.what() << std::endl;
             }
         }
+#ifdef WIN32
+        if (myPrevConsoleCP != -1) {
+            SetConsoleOutputCP(myPrevConsoleCP);
+        }
+#endif
     }
 }
 
 
-std::string
-OutputDevice::realString(const double v, const int precision) {
-    std::ostringstream oss;
-    if (v == 0) {
-        return "0";
-    }
-    if (v < pow(10., -precision)) {
-        oss.setf(std::ios::scientific, std::ios::floatfield);
-    } else {
-        oss.setf(std::ios::fixed, std::ios::floatfield);     // use decimal format
-        oss.setf(std::ios::showpoint);    // print decimal point
-        oss << std::setprecision(precision);
-    }
-    oss << v;
-    return oss.str();
-}
-
-
 // ===========================================================================
 // member method definitions
 // ===========================================================================
 OutputDevice::OutputDevice(const int defaultIndentation, const std::string& filename) :
-    myFilename(filename) {
-    myFormatter = new PlainXMLFormatter(defaultIndentation);
+    myFilename(filename), myFormatter(new PlainXMLFormatter(defaultIndentation)) {
 }
 
 
@@ -220,12 +258,13 @@ OutputDevice::setPrecision(int precision) {
 bool
 OutputDevice::writeXMLHeader(const std::string& rootElement,
                              const std::string& schemaFile,
-                             std::map<SumoXMLAttr, std::string> attrs) {
+                             std::map<SumoXMLAttr, std::string> attrs,
+                             bool includeConfig) {
     if (schemaFile != "") {
         attrs[SUMO_ATTR_XMLNS] = "http://www.w3.org/2001/XMLSchema-instance";
         attrs[SUMO_ATTR_SCHEMA_LOCATION] = "http://sumo.dlr.de/xsd/" + schemaFile;
     }
-    return myFormatter->writeXMLHeader(getOStream(), rootElement, attrs);
+    return myFormatter->writeXMLHeader(getOStream(), rootElement, attrs, myWriteMetadata, includeConfig);
 }
 
 
@@ -258,9 +297,9 @@ OutputDevice::postWriteHook() {}
 
 
 void
-OutputDevice::inform(const std::string& msg, const char progress) {
-    if (progress != 0) {
-        getOStream() << msg << progress;
+OutputDevice::inform(const std::string& msg, const bool progress) {
+    if (progress) {
+        getOStream() << msg;
     } else {
         getOStream() << msg << '\n';
     }
@@ -268,4 +307,29 @@ OutputDevice::inform(const std::string& msg, const char progress) {
 }
 
 
+const SumoXMLAttrMask
+OutputDevice::parseWrittenAttributes(const std::vector<std::string>& attrList, const std::string& desc, const std::map<std::string, SumoXMLAttrMask>& special) {
+    SumoXMLAttrMask result;
+    for (std::string attrName : attrList) {
+        if (attrName == "all") {
+            result.set();
+        } else if (special.count(attrName) > 0) {
+            result |= special.find(attrName)->second;
+        } else {
+            if (SUMOXMLDefinitions::Attrs.hasString(attrName)) {
+                int attrNr = SUMOXMLDefinitions::Attrs.get(attrName);
+                if (attrNr < (int)result.size()) {
+                    result.set(attrNr);
+                } else {
+                    WRITE_ERRORF(TL("Attribute '%' is not support for filtering written attributes in %."), attrName, desc);
+                }
+            } else {
+                WRITE_ERRORF(TL("Unknown attribute '%' to write in %."), attrName, desc);
+            }
+        }
+    }
+    return result;
+}
+
+
 /****************************************************************************/
