@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2012-2021 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2012-2026 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -26,9 +26,12 @@
 #include <foreign/zstr/zstr.hpp>
 #endif
 #ifdef HAVE_FOX
-#include <utils/foxtools/FXWorkerThread.h>
+#include <utils/foxtools/MFXWorkerThread.h>
 #endif
+#include <utils/common/MapMatcher.h>
+#include <utils/geom/PositionVector.h>
 #include <utils/router/ReversedEdge.h>
+#include <utils/router/SUMOAbstractRouter.h>
 
 #define UNREACHABLE (std::numeric_limits<double>::max() / 1000.0)
 
@@ -58,6 +61,8 @@
 template<class E, class V>
 class AbstractLookupTable {
 public:
+    virtual ~AbstractLookupTable() {}
+
     /// @brief provide a lower bound on the distance between from and to (excluding traveltime of both edges)
     virtual double lowerBound(const E* from, const E* to, double speed, double speedFactor, double fromEffort, double toEffort) const = 0;
 
@@ -97,19 +102,23 @@ private:
 };
 
 
-template<class E, class V>
+template<class E, class V, class M>
 class LandmarkLookupTable : public AbstractLookupTable<E, V> {
 public:
     LandmarkLookupTable(const std::string& filename, const std::vector<E*>& edges, SUMOAbstractRouter<E, V>* router,
                         SUMOAbstractRouter<ReversedEdge<E, V>, V>* reverseRouter,
-                        const V* defaultVehicle, const std::string& outfile, const int maxNumThreads) {
+                        const V* defaultVehicle, const std::string& outfile, const int maxNumThreads, M* mapMatcher) {
         myFirstNonInternal = -1;
         std::map<std::string, int> numericID;
+        bool haveTaz = false;
         for (E* e : edges) {
             if (!e->isInternal()) {
                 if (myFirstNonInternal == -1) {
                     myFirstNonInternal = e->getNumericalID();
                 }
+                if (e->isTazConnector()) {
+                    haveTaz = true;
+                }
                 numericID[e->getID()] = e->getNumericalID() - myFirstNonInternal;
             }
         }
@@ -119,37 +128,78 @@ public:
         std::ifstream strm(filename.c_str());
 #endif
         if (!strm.good()) {
-            throw ProcessError("Could not load landmark-lookup-table from '" + filename + "'.");
+            throw ProcessError(TLF("Could not load landmark-lookup-table from '%'.", filename));
         }
         std::string line;
+        std::vector<const E*> landmarks;
         int numLandMarks = 0;
+        bool haveData = false;
         while (std::getline(strm, line)) {
             if (line == "") {
                 break;
             }
-            //std::cout << "'" << line << "'" << "\n";
             StringTokenizer st(line);
             if (st.size() == 1) {
                 const std::string lm = st.get(0);
+                if (myLandmarks.count(lm) != 0) {
+                    throw ProcessError(TLF("Duplicate edge '%' in landmark file.", lm));
+                }
+                // retrieve landmark edge
+                const auto& it = numericID.find(lm);
+                if (it == numericID.end()) {
+                    throw ProcessError(TLF("Landmark edge '%' does not exist in the network.", lm));
+                }
                 myLandmarks[lm] = numLandMarks++;
                 myFromLandmarkDists.push_back(std::vector<double>(0));
                 myToLandmarkDists.push_back(std::vector<double>(0));
+                landmarks.push_back(edges[it->second + myFirstNonInternal]);
+            } else if (st.size() == 2) {
+                // geo landmark
+                try {
+                    std::string lonStr = st.get(0);
+                    if (!lonStr.empty() && lonStr.back() == ',') {
+                        // remove trailing comma
+                        lonStr = lonStr.substr(0, lonStr.size() - 1);
+                    }
+                    double lon = StringUtils::toDouble(lonStr);
+                    double lat = StringUtils::toDouble(st.get(1));
+                    std::vector<const E*> mapped;
+                    bool ok;
+                    mapMatcher->parseGeoEdges(PositionVector({Position(lon, lat)}), true, SVC_IGNORING, mapped, "LMLT", false, ok, true);
+                    if (mapped.size() != 1) {
+                        throw ProcessError(TLF("Invalid coordinate in landmark file, could not find edge at  '%'", line));
+                    }
+                    std::string lm = mapped.front()->getID();
+                    const auto& it = numericID.find(lm);
+                    if (it == numericID.end()) {
+                        throw ProcessError(TLF("Landmark edge '%' does not exist in the network.", lm));
+                    }
+                    myLandmarks[lm] = numLandMarks++;
+                    myFromLandmarkDists.push_back(std::vector<double>(0));
+                    myToLandmarkDists.push_back(std::vector<double>(0));
+                    landmarks.push_back(edges[it->second + myFirstNonInternal]);
+                } catch (const NumberFormatException&) {
+                    throw ProcessError(TLF("Broken landmark file, could not parse '%' as coordinates.", line));
+                }
             } else if (st.size() == 4) {
                 // legacy style landmark table
                 const std::string lm = st.get(0);
                 const std::string edge = st.get(1);
                 if (numericID[edge] != (int)myFromLandmarkDists[myLandmarks[lm]].size()) {
-                    WRITE_WARNING("Unknown or unordered edge '" + edge + "' in landmark file.");
+                    throw ProcessError(TLF("Unknown or unordered edge '%' in landmark file.", edge));
                 }
                 const double distFrom = StringUtils::toDouble(st.get(2));
                 const double distTo = StringUtils::toDouble(st.get(3));
                 myFromLandmarkDists[myLandmarks[lm]].push_back(distFrom);
                 myToLandmarkDists[myLandmarks[lm]].push_back(distTo);
+                haveData = true;
             } else {
-                assert((int)st.size() == 2 * numLandMarks + 1);
                 const std::string edge = st.get(0);
+                if ((int)st.size() != 2 * numLandMarks + 1) {
+                    throw ProcessError(TLF("Broken landmark file, unexpected number of entries (%) for edge '%'.", st.size() - 1, edge));
+                }
                 if (numericID[edge] != (int)myFromLandmarkDists[0].size()) {
-                    WRITE_WARNING("Unknown or unordered edge '" + edge + "' in landmark file.");
+                    throw ProcessError(TLF("Unknown or unordered edge '%' in landmark file.", edge));
                 }
                 for (int i = 0; i < numLandMarks; i++) {
                     const double distFrom = StringUtils::toDouble(st.get(2 * i + 1));
@@ -157,40 +207,29 @@ public:
                     myFromLandmarkDists[i].push_back(distFrom);
                     myToLandmarkDists[i].push_back(distTo);
                 }
+                haveData = true;
             }
         }
         if (myLandmarks.empty()) {
-            WRITE_WARNING("No landmarks in '" + filename + "', falling back to standard A*.");
+            WRITE_WARNINGF("No landmarks in '%', falling back to standard A*.", filename);
             return;
         }
 #ifdef HAVE_FOX
-        FXWorkerThread::Pool threadPool;
+        MFXWorkerThread::Pool threadPool;
         std::vector<RoutingTask*> currentTasks;
 #endif
-        std::vector<const E*> landmarks;
+        if (!haveData) {
+            WRITE_MESSAGE(TL("Calculating new lookup table."));
+        }
         for (int i = 0; i < numLandMarks; ++i) {
             if ((int)myFromLandmarkDists[i].size() != (int)edges.size() - myFirstNonInternal) {
-                const std::string landmarkID = getLandmark(i);
-                const E* landmark = nullptr;
-                // retrieve landmark edge
-                for (const E* const edge : edges) {
-                    if (edge->getID() == landmarkID) {
-                        landmark = edge;
-                        landmarks.push_back(edge);
-                        break;
-                    }
-                }
-                if (landmark == nullptr) {
-                    WRITE_WARNING("Landmark edge '" + landmarkID + "' does not exist in the network.");
-                    continue;
-                }
-                if (!outfile.empty()) {
-                    WRITE_WARNING("Not all network edges were found in the lookup table '" + filename + "' for landmark edge '" + landmarkID + "'. Saving new matrix to '" + outfile + "'.");
-                } else {
+                const E* const landmark = landmarks[i];
+                if (haveData) {
                     if (myFromLandmarkDists[i].empty()) {
-                        WRITE_WARNING("No lookup table for landmark edge '" + landmarkID + "', recalculating.");
+                        WRITE_WARNINGF(TL("No lookup table for landmark edge '%', recalculating."), landmark->getID());
                     } else {
-                        throw ProcessError("Not all network edges were found in the lookup table '" + filename + "' for landmark edge '" + landmarkID + "'.");
+                        const std::string tazWarning = haveTaz ? " Make sure that any used taz or junction-taz definitions are loaded when generating the table" : "";
+                        throw ProcessError(TLF("Not all network edges were found in the lookup table '%' for landmark edge '%'.%", filename, landmark->getID(), tazWarning));
                     }
                 }
 #ifdef HAVE_FOX
@@ -202,7 +241,7 @@ public:
                             // The CHRouter needs initialization
                             // before it gets cloned, so we do a dummy routing which is not in parallel
                             std::vector<const E*> route;
-                            router->compute(landmark, landmark, defaultVehicle, 0, route);
+                            router->compute(landmark, landmark, defaultVehicle, 0, route, true);
                         } else {
                             reverseRouter->setAutoBulkMode(true);
                         }
@@ -220,6 +259,8 @@ public:
                         }
                     }
                 }
+#else
+                UNUSED_PARAMETER(reverseRouter);
 #endif
             }
         }
@@ -231,8 +272,10 @@ public:
             if ((int)myFromLandmarkDists[i].size() != (int)edges.size() - myFirstNonInternal) {
                 const E* landmark = landmarks[i];
                 const double lmCost = router->recomputeCosts({landmark}, defaultVehicle, 0);
+                int unreachableFrom = 0;
+                int unreachableTo = 0;
                 for (int j = (int)myFromLandmarkDists[i].size() + myFirstNonInternal; j < (int)edges.size(); ++j) {
-                    const E* edge = edges[j];
+                    const E* const edge = edges[j];
                     double distFrom = -1;
                     double distTo = -1;
                     if (landmark == edge) {
@@ -251,14 +294,14 @@ public:
                             std::vector<const ReversedEdge<E, V>*> reversedRoute;
                             // compute from-distance (skip taz-sources and other unreachable edges)
                             if (edge->getPredecessors().size() > 0 && landmark->getSuccessors().size() > 0) {
-                                if (router->compute(landmark, edge, defaultVehicle, 0, route)) {
+                                if (router->compute(landmark, edge, defaultVehicle, 0, route, true)) {
                                     distFrom = MAX2(0.0, router->recomputeCosts(route, defaultVehicle, 0) - sourceDestCost);
                                     route.clear();
                                 }
                             }
                             // compute to-distance (skip unreachable landmarks)
                             if (landmark->getPredecessors().size() > 0 && edge->getSuccessors().size() > 0) {
-                                if (router->compute(edge, landmark, defaultVehicle, 0, route)) {
+                                if (router->compute(edge, landmark, defaultVehicle, 0, route, true)) {
                                     distTo = MAX2(0.0, router->recomputeCosts(route, defaultVehicle, 0) - sourceDestCost);
                                     route.clear();
                                 }
@@ -267,6 +310,18 @@ public:
                     }
                     myFromLandmarkDists[i].push_back(distFrom);
                     myToLandmarkDists[i].push_back(distTo);
+                    if (!edge->isTazConnector()) {
+                        if (distFrom == -1) {
+                            unreachableFrom++;
+                        }
+                        if (distTo == -1) {
+                            unreachableTo++;
+                        }
+                    }
+                }
+                if (unreachableFrom > 0 || unreachableTo > 0) {
+                    WRITE_WARNINGF(TL("Landmark % is not reachable from % edges and is unable to reach % out of % total edges."),
+                                   landmark->getID(), unreachableFrom, unreachableTo, numericID.size());
                 }
             }
         }
@@ -283,8 +338,9 @@ public:
 #endif
             if (!ostrm->good()) {
                 delete ostrm;
-                throw ProcessError("Could not open file '" + outfile + "' for writing.");
+                throw ProcessError(TLF("Could not open file '%' for writing.", outfile));
             }
+            WRITE_MESSAGEF(TL("Saving new matrix to '%'."), outfile);
             for (int i = 0; i < numLandMarks; ++i) {
                 (*ostrm) << getLandmark(i) << "\n";
             }
@@ -362,31 +418,32 @@ private:
 
 #ifdef HAVE_FOX
 private:
-    class WorkerThread : public FXWorkerThread {
+    class WorkerThread : public MFXWorkerThread {
     public:
-        WorkerThread(FXWorkerThread::Pool& pool,
+        WorkerThread(MFXWorkerThread::Pool& pool,
                      SUMOAbstractRouter<E, V>* router,
                      SUMOAbstractRouter<ReversedEdge<E, V>, V>* reverseRouter, const V* vehicle)
-            : FXWorkerThread(pool), myRouter(router), myReversedRouter(reverseRouter), myVehicle(vehicle) {}
+            : MFXWorkerThread(pool), myRouter(router), myReversedRouter(reverseRouter), myVehicle(vehicle) {}
 
         virtual ~WorkerThread() {
             delete myRouter;
+            delete myReversedRouter;
         }
 
         const std::pair<double, double> compute(const E* src, const E* dest, const double costOff) {
             double fromResult = -1.;
-            if (myRouter->compute(src, dest, myVehicle, 0, myRoute)) {
+            if (myRouter->compute(src, dest, myVehicle, 0, myRoute, true)) {
                 fromResult = MAX2(0.0, myRouter->recomputeCosts(myRoute, myVehicle, 0) + costOff);
                 myRoute.clear();
             }
             double toResult = -1.;
             if (myReversedRouter != nullptr) {
-                if (myReversedRouter->compute(src->getReversedRoutingEdge(), dest->getReversedRoutingEdge(), myVehicle, 0, myReversedRoute)) {
+                if (myReversedRouter->compute(src->getReversedRoutingEdge(), dest->getReversedRoutingEdge(), myVehicle, 0, myReversedRoute, true)) {
                     toResult = MAX2(0.0, myReversedRouter->recomputeCosts(myReversedRoute, myVehicle, 0) + costOff);
                     myReversedRoute.clear();
                 }
             } else {
-                if (myRouter->compute(dest, src, myVehicle, 0, myRoute)) {
+                if (myRouter->compute(dest, src, myVehicle, 0, myRoute, true)) {
                     toResult = MAX2(0.0, myRouter->recomputeCosts(myRoute, myVehicle, 0) + costOff);
                     myRoute.clear();
                 }
@@ -402,11 +459,11 @@ private:
         std::vector<const ReversedEdge<E, V>*> myReversedRoute;
     };
 
-    class RoutingTask : public FXWorkerThread::Task {
+    class RoutingTask : public MFXWorkerThread::Task {
     public:
         RoutingTask(const E* src, const E* dest, const double costOff)
             : mySrc(src), myDest(dest), myCostOff(-costOff) {}
-        void run(FXWorkerThread* context) {
+        void run(MFXWorkerThread* context) {
             myCost = ((WorkerThread*)context)->compute(mySrc, myDest, myCostOff);
         }
         double getFromCost() {
